'''
Created on 29/set/2011

@author: mario
'''

import sqlite3
import re
import progInfo
from BrickException import BrickException


def getDBVersion(dbfile):
    #------------------------------------- gets db version from properties table
    try:
        conn = sqlite3.connect(dbfile)
        v = conn.execute('select value from buproperties where name=\'dbver\'').fetchall()
    except sqlite3.OperationalError:
        return 0
    conn.close()
    if len(v) == 0:
        return 0
    return int(v[0][0])


class CatalogDB():
    '''
    attempt to aggregate all database functions in one class
    '''

    '''
    checks tables needed in catalog and create if needed
    '''
    def checkTables(self):
        self.connection.execute('create table if not exists setcatalog (setid text)')
        self.connection.execute('create table if not exists setbricks (setid integer, brickid integer, num integer)')
        self.connection.execute('create table if not exists catalog (lddid integer, lddcolorid integer, '+
                  'decorated bool, descr text, qty integer)')
        self.connection.execute('create table if not exists blcatlist (catid integer, catname text)')
        self.connection.execute('create table if not exists blsetlist (setid text, setname text, setcategory integer, setyear text)')
        self.connection.execute('create table if not exists bricklist (type text, blid text, qty integer, blcolorid integer,'+
                  'extra bool, counterpart bool, descr text,' + 
                  'lddid integer, lddrgb text, lddcolorname text, blrgb text,'+
                  'blcolorname text, blcategory integer, partid integer, lddcolorid integer)')
        self.connection.execute('create table if not exists colormap (pname text, blname text, blid integer, ldrawid integer,'+
                  'lddid integer, lddname text,rgb text,notes text,lmod bool, lastmod integer)')
        self.connection.execute('create table if not exists partmap (lddid integer, blid text,bltoldd bool,lddtobl bool,notes text, lmod bool, lastmod integer)')
        self.connection.execute('create table if not exists blcolorlist (colorid integer, name text, rgb text, type text, fromy integer, toy integer)')
        self.connection.execute('create table if not exists blpartlist (itemid text, itemname text, category integer,itemx text,itemy text,itemz text)')
        self.connection.execute('create table if not exists lddcomposite (lddid integer, colorsubid integer,'+
                                'decorsubid integer,coloridx integer,decoridx integer,notes text)')
        self.connection.execute('create table if not exists ldrawmap (lddid integer, ldrawid text,dattoldd bool,lddtodat bool,notes text, lmod bool, lastmod integer)')
        #------------------------------------------ tables for full text queries
        try:
            self.connection.execute('create virtual table blpartsfts using fts3(itemid text, itemdescr text)')
        except sqlite3.OperationalError:
            pass
        try:
            self.connection.execute('create virtual table blsetsfts using fts3(setid text, setdescr text)')
        except sqlite3.OperationalError:
            pass
        self.connection.commit()

    #--------------------------- checks db version and apply upgrades, if needed
    def checkDBversion(self):
        #-------------------------------- update partmap db to version 0.0.11.12
        try:
            self.connection.execute('select lmod from partmap limit 1')
        except sqlite3.OperationalError:
            #------------------------------------------- db version is 0.0.11.11
            self.connection.execute('alter table partmap add column lmod bool')
            self.connection.execute('alter table partmap add column lastmod integer')
            self.connection.execute('update partmap set lmod=0,lastmod=strftime(\'%s\',\'now\')')
            self.connection.commit()
            
        #------------------------------- update colormap db to version 0.0.11.12
        try:
            self.connection.execute('select lmod from colormap limit 1')
        except sqlite3.OperationalError:
            #------------------------------------------- db version is 0.0.11.11
            self.connection.execute('alter table colormap add column lmod bool')
            self.connection.execute('alter table colormap add column lastmod integer')
            self.connection.execute('update colormap set lmod=0,lastmod=strftime(\'%s\',\'now\')')
            self.connection.commit()
        #----------------------------------------- add properties table -> version 0.0.12.2
        try:
            c = self.connection.execute('select * from buproperties where name=\'dbversion\'').fetchall()
            if len(c) > 0:
                self.connection.execute('delete from buproperties where name=\'dbversion\'')
                self.connection.execute('insert into buproperties(name,value) values (?,?)',('dbver',str(progInfo.DBVersion)))
                self.connection.execute('insert into buproperties(name,value) values (?,?)',('milestone',str(progInfo.SWMilestone)))
                self.connection.execute('insert into buproperties(name,value) values (?,?)',('major',str(progInfo.SWMajor)))
                self.connection.execute('insert into buproperties(name,value) values (?,?)',('minor',str(progInfo.SWMinor)))
                self.connection.execute('insert into buproperties(name,value) values (?,?)',('minor',str(progInfo.SWPatchLevel)))
                self.connection.commit()
        except sqlite3.OperationalError:
            #------------------------------------------- db version < 15 (new numeration)
            self.connection.execute('drop table if exists buproperties')
            self.connection.execute('create table buproperties (name text, value text)')
            self.connection.execute('insert into buproperties(name,value) values (?,?)',('dbver',str(progInfo.DBVersion)))
            self.connection.execute('insert into buproperties(name,value) values (?,?)',('milestone',str(progInfo.SWMilestone)))
            self.connection.execute('insert into buproperties(name,value) values (?,?)',('major',str(progInfo.SWMajor)))
            self.connection.execute('insert into buproperties(name,value) values (?,?)',('minor',str(progInfo.SWMinor)))
            self.connection.execute('insert into buproperties(name,value) values (?,?)',('minor',str(progInfo.SWPatchLevel)))
            self.connection.commit()


    def getProperty(self,name):
        try:
            v = self.connection.execute('select value from buproperties where name=?',(name,)).fetchall()
        except sqlite3.OperationalError:
            return ''
        return v[0][0]
            
    #===========================================================================
    # LDraw parts and color mapping functions/tables
    #===========================================================================


    
    def cleanLDrawmapTable(self):
        self.connection.execute('drop table if exists ldrawmap')
        self.connection.execute('create table ldrawmap (lddid integer, ldrawid text,dattoldd bool,lddtodat bool,notes text, lmod bool, lastmod integer)')

    def getLDrawParts(self,filterQuery=None):
        #print filterQuery
        if filterQuery == None or filterQuery == '':
            return self.connection.execute('select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod,rowid from ldrawmap')
        else:
            return self.connection.execute('select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod,rowid from ldrawmap where '+filterQuery)
    
              
    def updateLDrawColor(self,lddid,ldrawid):
        r = self.connection.execute('select lddid from colormap where lddid=?',(lddid,)).fetchall()
        if len(r) == 0:
            #------------------------------- no LDD color with this id, rejected
            return False
        self.connection.execute('update colormap set ldrawid=? where lddid=?',(ldrawid,lddid))
        return True

    def insertLDrawPartMap(self,lddid,ldrawid,dat2ldd=True,ldd2dat=True,notes=None,lmod=None,lastmod=None):
        if not lmod:
            self.connection.execute('insert into ldrawmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) '+
                                    'values(?,?,?,?,?,?,?)',(lddid,ldrawid,dat2ldd,ldd2dat,notes,lmod,lastmod))
        else:
            self.connection.execute('insert into ldrawmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) '+
                                    'values(?,?,?,?,?,?,strftime(\'%s\',\'now\'))',(lddid,ldrawid,dat2ldd,ldd2dat,notes,lmod))
        r = self.connection.execute('select last_insert_rowid()').fetchone()
        return r[0]


    def updateLDrawMap(self,lddid,ldrawid,dattoldd,lddtodat,notes,rowid):
        self.connection.execute('update ldrawmap set lddid=?,ldrawid=?,dattoldd=?,lddtodat=?,notes=?,lmod=?,lastmod=strftime(\'%s\',\'now\') where rowid=?',
                                (lddid, ldrawid, dattoldd, lddtodat, notes,True,rowid))
        
    def deleteLDrawMap(self,rowid):
        self.connection.execute('delete from ldrawmap where rowid=?',(rowid,))


    def ldrMapDuplicated(self,lddid,ldrid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where lddid = ? and ldrawid = ? and rowid != ?',
                                           (lddid,ldrid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where lddid = ? and ldrawid = ?',
                                           (lddid,ldrid)).fetchall()
                                               
                
    def ldrMapExistsL2D(self,lddid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where lddid = ? and lddtodat and rowid != ?',
                                        (lddid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where lddid = ? and lddtodat',
                                        (lddid,)).fetchall()
            

    def ldrMapExistsD2L(self,ldrawid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where ldrawid = ? and dattoldd and rowid != ?',
                                        (ldrawid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,ldrawid,rowid from ldrawmap where ldrawid = ? and dattoldd',
                                        (ldrawid,)).fetchall()

    
    def partLdraw2Ldd(self,ldrid):
        return self.connection.execute('select lddid from ldrawmap where ldrawid=? and dattoldd',
                                       (ldrid,)).fetchall()
    def partBL2Ldraw(self,blid):
        r = self.partBl2Ldd(blid)
        if len(r) == 1:
            return self.partLdd2Ldraw(r[0][0])
        else:
            return r
        
    def getLDrawPart(self,ldrid):
#        return self.connection.execute('select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod from ldrawmap where ldrawid=?',
#                                       (ldrid,)).fetchall()
        return self.connection.execute('select ldrawid,descr from ldrpartlist where ldrawid=?',(ldrid,)).fetchall()
                                       
    def partLdd2Ldraw(self,lddid):
        return self.connection.execute('select ldrawid from ldrawmap where lddid=? and lddtodat',
                                       (lddid,)).fetchall()
    
    def colorLdraw2Ldd(self,ldrcolor):
        return self.connection.execute('select lddid,blid,rgb,lddname from colormap where ldrawid=?',
                                       (ldrcolor,)).fetchall()

    #------------------------------------------ LDraw parts table and FTS tables
    def cleanLdrPartsTable(self):
        # create table with LDraw parts list
        self.connection.execute('drop table if exists ldrpartlist')
        self.connection.execute('create table ldrpartlist (ldrawid text, descr text, category text, keywords text)')
        self.connection.execute('drop table if exists ldrpartsfts')
        self.connection.execute('create virtual table ldrpartsfts using fts3(ldrawid text, itemdescr text)')
    
    def insertLdrPart(self,ldrid,descr,category,kwords):
        self.connection.execute('insert into ldrpartlist values(?,?,?,?)',(ldrid,descr,category,kwords))
        try:
            self.connection.execute('insert into ldrpartsfts values(?,?)',
                                    (ldrid,descr+' '+category+' '+kwords))
        except sqlite3.OperationalError:
            pass

    def ftsLDRPart(self,text):
        return self.connection.execute('select ldrawid,descr,category from ldrpartlist '+
                                       'where ldrawid in (select ldrawid from ldrpartsfts where itemdescr match ?)',(text,)).fetchall()

    def getLDRPartByID(self,ldr):
        return self.connection.execute('select ldrawid,descr,category from ldrpartlist '+
                                       'where ldrawid like ?',(ldr,)).fetchall()


    #===========================================================================
    # Bricklink categories DB
    #===========================================================================
    
    def cleanBLCategoriesTable(self):
        self.connection.execute('drop table if exists blcatlist')
        self.connection.execute('create table blcatlist (catid integer, catname text)')
        
    def insertBLCategory(self,catId,catName):
        self.connection.execute('insert into blcatlist values(?,?)',(catId,catName))
        
    def getBLCategoryByID(self,catid):
        catname = self.connection.execute('select catname from blcatlist where catid=?',(catid,)).fetchall()[0][0]
        if not isinstance(catname,str):
            return '-'
        else:
            return catname
        
    #===========================================================================
    # BrickLink set database
    #===========================================================================
    
    def cleanBLSetTable(self):
        self.connection.execute('drop table if exists blsetlist')
        self.connection.execute('create table blsetlist (setid text, setname text, setcategory integer, setyear text)')
        self.connection.execute('drop table if exists blsetsfts')
        self.connection.execute('create virtual table blsetsfts using fts3(setid text, setdescr text)')
            
    def insertBLSet(self,setId,setName,setCategory,setYear):
        self.connection.execute('insert into blsetlist values(?,?,?,?)',(setId,setName,setCategory,setYear))
        try:
            self.connection.execute('insert into blsetsfts values(?,? || \' \' || (select catname from blcatlist where catid=?))',(setId,setName,setCategory))
        except sqlite3.OperationalError:
            pass
        
    def getBLSetByID(self,setid):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname '+
                                       'from blsetlist join blcatlist '+
                                       'on (setcategory=catid) where setid like ?',(setid,))
        
    def getBLSetByIDYear(self,setid,year,op):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from blsetlist join blcatlist '+
                          'on (setcategory=catid) where setid like ? and setyear ' + op + ' ?',(setid,year))
        
    def getBLSetByYear(self,year,op):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from blsetlist join blcatlist '+
                      'on (setcategory=catid) where setyear ' + op + ' ?',(year,))
        
    def searchBLSet(self,text):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from blsetlist join blcatlist '+
                          'on (setcategory=catid) where (setname like ? or catname like ? or '+
                          'setid like ?)',(text,text,text))
        
    def searchBLSetByYear(self,text,year,op):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from blsetlist join blcatlist '+
                          'on (setcategory=catid) where (setname like ? or catname like ? or '+
                          'setid like ?) and setyear ' + op + ' ?',(text,text,text,year))
        
    def ftsBLSet(self,text):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from (blsetlist join blcatlist '+
                          'on (setcategory=catid)) where setid in (select setid from blsetsfts where setdescr match ?)',(text,))
        
    def ftsYearBLSet(self,text,year,op):
        return self.connection.execute('select blsetlist.rowid,setid,setname,setyear,catname from (blsetlist join blcatlist '+
                          'on (setcategory=catid)) where setid in (select setid from blsetsfts where setdescr match ?) and setyear ' + 
                          op + ' ? ',(text,year))

    
    
    #===========================================================================
    # Bricklink part database and ldd mapping
    #===========================================================================

    def cleanBLPartsTable(self):
        # create table with BrickLink parts list
        self.connection.execute('drop table if exists blpartlist')
        self.connection.execute('create table blpartlist (itemid text, itemname text, category integer,itemx text,itemy text,itemz text)')
        self.connection.execute('drop table if exists blpartsfts')
        self.connection.execute('create virtual table blpartsfts using fts3(itemid text, itemdescr text)')
        
    def cleanPartMappingTable(self):
        self.connection.execute('drop table if exists partmap')
        self.connection.execute('create table partmap (lddid integer, blid text,bltoldd bool,lddtobl bool,notes text, lmod bool, lastmod integer)')
    
    def insertBLPart(self,itemId,itemName,category,x,y,z):
        self.connection.execute('insert into blpartlist values(?,?,?,?,?,?)',(itemId,itemName,category,x,y,z))
        self.connection.execute('insert into blpartsfts values(?,? || \' \' || (select catname from blcatlist where catid=?))',(itemId,itemName,category))
        
    def insertPartMapping(self,lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod=0):
        if not lmod:
            #------------------------------------------- if not locally modified
            self.connection.execute('insert into partmap(lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) values(?,?,?,?,?,?,?)',
                                (lddid, blid, bltoldd, lddtobl, notes,lmod,lastmod))
        else:
            self.connection.execute('insert into partmap(lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) values(?,?,?,?,?,?,strftime(\'%s\',\'now\'))',
                                (lddid, blid, bltoldd, lddtobl, notes,lmod))

        r = self.connection.execute('select last_insert_rowid()').fetchone()
        return r[0]
 
    def updatePartMap(self,lddid,blid,bltoldd,lddtobl,notes,rowid):
        self.connection.execute('update partmap set lddid=?,blid=?,bltoldd=?,lddtobl=?,notes=?,lmod=?,lastmod=strftime(\'%s\',\'now\') where rowid=?',
                                (lddid, blid, bltoldd, lddtobl, notes,True,rowid))
        
    def deletePartMap(self,rowid):
        self.connection.execute('delete from partmap where rowid=?',(rowid,))
 
    ''' 
    maps part "blid" to equivalent "lddid" part
    '''
    def partBl2Ldd(self,blid):
        return self.connection.execute('select lddid from partmap where blid = ? and bltoldd', (blid,)).fetchall()
    
    ''' 
    maps part LDDID to equivalent BLID part
    ''' 
    def partLdd2Bl(self,lddid):
        return self.connection.execute('select blid from partmap where lddid = ? and lddtobl', (lddid,)).fetchall()

    
#    def getAllParts(self):
#        return self.connection.execute('select lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod from partmap')
    
    def getFilteredParts(self,filterQuery=None):
        #print filterQuery
        if filterQuery == None or filterQuery == '':
            return self.connection.execute('select lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod,rowid from partmap')
        else:
            return self.connection.execute('select lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod,rowid from partmap where '+filterQuery)


    '''
    return true if exists a mapping in DB
    '''
    def isPartMap(self,lddid,blid):
        return (len(self.connection.execute('select lddid,blid from partmap where blid=? and lddid=?',
                                       (blid,lddid)).fetchall()) == 1)
    
    def partMapDuplicated(self,lddid,blid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,blid,rowid from partmap where lddid = ? and blid = ? and rowid != ?',
                                           (lddid,blid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,blid,rowid from partmap where lddid = ? and blid = ?',
                                           (lddid,blid)).fetchall()
                                               
                
    def partMapExistsL2B(self,lddid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,blid,rowid from partmap where lddid = ? and lddtobl and rowid != ?',
                                        (lddid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,blid,rowid from partmap where lddid = ? and lddtobl',
                                        (lddid,)).fetchall()
            

    def partMapExistsB2L(self,blid,rowid=None):
        if rowid!=None:
            return self.connection.execute('select lddid,blid,rowid from partmap where blid = ? and bltoldd and rowid != ?',
                                        (blid,rowid)).fetchall()
        else:
            return self.connection.execute('select lddid,blid,rowid from partmap where blid = ? and bltoldd',
                                        (blid,)).fetchall()
        
            
    def partMapLDD2BLExists(self,lddid):
        return self.connection.execute('select lddid,blid from partmap where lddtobl and lddid=?',
                                       (lddid,)).fetchall()

    def partMapBL2LDDExists(self,blid):
        return self.connection.execute('select lddid,blid from partmap where bltoldd and blid=?',
                                       (blid,)).fetchall()

    '''
    search parts from BrickLink parts database
    '''
    #-------------------------------------------------------- get a part from ID
    def partBL(self,blid):
        return self.connection.execute('select itemid,itemname,category from blpartlist where itemid = ?',
                                       (blid,)).fetchall()
    
    def getBLPartByID(self,blid):
        return self.connection.execute('select p.itemid,p.itemname,c.catname from blpartlist as p join blcatlist as c on(p.category = c.catid) '+
                                       'where itemid like ?',
                                       (blid,)).fetchall()
                                       
    def searchBLPart(self,text):
        return self.connection.execute('select p.itemid,p.itemname,c.catname from blpartlist as p join blcatlist as c on(p.category = c.catid) '+
                                       'where p.itemname like ? or c.catname like ?',(text,text)).fetchall()
                                       

    def ftsBLPart(self,text):
        return self.connection.execute('select p.itemid,p.itemname,c.catname from blpartlist as p join blcatlist as c on(p.category = c.catid) '+
                                       'where p.itemid in (select itemid from blpartsfts where itemdescr match ?)',(text,)).fetchall()


    #===========================================================================
    # Bricklink part, color and partno database
    # used for "sanity checks"
    #===========================================================================

    def cleanBLCodesTable(self):
        self.connection.execute('drop table if exists blcodeslist')        
        self.connection.execute('create table blcodeslist (blid text, blcolorid integer, partno text)')
        
    def insertBLCode(self,blid,color,partno):
        if isinstance(color,int): 
            c = color
        else:
            c = self.getBLColorByName(color)
        self.connection.execute('insert into blcodeslist values(?,?,?)',
                                (blid, c,partno))
            


    #===========================================================================
    # BrickLink color database and ldd mapping
    #===========================================================================
    
    def cleanBLColorTable(self):
        self.connection.execute('drop table if exists blcolorlist')        
        self.connection.execute('create table blcolorlist (colorid integer, '+
                                'name text, rgb text, type text, fromy integer, toy integer)')
        
    def insertBLColor(self,colorId,colorName,rgb,colorType,fromYear,toYear):
        self.connection.execute('insert into blcolorlist values(?,?,?,?,?,?)',
                                (colorId,colorName,rgb,colorType,fromYear,toYear))    
    
    def cleanColormapTable(self):
        self.connection.execute('drop table if exists colormap')
        self.connection.execute('create table colormap (pname text, blname text, blid integer, ldrawid integer,'+
            'lddid integer, lddname text,' + 
            'rgb text,notes text,lmod bool, lastmod integer)')        
    
    def insertColormap(self,pname,blname,blid,ldrawid,legoid,legoname,rgb,notes,lmod,lastmod):
        if not lmod:
            self.connection.execute('insert into colormap values(?,?,?,?,?,?,?,?,?,?)',
                     (pname,blname,blid,ldrawid,legoid,legoname,rgb,notes,lmod,lastmod))
        else:
            self.connection.execute('insert into colormap values(?,?,?,?,?,?,?,?,?,strftime(\'%s\',\'now\'))',
                     (pname,blname,blid,ldrawid,legoid,legoname,rgb,notes,lmod))            
    
    '''
    maps color from Bricklink to LDD
    '''        
    def colorBL2Ldd(self,blcolorid):
        return self.connection.execute('select blname,blid,lddname,lddid,rgb from colormap where blid = ?',
                                       (blcolorid,)).fetchall()

    def colorLDD2BL(self,lddcolor):
        return self.connection.execute('select blname,blid,lddname,lddid,rgb from colormap where lddid = ?',
                                       (lddcolor,)).fetchall()
        
    '''
    retrieve all color data from BrickLink color database
    '''
    
    def getBLColors(self):
        return self.connection.execute('select colorid,name,rgb from blcolorlist order by colorid').fetchall()
    
    def getBLColorByName(self,colorname):
        c = self.connection.execute('select blid from colormap where blname=?',(colorname,)).fetchone()
        if c != None:
            return c[0]
        else:
            return -1
                                    
    def colorBL(self,blcolorid): 
        return self.connection.execute('select name,rgb,colorid from blcolorlist where colorid = ?',
                                (blcolorid,)).fetchall()

    def getAllColormap(self):
        return self.connection.execute('select pname ,blname, blid, ldrawid, lddid, lddname,' + 
            'rgb,notes,lmod,lastmod from colormap')

    def getLDDColors(self):
        return self.connection.execute('select lddid,lddname,rgb from colormap order by lddid')
    
    def getLDDColor(self,lddcolorid):
        c = self.connection.execute('select lddname,rgb from colormap where lddid=?',(lddcolorid,)).fetchone()
        if c != None:
            return (c[0],c[1])
        else:
            return (0,None)
         

    #===============================================================================
    # LDD color and decoration for composite parts
    #===============================================================================
    
    def cleanLDDCompositeTable(self):
        self.connection.execute('drop table if exists lddcomposite')
        self.connection.execute('create table if not exists lddcomposite (lddid integer, colorsubid integer,'+
                                'decorsubid integer,coloridx integer,decoridx integer,notes text)')

    def insLDDComposite(self,lddid,colorsubid,decorsubid,coloridx,decoridx,notes):
        self.connection.execute('insert into lddcomposite (lddid,colorsubid,decorsubid,coloridx,decoridx,notes) '+
                                'values(?,?,?,?,?,?)',(lddid,colorsubid,decorsubid,coloridx,decoridx,notes.decode('utf-8')))
        
    def readLDDComposite(self):
        return self.connection.execute('select lddid,colorsubid,decorsubid,coloridx,decoridx,notes from lddcomposite').fetchall()
    
    def getLDDComposite(self,lddid):
        return self.connection.execute('select lddid,colorsubid,decorsubid,coloridx,decoridx,notes from lddcomposite where lddid = ?',(lddid,)).fetchall()

    #===========================================================================
    # current brick list db interface
    #===========================================================================
    def readCurrList(self):
        return self.connection.execute('select type, blid, qty, blcolorid,'+
                          'extra, counterpart, descr,' + 
                          'lddid, lddrgb, lddcolorname, blrgb,'+
                          'blcolorname, blcategory, partid, lddcolorid,'+
                          'decorated, altern from bricklist').fetchall()
                          
    def cleanCurrList(self):
        self.connection.execute('drop table if exists bricklist')

        self.connection.execute('create table bricklist (type text, blid text, qty integer, blcolorid integer,'+
                                'extra bool, counterpart bool, descr text,' + 
                                'lddid integer, lddrgb text, lddcolorname text, blrgb text,'+
                                'blcolorname text, blcategory integer, partid integer, lddcolorid integer,'+
                                'decorated bool, altern bool)')
        
    def add2CurrList(self,part):
        self.connection.execute('insert into bricklist (type, blid, qty, blcolorid,'+
                                'extra, counterpart, descr,' + 
                                'lddid, lddrgb, lddcolorname, blrgb,'+
                                'blcolorname, blcategory, partid, lddcolorid,'+
                                'decorated, altern)' +
                                'values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)',part)
        
    def getCurrMaxPartID(self):
        c = self.connection.execute('select max(partid) from bricklist').fetchone()
        return c[0]    

    def searchCurrBrick(self,lddid,lddcolorid,decorated):
        return self.connection.execute('select type, blid, qty, blcolorid,'+
                                'extra, counterpart, descr,' + 
                                'lddid, lddrgb, lddcolorname, blrgb,'+
                                'blcolorname, blcategory, partid, lddcolorid,'+
                                'decorated, altern from bricklist '+
                                'where lddid=? and lddcolorid=? and decorated=?',
                                (lddid,lddcolorid,decorated)).fetchone()
                                
    def countParts(self):
        return self.connection.execute('select sum(qty) from bricklist').fetchall()[0][0]

    #===========================================================================
    # brick catalog db interface
    #===========================================================================

    def cleanCatalog(self):
        self.connection.execute('drop table if exists catalog')
        self.connection.execute('create table catalog (lddid integer, lddcolorid integer, '+
                                'decorated bool, descr text, qty integer)')

    def createTempCatalog(self):
        self.connection.execute('alter table catalog rename to tempcatalog')
        self.connection.execute('create table catalog (lddid integer, lddcolorid integer, '+
                                'decorated bool, descr text, qty integer)')

    def restoreTempCatalog(self):
        self.connection.execute('drop table if exists catalog')
        self.connection.execute('alter table tempcatalog rename to catalog')

    def removeTempCatalog(self):
        self.connection.execute('drop table if exists tempcatalog')
        self.commit()
    
    def getCatalog(self):
        return self.connection.execute('select c.rowid, c.lddid as partid, c.lddcolorid as colorid, c.decorated,'+
                                'c.descr, c.qty, m.lddname, m.rgb from '+
                                'catalog as c join colormap as m on c.lddcolorid = m.lddid')
    
    def insert2Catalog(self,lddid,lddcolorid,decorated,descr,qty):
        self.connection.execute('insert into catalog (lddid,lddcolorid,decorated, descr, qty)'+
                                'values (?,?,?,?,?)',
                                (lddid,lddcolorid,decorated, descr, qty))
        r = self.connection.execute('select last_insert_rowid()').fetchone()
        return r[0]
    
    def insert2CatalogID(self,rowid,lddid,lddcolorid,decorated,descr,qty):
        self.connection.execute('insert into catalog (rowid, lddid, lddcolorid, '+
                                'decorated, descr, qty) '+
                                'values (?,?,?,?,?,?)',
                                (rowid,lddid,lddcolorid,decorated,descr,qty))

    '''
    gets a specific brick from catalog
    @return: all db rows with brick and rowid
    @warning: in normal situation this method returns zero or one row. If returns more row
    then there are duplicated brick records in catalog DB
    '''
    def getBrick(self,lddid,lddcolorid,decorated):
        return self.connection.execute('select rowid,lddid,lddcolorid,decorated,qty from catalog '+
                      'where lddid=? and lddcolorid=? and decorated=?',(lddid,lddcolorid,decorated)).fetchall()

    def updateBrickQty(self,rowid,quantity):
        self.connection.execute('update catalog set qty=? where rowid=?',(quantity,rowid))
        
    def updateBrick(self,rowid,lddid,lddcolorid,decorated,descr,qty):
        self.connection.execute('update catalog set lddid=?,lddcolorid=?,decorated=?,descr=?,qty=? where rowid=?',
                                (lddid,lddcolorid,decorated,descr,qty,rowid))
        
    def removeBricks(self,brickid,qty):
        c = self.connection.execute('select qty from catalog where rowid=?',
                                    (brickid,)).fetchall()
        if c != None and len(c) == 1:
            remaining = c[0][0] - qty
            if remaining <= 0:
                remaining = 0
            self.connection.execute('update catalog set qty=? where rowid=?',
                                    (remaining,brickid))
        elif c != None and len(c) > 1:
            raise BrickException('Duplicate brick id in catalog')
            
    def deleteBrick(self,brickid):
        self.connection.execute('delete from catalog where rowid=?',(brickid,))

    # count part type and total parts in catalog
    def getCatalogCount(self):
        c = self.connection.execute('select count(*) from catalog where qty > 0').fetchall()[0][0]
        t = self.connection.execute('select sum(qty) from catalog').fetchall()[0][0]
        return (c,t)

        
    '''
    deletes bricks description from catalog where quantity is zero
    '''
    def deleteEmptyBricks(self):
        self.connection.execute('delete from catalog where qty = 0')
        
        
    def dumpCatalog(self):
        return self.connection.execute('select rowid,lddid,lddcolorid,decorated,descr,qty from catalog')
        

    #===========================================================================
    # set catalog db interface
    #===========================================================================

    def createTempSet(self):
        self.connection.execute('alter table setcatalog rename to tempsetcatalog')
        self.connection.execute('alter table setbricks rename to tempsetbricks')
        self.commit()
        self.connection.execute('create table setcatalog (setid text)')
        self.connection.execute('create table setbricks (setid integer, brickid integer, num integer)')
        self.commit()
        
    def restoreTempSet(self):
        self.connection.execute('drop table if exists setcatalog')
        self.connection.execute('drop table if exists setbricks')
        self.connection.execute('alter table tempsetcatalog rename to setcatalog')
        self.connection.execute('alter table tempsetbricks rename to setbricks')
        
    def removeTempSet(self):
        self.connection.execute('drop table if exists tempsetcatalog')
        self.connection.execute('drop table if exists tempsetbricks')
        self.commit()        
    
    def getSet(self,setid):
        return self.connection.execute('select rowid,setid,num from setcatalog where setid = ?',
                                       (setid,)).fetchall()
                                       
    def getAllSetDetails(self):
        return self.connection.execute('select s.rowid as rowid, s.setid as setid, coalesce(b.setname,s.setid) as setname, coalesce(b.setyear,0) as setyear, '+
                                       'coalesce(c.catname,\'MOCs/Bricks Lot\') as catname '+
                      'from (setcatalog as s left join blsetlist as b using(setid)) '+
                      'left join blcatlist as c on b.setcategory = c.catid')
        
    '''
    insert a set in set table
    @return: rowid of set added
    '''
    def insertSet(self,setid):
        self.connection.execute('insert into setcatalog (setid) values (?)', (setid,))
        #------------------- reads back the rowid assigned to new set record
        r = self.connection.execute('select last_insert_rowid()').fetchone()
        return r[0]
    
    def insertSetId(self,ids,setid):
        self.connection.execute('insert into setcatalog (rowid,setid) values (?,?)', (ids,setid))        
    
    def deleteSet(self,rowid):
        self.connection.execute('delete from setcatalog where rowid=?',(rowid,))

    def getBrickSet(self,setrowid,brickid):
        return self.connection.execute('select rowid,setid,brickid,num from setbricks where setid = ? and brickid = ?',
                      (setrowid,brickid)).fetchall()

    def getAllBrickSet(self,setrowid):
        return self.connection.execute('select rowid,setid,brickid,num from setbricks where setid = ?',
                      (setrowid,)).fetchall()
                      
    def dumpSetCatalog(self):
        return self.connection.execute('select rowid, setid from setcatalog')
    

    def delAllBrickSet(self,setrowid):
        self.connection.execute('delete from setbricks where setid = ?',
                                (setrowid,))

    def insert2BrickSet(self,setrowid,brickid,qty):
        self.connection.execute('insert into setbricks (setid,brickid,num) values(?,?,?)',
                                (setrowid,brickid,qty))
        
    def updateBrickSetQty(self,rowid,qty):
        self.connection.execute('update setbricks set num=? where rowid=?',
                          (qty,rowid))

    '''
    counts sets that use a specific brick
    '''
    def countSetWithBrick(self,brickid):
        c = self.connection.execute('select count(setid) from setbricks where brickid=?',(brickid,)).fetchone()
        if len(c) == 1:
            return c[0]
        else:
            return 0        
        
    def commit(self):
        self.connection.commit()
        
    #===========================================================================
    # special high level functions
    #===========================================================================
    
    def checkIfBuildable(self):
        return self.connection.execute('select b.partid,b.lddid,b.lddcolorid,b.decorated,b.descr as name, '+
                                       'coalesce((b.qty-c.qty),b.qty) as needed,b.lddcolorname,b.lddrgb '+
                                       'from bricklist as b left outer join catalog as c '+
                                       'using(lddid,lddcolorid,decorated) '+
                                       'where needed > 0;').fetchall()

    def findAltPart(self,lddid):
        if lddid == 0:
            return []
        return self.connection.execute('select c.rowid, c.lddid,c.lddcolorid,c.decorated,c.descr, '+
                                       'coalesce((c.qty-b.qty),c.qty) as remain,p.lddname,p.rgb '+
                                       'from (catalog c left outer join bricklist b using(lddid,lddcolorid,decorated)) '+
                                       'join colormap p on (c.lddcolorid = p.lddid) '+
                                       'where c.lddid=? and remain > 0',(lddid,)).fetchall()

    def findAltByCategory(self,lddid):
        p = self.partLdd2Bl(lddid)
        if len(p) == 0:
            return []
        blid = p[0][0]
        p = self.partBL(blid)
        if len(p) == 0:
            return []
        cat = p[0][2]
        return self.connection.execute('select c.rowid, c.lddid,c.lddcolorid,c.decorated,c.descr, '+
                                   'coalesce((c.qty-b.qty),c.qty) as remain,p.lddname,p.rgb '+
                                   'from (catalog c left outer join bricklist b using(lddid,lddcolorid,decorated)) '+
                                   'join colormap p on (c.lddcolorid = p.lddid) where remain > 0 and c.lddid in '+
                                   '(select c.lddid from (catalog c join partmap m using(lddid)) join blpartlist b on '+
                                   '(m.blid = b.itemid) where m.lddtobl and b.category = ?)',(cat,)
                                   ).fetchall()
#        return self.connection.execute('select c.rowid, c.lddid,c.lddcolorid,c.decorated,c.descr, '+
#                                   'coalesce((c.qty-b.qty),c.qty) as remain,p.lddname,p.rgb '+
#                                   'from (((catalog c left outer join bricklist b using(lddid,lddcolorid,decorated)) '+
#                                   'join colormap p on (c.lddcolorid = p.lddid) join partmap m using(lddid)) join blpartlist bl '+
#                                   'on(m.blid=bl.itemid)) where m.lddtobl and bl.category=?',(cat,)).fetchall()
        

    def convertLDD2BL(self,lddid,lddcolorid,decorated):
        result = self.partLdd2Bl(lddid)
        if len(result) == 0:
            blid = 0
        elif len(result) == 1:
            blid = result[0][0]
        else:
            #---------------- at the moment we consider only 1:1 mapping
            raise BrickException('Internal: duplicate part in BL part mapping')
        
        result = self.partBL(blid)
        
        #--------------------- checks for duplicate part description
        if len(result) > 1:
            raise BrickException('Internal: duplicate part in BL partlist')
        #------------------ no part reference in BL part list, unknown part
        elif len(result) == 0:
            descr = 'no match found - unknown part'
            blcategory = -1
        else:
            descr = result[0][1]
            blcategory = result[0][2]
        
        result = self.colorLDD2BL(lddcolorid)
        if len(result) == 0:
            blcolorid = 0
            lddrgb = None
            lddcolorname = 'no match'
        else:
            blcolorid = result[0][1]
            lddrgb = result[0][4]
            lddcolorname = result[0][2]
            
        result = self.colorBL(blcolorid)
        if len(result) == 0:
            blcolor = None
            blcolorname = 'unknown'
        else:
            blcolor = result[0][1]
            blcolorname = result[0][0]
        return {'blid':blid, 'blcolorid':blcolorid,'descr':descr,'lddid':lddid,'lddrgb':lddrgb,
                'lddcolorname':lddcolorname,'blrgb':blcolor,'blcolorname':blcolorname,
                'blcategory':blcategory,'lddcolorid':lddcolorid,'decorated':decorated}
        
        
    def detectBLdecoration(self,blid):
        # returns a tuple with (cleanblid,decorflag) where:
        # cleanblid is a blid without "decoration" field
        # decorflag is true if decoration detected
        
        g = re.match('(\w+)(p[bsxaert]{0,1}\d+)(c\d+)*',blid,re.IGNORECASE)
        if g != None:
            try:
                if g.group(2) != None:
                    return (g.group(1),True)
                else:
                    return ('-',False)
            except IndexError:
                return ('-',False)
        return ('-',False)
    
    
    def detectLDRdecoration(self,ldr):
        # returns a tuple with (cleanldrid,decorflag) where:
        # cleanldrid is a ldrawid without "decoration" field
        # decorflag is true if decoration detected
        
        g = re.match('(\w+)(p[bsxat]{0,1}\d+)(c\d+)*\.dat',ldr,re.IGNORECASE)
        if g != None:
            try:
                if g.group(2) != None:
                    return (g.group(1),True)
                else:
                    return ('-',False)
            except IndexError:
                return ('-',False)
        return ('-',False)
    
    
    def sanityChecks(self):
        parts = self.connection.execute('select b.partid,b.lddid,b.lddcolorid,b.decorated,b.descr as name, '+
                                       'b.qty,b.lddcolorname,b.lddrgb from '+
                                       'bricklist b left join blcodeslist using(blid,blcolorid) '+
                                       'where partno is null').fetchall()
        errMsg = 'Part is not available in this color'
        errRank = 3
        errAction = 'Choose a different color for this part\ncheck BrickLink for existing production colors for this part'
        return (parts,errMsg,errRank,errAction)
                        

    def close(self):
        self.connection.commit()
        self.connection.close()
        

    def __init__(self,catalogFile):
        '''
        Constructor
        '''
        self.catalogFile = catalogFile
        self.connection = sqlite3.connect(self.catalogFile,check_same_thread = False)
        self.connection.row_factory = sqlite3.Row
        