'''
Created on 17/feb/2012

functions for exporting and importing brick and set catalogs

@author: mario
'''

import gtk
import sqlite3
import xml.parsers.expat
from BrickException import BrickException


def exportBrickCatalog(catalog,fileExp):
    
    bricks = catalog.dumpCatalog()
    
    fileExp.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
    fileExp.write('<brickutils>\n  <bricks>\n')
    
    for b in bricks:
        fileExp.write('    <brick id=\'{0}\' lddid=\'{1}\' lddcolor=\'{2}\' decorated=\'{3}\' qty=\'{4}\'>'.format(
                        b[0],b[1],b[2],b[3],b[5]))
        fileExp.write(b[4])
        fileExp.write('</brick>\n')
    fileExp.write('  </bricks>\n</brickutils>\n')
        

def exportSetCatalog(catalog,fileExp):
    
    sets = catalog.dumpSetCatalog()
    
    fileExp.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
    fileExp.write('<brickutils>\n  <sets>\n')
    
    for s in sets:
        bricks = catalog.getAllBrickSet(s[0])
        fileExp.write('    <set id=\'{0}\' setid=\'{1}\'>\n'.format(s[0],s[1]))
        
        for b in bricks:
            fileExp.write('      <brick id=\'{0}\' num=\'{1}\'/>\n'.format(b[2],b[3]))
        fileExp.write('    </set>\n')
    fileExp.write('  </sets>\n</brickutils>\n')


class importBrickCatalog():
    
    def hStartEl(self,name,attrs):
        #-------------------------------------------- let the GUI updates itself
        while gtk.events_pending():
            gtk.main_iteration(block=False)
        if name == 'brickutils':
            #--------------------------------- ok, this is a start in catalog export XML
            self.fBrickutils = True
            return
        if name == 'bricks':
            #-------------------------- it's ok if we are into "bricks" element
            if self.fBrickutils:
                self.fBricks = True
                return
            else:
                raise BrickException("Invalid XML catalog file, 'bricks' element outside 'brickutils'")
        if name == 'brick':
            if self.fBricks:
                self.fBrick = True
                if (attrs.has_key('id')):
                    self.brickId = attrs['id']
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'id'")
                if (attrs.has_key('lddid')):
                    try:
                        self.brickDesignId = int(attrs['lddid'])
                    except ValueError:
                        raise BrickException("Invalid lddid: '"+attrs['lddid']+"'. Please check.")
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'lddid'")
                if (attrs.has_key('lddcolor')):
                    try:
                        self.brickColor = int(attrs['lddcolor'])
                    except ValueError:
                        raise BrickException("Invalid colorID: '"+attrs['lddcolor']+"'. Please check.")
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'lddcolor'")
                if (attrs.has_key('decorated')):                    
                    self.brickDecorated = attrs['decorated'] == '1'
                else:
                    self.brickDecorated = False
                if (attrs.has_key('qty')):
                    try:
                        self.brickQty = int(attrs['qty'])
                    except ValueError:
                        raise BrickException("Invalid quantity: '"+attrs['qty']+"'. Please check.")
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'qty'")                    
            else:
                raise BrickException("Invalid catalog XML file, 'brick' element outside 'bricks'")

    def hCharData(self,data):
        if self.fBrick:
            self.brickDescr = data.strip()
            
    def hEndEl(self,name):
        if name == 'brick':
            self.catalog.insert2CatalogID(self.brickId,self.brickDesignId,self.brickColor,
                                          self.brickDecorated,self.brickDescr,self.brickQty)
            self.fBrick = False
            self.brickId = 0
            self.brickDesignId = 0
            self.brickColor = 0
            self.brickDecorated = False
            self.brickQty = 0
            self.brickDescr = ''

    
    def __init__(self,fileXml=None,catalog=None):
        '''
        Takes as arguments: 
        fileXml - file handle of catalog xml export
        catalog - connection to sqlite3 catalog db
        '''
        
        self.fileXml = fileXml
        self.catalog = catalog
        #-------------------------------------------------- creates Expat parser
        self.Parser = xml.parsers.expat.ParserCreate()
        #----------------------------------------------------- defines callbacks
        self.Parser.StartElementHandler = self.hStartEl
        self.Parser.CharacterDataHandler = self.hCharData
        self.Parser.EndElementHandler = self.hEndEl
        self.Parser.buffer_text = True
        self.fBrickutils = False
        self.fBricks = False
        self.fBrick = False
        self.brickId = 0
        self.brickDesignId = 0
        self.brickColor = 0
        self.brickDecorated = False
        self.brickQty = 0
        self.brickDescr = ''
        
        
    def run(self):
        try:
            self.catalog.createTempCatalog()
            self.Parser.ParseFile(self.fileXml)
            self.catalog.removeTempCatalog()
        except (BrickException,sqlite3.OperationalError):
            self.catalog.restoreTempCatalog()
        self.catalog.commit()
        


class importSetCatalog():
    
    def hStartEl(self,name,attrs):
        #-------------------------------------------- let the GUI updates itself
        while gtk.events_pending():
            gtk.main_iteration(block=False)
        if name == 'brickutils':
            #--------------------------------- ok, this is a start in set XML catalog
            self.fBrickutils = True
            return
        if name == 'sets':
            #-------------------------- it's ok if we are into "sets" element
            if self.fBrickutils:
                self.fSets = True
                return
            else:
                raise BrickException("Invalid XML catalog file, 'sets' element outside 'brickutils'")
        if name == 'set':
            if self.fSets:
                self.fSet = True
                if (attrs.has_key('id')):
                    self.setId = attrs['id']
                else:
                    raise BrickException("Invalid XML catalog file, 'set' without 'id'")
                if (attrs.has_key('setid')):
                    self.setOfficialId = attrs['setid']
                else:
                    raise BrickException("Invalid XML catalog file, 'set' without 'setid'")
                self.catalog.insertSetId(self.setId,self.setOfficialId)
        if name == 'brick':
            if self.fSet:
                if (attrs.has_key('id')):
                    try:
                        self.brickId = int(attrs['id'])
                    except ValueError:
                        raise BrickException("Invalid brickId: '"+attrs['id']+"'. Please check.")
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'id'")
                if (attrs.has_key('num')):
                    try:
                        self.brickQty = int(attrs['num'])
                    except ValueError:
                        raise BrickException("Invalid quantity: '"+attrs['num']+"'. Please check.")
                else:
                    raise BrickException("Invalid XML catalog file, 'brick' without 'num'")
                self.catalog.insert2BrickSet(self.setId,self.brickId,self.brickQty)
            else:
                raise BrickException("Invalid set XML file, 'brick' element outside 'set'")

    def hEndEl(self,name):
        if name == 'brick':
            self.brickId = 0
            self.brickQty = 0
        if name == 'set':
            self.fSet = False
            self.setId = 0
            self.setOfficialId = 0
        if name == 'sets':
            self.fSets = False

    
    def __init__(self,fileXml=None,catalog=None):
        '''
        Takes as arguments: 
        fileXml - file handle of catalog xml export
        catalog - connection to sqlite3 catalog db
        '''
        
        self.fileXml = fileXml
        self.catalog = catalog
        #-------------------------------------------------- creates Expat parser
        self.Parser = xml.parsers.expat.ParserCreate()
        #----------------------------------------------------- defines callbacks
        self.Parser.StartElementHandler = self.hStartEl
        self.Parser.EndElementHandler = self.hEndEl
        self.Parser.buffer_text = True
        self.fBrickutils = False
        self.fSets = False
        self.fSet = False
        self.setId = 0
        self.setOfficialId = 0
        self.brickId = 0
        self.brickDecorated = False
        self.brickQty = 0
        self.brickDescr = ''
        
        
    def run(self):
        try:
            self.catalog.createTempSet()
            self.Parser.ParseFile(self.fileXml)
            self.catalog.removeTempSet()
        except (BrickException,sqlite3.OperationalError):
            self.catalog.restoreTempSet()
        self.catalog.commit()
        
        