#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  ColorMaintDialog - class for utility to edit colormap DB
  Copyright (C) 2011 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
# - 
#===============================================================================



#===============================================================================
# Imports
#===============================================================================


import os.path
import time
import sqlite3
import gtk
import gobject
import pygtk
pygtk.require('2.0')



class ColorList(gtk.ListStore):
    
    def read(self):
        c = self.catalog.getAllColormap()
        
        self.clear()
        for r in c:
            try:
                blid = int(r[2])
            except ValueError:
                blid = 0
                
            try: 
                ldraw = int(r[3])
            except ValueError:
                ldraw = 0
                
            try:
                lddid = int(r[4])
            except ValueError:
                lddid = 0
                
            self.append(row=(r[0],
                       r[1],
                       blid,
                       ldraw,
                       lddid,
                       r[5],
                       r[6],
                       r[7],
                       r[8],
                       r[9]
                       ))
      
        self.set_sort_column_id(4,gtk.SORT_ASCENDING)

    def write(self):
        # re-create table with color map list
        self.catalog.cleanColormapTable()
        
        for r in self:
            self.catalog.insertColormap(r[0].decode('utf-8') if r[0] != None else None,
                                        r[1].decode('utf-8') if r[1] != None else None,
                                        r[2],r[3],r[4],
                                        r[5].decode('utf-8') if r[5] != None else None,
                                        r[6],
                                        r[7].decode('utf-8') if r[7] != None else None,
                                        r[8],r[9]
                                        )
                            # (pname,blname,blid,ldrawid,legoid,legoname,rgb,notes)
        self.catalog.commit()        
    
    def __init__(self,catalog=None):
        super(ColorList, self).__init__(gobject.TYPE_STRING,    # peeron name
                                        gobject.TYPE_STRING,    # bl name 
                                        gobject.TYPE_INT,       # bl color id
                                        gobject.TYPE_INT,       # ldraw color id
                                        gobject.TYPE_INT,       # ldd color id
                                        gobject.TYPE_STRING,    # ldd color name
                                        gobject.TYPE_STRING,    # rgb as 'RRGGBB' hex
                                        gobject.TYPE_STRING,     # notes
                                        gobject.TYPE_BOOLEAN,   # locally modified
                                        gobject.TYPE_INT        # timestamp
                                        )
        self.catalog = catalog


class ColorListView(gtk.TreeView):

    def setSearchCB(self,widget,column):
        self.set_search_column(column)
    
    def __init__(self,liststore=None):
        super(ColorListView, self).__init__(liststore)
        
        self.set_search_column(4)
        self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_HORIZONTAL)

        #---------------------- column and cell renderer definition for treeview
        self.renderid = gtk.CellRendererText()
        self.tvcolid = gtk.TreeViewColumn('LDD ID',self.renderid)
        self.tvcolid.set_resizable(True)
        
        self.renderblid = gtk.CellRendererText()
        self.tvcolblid = gtk.TreeViewColumn('BrickLink ID',self.renderblid)
        self.tvcolblid.set_resizable(True)
        
        self.rendername = gtk.CellRendererText()
        self.tvcolname = gtk.TreeViewColumn('LDD color name',self.rendername)
        self.tvcolname.set_resizable(True)
        
        self.rendercol = gtk.CellRendererText()
        self.tvcolsample = gtk.TreeViewColumn('Color sample',self.rendercol)
        self.tvcolsample.set_resizable(True)
        
        self.rendernotes = gtk.CellRendererText()
        self.tvcolnotes = gtk.TreeViewColumn('Notes & comments',self.rendernotes)
        self.tvcolnotes.set_resizable(True)

        self.append_column(self.tvcolid)
        self.append_column(self.tvcolblid)
        self.append_column(self.tvcolname)
        self.append_column(self.tvcolsample)
        self.append_column(self.tvcolnotes)
        self.set_expander_column(self.tvcolnotes)
        self.tvcolid.set_attributes(self.renderid,text=4)
        self.tvcolblid.set_attributes(self.renderblid,text=2)
        self.tvcolname.set_attributes(self.rendername,text=5)
        self.tvcolsample.set_attributes(self.rendercol,cell_background=6)
        self.tvcolnotes.set_attributes(self.rendernotes,text=7)
        self.tvcolblid.set_sort_column_id(2)
        self.tvcolname.set_sort_column_id(5)        
        self.tvcolid.set_sort_column_id(4)
        
        self.tvcolblid.connect('clicked',self.setSearchCB,2)
        self.tvcolname.connect('clicked',self.setSearchCB,5)
        self.tvcolid.connect('clicked',self.setSearchCB,4)
        

class CMEditDialog(gtk.Dialog):
    
    #---------------------------------------------- color select button callback
    def colorSelectCB(self,widget):
        color = self.colorButton.get_color()
    
        rgbhex = '#%(red)02X%(green)02X%(blue)02X' % {'red':(color.red/256), 
                                                      'green':(color.green/256), 
                                                      'blue':(color.blue/256)}
        self.rgb.set_text(rgbhex)
        self.colorSample.modify_bg(gtk.STATE_NORMAL,color)
    
    
    def __init__(self,title=None, parent=None, flags=0, 
                 buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT)):
        super(CMEditDialog, self).__init__(title=title, parent=parent, flags=flags, buttons=buttons)

        self.vBox = self.get_content_area()
        
        self.table = gtk.Table(rows=5, columns=5)
        self.vBox.pack_start(self.table,True)
        
        self.lab1 = gtk.Label('LDD ID (primary)')
        self.lab1.set_alignment(1,0.5)
        self.table.attach(self.lab1,0,1,0,1,xoptions=gtk.FILL,xpadding=3)
        
        self.lddid = gtk.Entry(max=6)
        self.table.attach(self.lddid,1,2,0,1,xoptions=gtk.SHRINK)
        
        self.lab2 = gtk.Label('   ')
        self.table.attach(self.lab2,2,3,0,1,xoptions=gtk.FILL)
        
        self.lab3 = gtk.Label('LDD color name')
        self.lab3.set_alignment(1,0.5)
        self.table.attach(self.lab3,3,4,0,1,xoptions=gtk.FILL,xpadding=3)
        
        self.lddname = gtk.Entry()
        self.table.attach(self.lddname,4,5,0,1)

        self.lab4 = gtk.Label('Bricklink ID')
        self.lab4.set_alignment(1,0.5)
        self.table.attach(self.lab4,0,1,1,2,xoptions=gtk.FILL,xpadding=3)
        
        self.blid = gtk.Entry(max=6)
        self.table.attach(self.blid,1,2,1,2,xoptions=gtk.SHRINK)
                
        self.lab5 = gtk.Label('Bricklink color name')
        self.lab5.set_alignment(1,0.5)
        self.table.attach(self.lab5,3,4,1,2,xoptions=gtk.FILL,xpadding=3)
        
        self.blname = gtk.Entry()
        self.table.attach(self.blname,4,5,1,2)        

        self.lab6 = gtk.Label('LDraw ID')
        self.lab6.set_alignment(1,0.5)
        self.table.attach(self.lab6,0,1,2,3,xoptions=gtk.FILL,xpadding=3)
        
        self.ldrawid = gtk.Entry(max=6)
        self.table.attach(self.ldrawid,1,2,2,3,xoptions=gtk.SHRINK)
                
        self.lab7 = gtk.Label('Peeron color name')
        self.lab7.set_alignment(1,0.5)
        self.table.attach(self.lab7,3,4,2,3,xoptions=gtk.FILL,xpadding=3)
        
        self.pname = gtk.Entry()
        self.table.attach(self.pname,4,5,2,3)
        
        self.lab9 = gtk.Label('Sample color (RGB)')
        self.lab9.set_alignment(1,0.5)
        self.table.attach(self.lab9,0,1,3,4,xoptions=gtk.FILL,xpadding=3)
        
        self.rgb = gtk.Entry()
        self.table.attach(self.rgb,1,2,3,4)
        self.rgb.set_editable(False)
        
        self.colorButton = gtk.ColorButton()
        self.colorButton.connect('color-set',self.colorSelectCB)
        self.colorButton.set_title("Select a color")
        self.table.attach(self.colorButton,2,3,3,4)
        
        #---------------------------------------------- box showing color sample
        self.lab10 = gtk.Label('Color (approx.)')
        self.lab10.set_alignment(1,0.5)
        self.table.attach(self.lab10,3,4,3,4,xpadding=3)
        
        self.colorSample = gtk.EventBox()
        self.table.attach(self.colorSample,4,5,3,4,xpadding=5,ypadding=5)
        self.colorSample.modify_bg(gtk.STATE_NORMAL,gtk.gdk.Color('#fff'))
        
        self.lab8 = gtk.Label('Notes')
        self.lab8.set_alignment(1,0.5)
        self.table.attach(self.lab8,0,1,4,5,xpadding=3)
        
        self.notes = gtk.Entry()
        self.table.attach(self.notes,1,5,4,5)
        
        self.vBox.show_all()




class BrickColorMaint(gtk.Dialog):
    
    #---------------------------------------------------- some utility functions
    
    def readCatalog(self):
        self.colorList.read()
        self.saveDBButton.set_sensitive(False)
        self.modified = False
        self.imported = False
    
    #----------------------------------------------------------------- callbacks
    
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')

    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Color database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Color database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

  
#    #------------------------------------------------- import button callback
#    def importButtonCB(self,widget):
#        #----------------------------- color DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Color database modified, ignore and lose changes?")
#            msg.set_title('Confirm open')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#            
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            colormapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of colors
#            conn = sqlite3.connect(colormapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query color table
#            c.execute('select pname, blname, blid, ldrawid, lddid, lddname, rgb, notes, lmod,lastmod '+
#                      'from colormap order by lddid')
#            
#            #------------------------------------ iterates through query results
#            self.colorList.clear()
#            for r in c:
#                try:
#                    blid = int(r[2])
#                except ValueError:
#                    blid = 0
#                    
#                try: 
#                    ldraw = int(r[3])
#                except ValueError:
#                    ldraw = 0
#                    
#                try:
#                    lddid = int(r[4])
#                except ValueError:
#                    lddid = 0
#                    
#                self.colorList.append(row=(r[0],
#                           r[1],
#                           blid,
#                           ldraw,
#                           lddid,
#                           r[5],
#                           r[6],
#                           r[7], r[8], r[9]
#                           ))
#            conn.close()
#            
#            self.colorList.set_sort_column_id(4,gtk.SORT_ASCENDING)
#            
#            self.saveDBButton.set_sensitive(True)
#            self.modified = True
#            self.imported = True
#
#
#    #------------------------------- update color map database with a new release
#    def updateDBButtonCB(self,widget):
#        #----------------------------- color DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Color database modified, ignore and lose changes?")
#            msg.set_title('Confirm update')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#        
#        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Want to keep local colors editings?")
#        msg.set_title('Keep your edits?')
#        response = msg.run()
#        msg.hide()
#        msg.destroy()
#        if response != gtk.RESPONSE_YES and response != gtk.RESPONSE_NO:
#            return
#        keepEdits = response == gtk.RESPONSE_YES
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            colormapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of colors
#            conn = sqlite3.connect(colormapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query color table
#            c.execute('select pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod from colormap')
#            #-------------------------------------------- fills in working table
#            self.catalog.connection.execute('drop table if exists newcolormap')
#            self.catalog.connection.execute('create table newcolormap (pname text,blname text,blid integer,ldrawid integer,lddid integer,lddname text,rgb text,notes text,lmod bool,lastmod integer)')
#            for color in c:
#                self.catalog.connection.execute('insert into newcolormap (pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod) values(?,?,?,?,?,?,?,?,?,?)',
#                                     (color[0],color[1],color[2],color[3],color[4],color[5],color[6],color[7],color[8],color[9]))
#            if keepEdits:
#                #---------------------------------- creates working tables for update
#                self.catalog.connection.execute('drop table if exists updatedcolormap')
#                self.catalog.connection.execute('create table updatedcolormap (pname text,blname text,blid integer,ldrawid integer,lddid integer,lddname text,rgb text,notes text,lmod bool,lastmod integer)')
#                #----------------------------- insert colors not locally modified
#                self.catalog.connection.execute('insert into updatedcolormap (pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod) '+
#                                     'select n.pname,n.blname,n.blid,n.ldrawid,n.lddid,n.lddname,n.rgb,n.notes,n.lmod,n.lastmod from newcolormap as n join colormap as o '+
#                                     'using (lddid) where n.lddid is not null and o.lmod=0'
#                                     )
#                #--------------------------- insert new colors not locally listed
#                self.catalog.connection.execute('insert into updatedcolormap (pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod) '+
#                                                'select n.pname,n.blname,n.blid,n.ldrawid,n.lddid,n.lddname,n.rgb,n.notes,n.lmod,n.lastmod from newcolormap as n left join colormap as o '+
#                                                'using (lddid) where o.lddid is null') 
#                #------------------------ insert locally modified from old table
#                self.catalog.connection.execute('insert into updatedcolormap (pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod) '+
#                                     'select pname,blname,blid,ldrawid,lddid,lddname,rgb,notes,lmod,lastmod from colormap where lmod=1'
#                                     )
#                self.catalog.connection.execute('drop table if exists colormap')
#                self.catalog.connection.execute('alter table updatedcolormap rename to colormap')
#                self.catalog.connection.execute('drop table if exists newcolormap')
#            else:
#                self.catalog.connection.execute('drop table if exists colormap')
#                self.catalog.connection.execute('alter table newcolormap rename to colormap')
#            self.catalog.commit()
#            conn.close()
#            self.colorList.read()


    #----------------------------------------------- save colormap database file
    def saveDBButtonCB(self,widget):
        if self.imported:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                            message_format="Do you want to overwrite color mapping in main catalog with the imported one?")
            msg.set_title('Confirm color mapping overwrite')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_OK:
                return
        self.colorList.write()
        self.saveDBButton.set_sensitive(False)
        self.modified = False
        self.imported = False
    
#    #------------------------------------- export colormap database as new file
#    def exportButtonCB(self,widget):
#        response = self.saveDialog.run()
#        self.saveDialog.hide()
#        if response == gtk.RESPONSE_ACCEPT:
#            colormapfile = self.saveDialog.get_filename()
#            if os.path.splitext(colormapfile)[1] == '':
#                colormapfile += '.sq3'
#        else:
#            return
#                
#        conn = sqlite3.connect(colormapfile)        
#        c = conn.cursor()
#
#        # re-create table with color map list
#        c.execute('drop table if exists colormap')
#        
#        c.execute('create table colormap (pname text, blname text, blid integer, ldrawid integer,'+
#            'lddid integer, lddname text,' + 
#            'rgb text,notes text,lmod bool, lastmod integer)')
#        
#        for r in self.colorList:
#            c.execute('insert into colormap values(?,?,?,?,?,?,?,?,?,?)',
#                      (r[0].decode('utf-8') if r[0] != None else None, 
#                       r[1].decode('utf-8') if r[1] != None else None,
#                       r[2],r[3],r[4],
#                       r[5].decode('utf-8') if r[5] != None else None,
#                       r[6],
#                       r[7].decode('utf-8') if r[7] != None else None,
#                       r[8], r[9]
#                       )    # (pname,blname,blid,ldrawid,legoid,legoname,rgb,notes)
#                      )
#        conn.commit()
#        conn.close()
        
    #------------------------------------------------- new color button callback
    def newButtonCB(self,widget):
        dlg = CMEditDialog(title='New color mapping', parent=self)
        response = dlg.run()
        dlg.hide()
        if response != gtk.RESPONSE_ACCEPT:
            return
        try:
            lddid = int(dlg.lddid.get_text())
        except ValueError:
            lddid = 0
        
        lddname = dlg.lddname.get_text()
        
        if len(lddname) == 0 or lddid == 0:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="LDD color ID and name must be set and valid.")
            msg.set_title('Error in color definition')
            msg.run()
            msg.hide()
            msg.destroy()
            return

        #----------------------------------------- retrieve color definition
        try:
            blid = int(dlg.blid.get_text())
        except ValueError:
            blid = 0
            
        try:
            ldrawid = int(dlg.ldrawid.get_text())
        except ValueError:
            ldrawid = 0
        
        rgb = dlg.rgb.get_text()
        if len(rgb) == 0:
            rgb = None
            
        for r in self.colorList:
            if r[4] == lddid:
                #---------------------------------------- duplicate ldd color id
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Duplicate LDD color ID in list")
                msg.set_title('Duplicate color ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return             
        #---------------------------------------------------------- save editing
        self.colorList.append(row=[dlg.pname.get_text(),     # peeron name
                                      dlg.blname.get_text(),    # Bricklink name
                                      blid,      # Bricklink  color ID
                                      ldrawid,   # LDraw color ID
                                      lddid,                     # LDD color ID (primary key)
                                      lddname,                   # LDD name
                                      rgb,     # RGB values
                                      dlg.notes.get_text(),      # notes
                                      True,int(time.time())
                                      ])

        #---------------------------------------------------- re-sort color list
        self.colorList.set_sort_column_id(4,gtk.SORT_ASCENDING)
        #------------------------------------------ re-select added/modified row
        for row in self.colorList:
            if row[4] == lddid:
                path = self.colorList.get_path(row.iter)
                break        
        self.colorListView.set_cursor(path)
        self.modified = True
        self.saveDBButton.set_sensitive(True)
        
    
    #------------------------------------------------------- delete color button
    def deleteButtonCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected color will be deleted and cannot be recovered.")
        msg.set_title('Delete color mapping')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete color mapping from DB
            selPath = self.colorListView.get_cursor()
            selRow = self.colorList.get_iter(selPath[0])
            self.colorList.remove(selRow)
            self.saveDBButton.set_sensitive(True)
            self.modified = True
            
    
    def editButtonCB(self,widget,selPath=None,selRow=None):
        if selPath == None:
            selPath = self.colorListView.get_cursor()
        if selPath == None:
            return
        dlg = CMEditDialog(title='Edit color mapping',parent=self)
        row = self.colorList[selPath[0]]
        dlg.lddid.set_text(str(row[4]))
        dlg.lddname.set_text(row[5] if row[5] != None else '')
        dlg.blid.set_text(str(row[2]))
        dlg.blname.set_text(row[1] if row[1] != None else '')
        dlg.ldrawid.set_text(str(row[3]))
        dlg.pname.set_text(row[0] if row[0] != None else '')
        dlg.rgb.set_text(row[6] if row[6] != None else '')
        dlg.colorButton.set_color(gtk.gdk.Color(row[6] if row[6] != None else '#fff'))
        dlg.colorSample.modify_bg(gtk.STATE_NORMAL,
                                  gtk.gdk.Color(row[6] if row[6] != None else '#fff'))
        dlg.notes.set_text(row[7] if row[7] != None else '')
        response = dlg.run()
        dlg.hide()
        if response != gtk.RESPONSE_ACCEPT:
            return
        try:
            lddid = int(dlg.lddid.get_text())
        except ValueError:
            lddid = 0
        
        lddname = dlg.lddname.get_text()
        
        if len(lddname) == 0 or lddid == 0:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="LDD color ID and name must be set and valid.")
            msg.set_title('Error in color definition')
            msg.run()
            msg.hide()
            msg.destroy()
            return

        #----------------------------------------- retrieve color definition
        try:
            blid = int(dlg.blid.get_text())
        except ValueError:
            blid = 0
            
        try:
            ldrawid = int(dlg.ldrawid.get_text())
        except ValueError:
            ldrawid = 0
        
        rgb = dlg.rgb.get_text()
        if len(rgb) == 0:
            rgb = None
            
        epath = self.colorList.get_string_from_iter(row.iter)
        for r in self.colorList:
            if r[4] == lddid and epath != self.colorList.get_string_from_iter(r.iter):
                #---------------------------------------- duplicate ldd color id
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Duplicate LDD color ID in list")
                msg.set_title('Duplicate color ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return             
        #---------------------------------------------------------- save editing
        self.colorList[selPath[0]] = [dlg.pname.get_text(),     # peeron name
                                      dlg.blname.get_text(),    # Bricklink name
                                      blid,      # Bricklink  color ID
                                      ldrawid,   # LDraw color ID
                                      lddid,                     # LDD color ID (primary key)
                                      lddname,                   # LDD name
                                      rgb,     # RGB values
                                      dlg.notes.get_text(),      # notes
                                      True,int(time.time())
                                      ]

        #---------------------------------------------------- re-sort color list
        self.colorList.set_sort_column_id(4,gtk.SORT_ASCENDING)
        #------------------------------------------ re-select added/modified row
        for row in self.colorList:
            if row[4] == lddid:
                path = self.colorList.get_path(row.iter)
                break        
        self.colorListView.set_cursor(path)
        self.modified = True
        self.saveDBButton.set_sensitive(True)
    
    
    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None):
        super(BrickColorMaint,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.dialogTitle = title
        self.catalog = catalog
        self.imported = False
        self.modified = False
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)
      
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
#        self.openButton = gtk.ToolButton()
#        self.openButton.set_stock_id(gtk.STOCK_OPEN)
#        self.openButton.set_tooltip_text("Import colormap database")
#        self.openButton.connect('clicked',self.importButtonCB)
#        self.toolBar.insert(self.openButton,-1)
        
        self.saveDBButton = gtk.ToolButton()
        self.saveDBButton.set_stock_id(gtk.STOCK_SAVE)
        self.saveDBButton.set_tooltip_text("Save colormap database in main catalog")
        self.saveDBButton.set_sensitive(False)
        self.saveDBButton.connect('clicked',self.saveDBButtonCB)
        self.toolBar.insert(self.saveDBButton,-1)
        
#        self.saveAsDBButton = gtk.ToolButton()
#        self.saveAsDBButton.set_stock_id(gtk.STOCK_SAVE_AS)
#        self.saveAsDBButton.set_tooltip_text("Export colormap database as new file")
#        self.saveAsDBButton.connect('clicked',self.exportButtonCB)
#        self.toolBar.insert(self.saveAsDBButton,-1)        
#
#        self.updateDBButton = gtk.ToolButton()
#        self.updateDBButton.set_stock_id(gtk.STOCK_REFRESH)
#        self.updateDBButton.set_label('Update')
#        self.updateDBButton.set_tooltip_text('Update colormap database from file')
#        self.updateDBButton.connect('clicked',self.updateDBButtonCB)
#        self.toolBar.insert(self.updateDBButton,-1)

        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Quit to main")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
#        #-------------------------------------------- various dialogs and popups
#        self.openDialog = gtk.FileChooserDialog("Select colormap file",self,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        
#        self.saveDialog = gtk.FileChooserDialog("Save as colormap file",self,
#            action=gtk.FILE_CHOOSER_ACTION_SAVE,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        self.saveDialog.set_do_overwrite_confirmation(True)
#        
#        #------------------------------------ add a filter for SQLite3 files
#        sqliteFilter = gtk.FileFilter()
#        sqliteFilter.add_pattern("*.sq3")
#        sqliteFilter.set_name("SQlite database files")
#        self.openDialog.add_filter(sqliteFilter)
#
#        #-------------------------------------- add a filter for any (all) files
#        allFilter = gtk.FileFilter()
#        allFilter.add_pattern("*")
#        allFilter.set_name("All files")
#        self.openDialog.add_filter(allFilter)
#        self.saveDialog.add_filter(allFilter)
        
        #----------------------------------------------- list store for treeview
        self.colorList = ColorList(self.catalog)
        self.colorListView = ColorListView(self.colorList)
        self.colorListView.connect('row-activated',self.editButtonCB)
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.colorListView)
        self.vBox.add(self.listWindow)
        
        self.hButtonBox = gtk.HButtonBox()
        self.vBox.pack_end(self.hButtonBox,False)
        
        self.newButton = gtk.Button(stock=gtk.STOCK_NEW)
        self.newButton.connect('clicked',self.newButtonCB)
        self.editButton = gtk.Button(stock=gtk.STOCK_EDIT)
        self.editButton.connect('clicked',self.editButtonCB)
        self.deleteButton = gtk.Button(stock=gtk.STOCK_DELETE)
        self.deleteButton.connect('clicked',self.deleteButtonCB)
        
        self.hButtonBox.set_layout(gtk.BUTTONBOX_END)
        self.hButtonBox.pack_start(self.newButton,False)
        self.hButtonBox.pack_start(self.editButton,False)
        self.hButtonBox.pack_start(self.deleteButton,False)
    
        self.readCatalog()
        self.set_default_size(500,300)
        
        self.vBox.show_all()
        
       

