#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  ImportBLSet - Tool to import Bricklink set inventory
  Copyright (C) 2011 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
#
#===============================================================================



#===============================================================================
# Imports
#===============================================================================



import gtk
import pygtk
import BrickListStore
import BrickListView
from BrickException import BrickException 
from ParseBLSetXML import ParseBLSetXML

pygtk.require('2.0')


class ImportBLSet(gtk.Dialog):
    
    #__decorated = (118,535,736,643,642,440,538,738,132,83,728,461,33,741,740,39,578,84,579)
    
    #----------------------------------------------------------------- callbacks
    
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')

    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

    
    #------------------------------------------------------ open button callback
    def openButtonCB(self,widget):
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            self.partCount.set_text('Reading XML...')
          
            # stores in "current list"
            currList = dict()
            counterpartList = dict()
            alternatepartList = dict()

            self.partListView.set_sensitive(False)
            parser = ParseBLSetXML(blxml,self.usePartsOnly.get_active(),self.exCounterpart.get_active(),
                                   currList,counterpartList,alternatepartList,self.partCount)
            try:
                parser.parse()
                blxml.close()
            except BrickException as e:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: "+e.message)
                msg.run()
                msg.hide()
                msg.destroy()
                self.partListView.set_sensitive(True)
                self.partCount.set_text('')
                return
           
            
            partID = 1
            cList = BrickListStore.BrickListStore(self.catalog)
            
            #------------------------ make cross references for parts and colors
            for part in currList.itervalues():
                #-------------------------------------------- let the GUI updates itself
                while gtk.events_pending():
                    gtk.main_iteration(block=False)
                
                decorated = False
                # use alternative parts
                if self.useAlternate.get_active() and part[6] != 0:
                    blid = alternatepartList[str(part[6])][1]
                else:
                    blid = part[1]  

                result = self.catalog.partBl2Ldd(blid)
                b,decorated = self.catalog.detectBLdecoration(blid)
                lddid = '-'
                #------------------ no part reference in partmap, please correct
                if len(result) == 0:
                    #---------- check if a decorated part can be "converted" 
                    if decorated and self.convertDecor.get_active():
                        result = self.catalog.partBl2Ldd(b)
                        if len(result) == 0:
                            lddid = '-'
                        else:
                            lddid = result[0][0]
                else:
                    #---------------------- 
                    lddid = result[0][0]

                # retrieve part description from BL part list
                result = self.catalog.partBL(blid)
                
                #--------------------- checks for duplicate part description
                if len(result) > 1:
                    msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                        message_format="Duplicate part in BL partlist "+str(result))
                    msg.run()
                    msg.hide()
                    msg.destroy()
                    break

                #------------------ no part reference in BL part list, unknown part
                if len(result) == 0:
                    descr = 'no match found - unknown part'
                    blcategory = -1
                else:
                    descr = result[0][1]
                    blcategory = result[0][2]
                
                #---------------------------------------------- checks for color
                if self.useAlternate.get_active() and part[6] != 0:
                    blcolorid = alternatepartList[str(part[6])][3]
                else:
                    blcolorid = part[3]
                result = self.catalog.colorBL2Ldd(blcolorid)
                if len(result) == 0 or blcolorid == '0':
                    lddcolorid = 0
                    lddcolor = None
                    lddcolorname = 'no match'
                else:
                    lddcolorid = result[0][3]
                    lddcolor = result[0][4]
                    lddcolorname = result[0][2]
                    
                result = self.catalog.colorBL(blcolorid)
                if len(result) == 0:
                    blcolor = None
                    blcolorname = 'unknown'
                else:
                    blcolor = result[0][1]
                    blcolorname = result[0][0]
                
                
                    
                row=(part[0],    # type
                    blid,    # blpartid
                    int(part[2]),    # qty
                    blcolorid,    # blcolorid
                    part[4],    # extraPart
                    part[7],    # counterpart
                    descr,      # description (from BL)
                    lddid if isinstance(lddid,str) else str(lddid),      # LDD part id
                    lddcolor,   # rgb color from LDD
                    lddcolorname,   # color name from LDD
                    blcolor,    # rgb color from BL
                    blcolorname,    # name from BL
                    blcategory, # category in BLparts
                    partID,      # unique ID for part
                    lddcolorid,     # ldd color ID
                    decorated,       # decorated part?
                    True if part[6] != 0 else False  # has alternatePart if != '0'
                    )

                cList.append(row=row)
                if (partID % 20) == 0:
                    self.partCount.set_text(str(partID))
                partID += 1
            self.currList = cList
            self.partListView.set_model(self.currList)
            self.partCount.set_text(str(partID-1))
            self.partListView.set_sensitive(True)
            self.saveButton.set_sensitive(True)
            self.modified = True            
                
    #----------------------------------------------- save brick list
    def saveDBButtonCB(self,widget):
        self.currList.write()
        self.saveButton.set_sensitive(False)
        self.modified = False
    
    #------------------------------------------------------- delete button
    def deletePartCB(self,widget):
        selPath = self.partListView.get_cursor()[0]
        if selPath == None:
            return
        
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete part from import')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete item from bricklist
            self.currList.removePath(selPath)
            self.saveButton.set_sensitive(True)
            self.modified = True
    
    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None,filedlg=None):
        super(ImportBLSet,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.dialogTitle = title
        self.catalog = catalog
        self.counterpartList = dict()
        self.alternatepartList = dict()
        self.modified = False
        self.blXMLfile = filedlg
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)
      
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
        self.openButton = gtk.ToolButton()
        self.openButton.set_stock_id(gtk.STOCK_OPEN)
        self.openButton.set_tooltip_text("Open BrickLink Set Inventory")
        self.openButton.connect('clicked',self.openButtonCB)
        self.toolBar.insert(self.openButton,-1)
        
        self.saveButton = gtk.ToolButton()
        self.saveButton.set_stock_id(gtk.STOCK_SAVE)
        self.saveButton.set_tooltip_text("Save imported part list")
        self.saveButton.set_sensitive(False)
        self.saveButton.connect('clicked',self.saveDBButtonCB)
        self.toolBar.insert(self.saveButton,-1)
        
        #------------- expand ON + draw OFF = next widgets go to left of toolbar
        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Return")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        

        #----------------------------------------------- list store for treeview
        self.currList = BrickListStore.BrickListStore(self.catalog)
        self.partListView = BrickListView.BrickListView(self.currList)
        
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        self.vBox.add(self.listWindow)
        
        self.frame1 = gtk.Frame('Import Options')
        self.vBox.pack_start(self.frame1,False)
        
        self.table = gtk.Table(5,4)
        self.table.set_border_width(2)
        
        self.frame1.add(self.table)
        
        self.exCounterpart = gtk.CheckButton('Exclude Counterparts')
        self.exCounterpart.set_active(True)
        self.exCounterpart.set_alignment(0,0.5)
        self.usePartsOnly = gtk.CheckButton('Use "Parts" type only')
        self.usePartsOnly.set_active(True)
        self.usePartsOnly.set_alignment(0,0.5)
        self.useAlternate = gtk.CheckButton('Use Alternate parts')
        self.useAlternate.set_active(False)
        self.useAlternate.set_alignment(0,0.5)
        self.convertDecor = gtk.CheckButton('Try to convert decorated parts')
        self.convertDecor.set_active(True)
        self.convertDecor.set_alignment(0,0.5)
        self.labCount = gtk.Label('Parts in file: ')
        self.labCount.set_alignment(1,0.5)
        self.partCount = gtk.Entry()
        self.partCount.set_editable(False)
        
        self.table.attach(self.exCounterpart,0,3,0,1)
        self.table.attach(self.usePartsOnly,0,3,1,2)
        self.table.attach(self.useAlternate,0,3,2,3)
        self.table.attach(self.convertDecor,0,3,3,4)
        self.table.attach(self.labCount,0,1,4,5,xoptions=gtk.FILL)
        self.table.attach(self.partCount,1,2,4,5,xoptions=gtk.SHRINK|gtk.FILL)

        self.table.attach(gtk.Label(' '),2,3,4,5)
        
        self.deletePart = gtk.Button('Remove part')
        self.deletePart.connect('clicked',self.deletePartCB)
        self.table.attach(self.deletePart,3,4,1,2,xoptions=gtk.SHRINK)
       
        self.set_default_size(600,400)
        self.vBox.show_all()
        
       

