#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  ImportLDDProject - Tool to import LDD project files
  Copyright (C) 2011 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
#
#===============================================================================



#===============================================================================
# Imports
#===============================================================================


import gtk
import pygtk
import zipfile
import BrickListStore
import BrickListView
from ParseLDDXML import ParseLDDXML

pygtk.require('2.0')



class ImportLDDProject(gtk.Dialog):
    
    
    #----------------------------------------------------------------- callbacks
    
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')

    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

    
    #------------------------------------------------------ open button callback
    def openButtonCB(self,widget):
        response = self.lddFile.run()
        self.lddFile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            lddXMLfile = self.lddFile.get_filename()
            try :
                lddzip = zipfile.ZipFile(lddXMLfile, 'r')
                lddxml = lddzip.open('IMAGE100.LXFML','r')
            except zipfile.BadZipfile:
                try :
                    lddxml = open(lddXMLfile)
                except IOError:
                    msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                        message_format="Cannot read file: "+lddXMLfile)
                    msg.run()
                    msg.hide()
                    msg.destroy()
                    return

            # at this point lddxml contains a "file" object, ready for parsing
            # parse LDD file
            self.partListView.set_sensitive(False)
            currList = BrickListStore.BrickListStore(self.catalog)
            parser = ParseLDDXML(lddxml,currList,self.catalog,self.partCount)
            parser.run()
            self.currList = currList
            self.partListView.set_model(self.currList)
            self.partListView.set_sensitive(True)
            lddxml.close()
            try:
                lddzip.close()
            except IOError:
                pass
            self.saveButton.set_sensitive(True)
            self.modified = True            
                
    #----------------------------------------------- save brick list
    def saveDBButtonCB(self,widget):
        self.currList.write()
        self.saveButton.set_sensitive(False)
        self.modified = False
    
    #------------------------------------------------------- delete button
    def deletePartCB(self,widget):
        selPath = self.partListView.get_cursor()[0]
        if selPath == None:
            return
        
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete part from import')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete item from bricklist
            self.currList.removePath(selPath)
            self.saveButton.set_sensitive(True)
            self.modified = True
    
    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None, filedlg=None):
        super(ImportLDDProject,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.dialogTitle = title
        self.catalog = catalog
        self.modified = False
        self.lddFile = filedlg
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)
      
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
        self.openButton = gtk.ToolButton()
        self.openButton.set_stock_id(gtk.STOCK_OPEN)
        self.openButton.set_tooltip_text("Open LDD project file")
        self.openButton.connect('clicked',self.openButtonCB)
        self.toolBar.insert(self.openButton,-1)
        
        self.saveButton = gtk.ToolButton()
        self.saveButton.set_stock_id(gtk.STOCK_SAVE)
        self.saveButton.set_tooltip_text("Save imported part list")
        self.saveButton.set_sensitive(False)
        self.saveButton.connect('clicked',self.saveDBButtonCB)
        self.toolBar.insert(self.saveButton,-1)
        
        #------------- expand ON + draw OFF = next widgets go to left of toolbar
        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Return")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
        
        #----------------------------------------------- list store for treeview
        self.currList = BrickListStore.BrickListStore(self.catalog)
        self.partListView = BrickListView.BrickListView(self.currList)
        
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        self.vBox.add(self.listWindow)
        
        self.frame1 = gtk.Frame('Import Options')
        self.vBox.pack_start(self.frame1,False)
        
        self.hBox = gtk.HBox()
        self.table = gtk.Table(2,4)
        
        self.hBox.pack_start(self.table,True)
        self.frame1.add(self.hBox)
        
        self.partCountLab = gtk.Label('Bricks in file: ')
        self.partCountLab.set_alignment(0,0.5)
        self.table.attach(self.partCountLab,0,1,1,2,xoptions=gtk.SHRINK)
        self.partCount = gtk.Entry()
        self.partCount.set_editable(False)
        self.table.attach(self.partCount,1,2,1,2,xoptions=gtk.SHRINK)
        self.table.attach(gtk.Label(' '),2,3,1,2)
        
        self.deletePart = gtk.Button('Remove part')
        self.deletePart.connect('clicked',self.deletePartCB)
        self.table.attach(self.deletePart,3,4,1,2,xoptions=gtk.SHRINK)

        self.set_default_size(600,400)
       
        self.vBox.show_all()
        
       

