#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  ImportLDraw - Tool to import LDraw files 
  Copyright (C) 2012 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
#
#===============================================================================



#===============================================================================
# Imports
#===============================================================================


import os.path
import re
import gtk
import pygtk
import BrickListStore
import BrickListView
from BrickException import BrickException


pygtk.require('2.0')


def addPart(part,color,num,partList):
    if partList.has_key((part,color)):
        partList[(part,color)] += num
    else:
        partList[(part,color)] = num     


def expandSubFile(path,file,color,partList):
    try :
        ldrawFile = open(os.path.join(path,file))
    except IOError:
        msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                            message_format="Error: cannot read file '"+file+"'")
        msg.run()
        msg.hide()
        msg.destroy()
        return
    for l in ldrawFile:
        if gtk.events_pending():
            gtk.main_iteration(block=False)
        line = l.split()
        if len(line) > 0 and line[0] == '1':
            try:
                ldrColor = int(line[1])
            except ValueError:
                ldrColor = 0
            if os.path.splitext(line[14])[1].lower() == '.ldr':
                #----- it is a submodel, expand it and add all parts
                expandSubFile(path,line[14],ldrColor,partList)
            else:
                ldrPart = line[14].lower()
                addPart(ldrPart,ldrColor,1,partList)
    ldrawFile.close()        

def expandSubModel(partList,firstModel,subModels,color,multiplier):
    #------------------------------ expands a model inserting submodel parts
    #------------------------------------------------- function is recursive
    for part in firstModel.iteritems():
        p = part[0][0]
        c = part[0][1]
        if c == 16:
            c = color
        n = part[1] * multiplier
#        if (os.path.splitext(p)[1].lower() == '.ldr'):
        if subModels.has_key(p):
            #---------------------------------------------- it is a submodel
            expandSubModel(partList,subModels[p],subModels,c,n)
        else:
            addPart(p,c,n,partList)


class ImportLDraw(gtk.Dialog):
    
    __decorated = (118,535,736,643,642,440,538,738,132,83,728,461,33,741,740,39,578,84,579)
    
    #----------------------------------------------------------------- callbacks
    
   
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')

    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Imported part list not saved, lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

    
    #------------------------------------------------------ open button callback
    def openButtonCB(self,widget):
        response = self.ldrFile.run()
        self.ldrFile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                ldrawFilename = self.ldrFile.get_filename()
                if os.path.splitext(ldrawFilename.lower())[1] == '.mpd':
                    #----------------------------------------- it is an MPD file
                    mpdFlag = True
                else:
                    mpdFlag = False
                ldrawFile = open(ldrawFilename)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            self.partCount.set_text('Reading main file...')
            modelDir = os.path.dirname(ldrawFilename)
            
            self.partListView.set_sensitive(False)
            
            if mpdFlag:
                #-------- if it is a Multi Part Document, reads sub-models first
                partList = dict()
                subModel = dict()
                subModelName = ''
                firstModel = None
                inSubModel = False
                for l in ldrawFile:
                    if gtk.events_pending():
                        gtk.main_iteration(block=False)
                    ldrLine = l.split()
                    if len(ldrLine) > 1 and ldrLine[0] == '0':
                        if ldrLine[1].upper() == 'FILE':
                            if inSubModel:
                                #--------- we are in submodel, this is next submodel
                                subModel[subModelName] = partList
                                partList = dict()
                            # filename with spaces!!! (damn...)
                            f = re.match('\s*0\s+file\s+(.*)\s*\Z', l, re.IGNORECASE)
                            if f == None:
                                #------------------- unable to read filename
                                print "No match file name in mpd " + l
                                continue
                            try:
                                subModelName = f.group(1).strip().lower()
                            except IndexError:
                                #------------------- unable to read filename
                                print "no file name in mpd " + l
                                continue
                            inSubModel = True
                            #print "File: "+subModelName
                            if firstModel == None:
                                firstModel = subModelName
                        elif ldrLine[1].upper() == 'NOFILE' and inSubModel:
                            #--------- we are in submodel, this is next submodel
                            subModel[subModelName] = partList
                            partList = dict()
                            inSubModel = False
                    elif len(ldrLine) > 1 and ldrLine[0] == '1' and inSubModel:
                        try:
                            ldrColor = int(ldrLine[1])
                        except ValueError:
                            ldrColor = 0
                        # done - filename with spaces!!!
                        f = re.match('\s*1\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+(.*)\s*\Z',
                                     l, re.IGNORECASE)
                        if f == None:
                            #------------------- unable to read filename
                            print "No file name in mpd " + l
                            continue
                        try:
                            p = f.group(1).strip()
                        except IndexError:
                            #------------------- unable to read filename
                            print "no file name in mpd " + l
                            continue
                        ldrPart = p.lower()
                        addPart(ldrPart,ldrColor,1,partList)
                if inSubModel:
                    subModel[subModelName] = partList
                #print subModel.keys()
                firstModel = subModel[firstModel]
                partList = dict()
                expandSubModel(partList,firstModel,subModel,0,1)
            else:
                #------------------------------------- it is a standard ldr file
                partList = dict()
                inSubModel = False
                for l in ldrawFile:
                    #-------------------------------------------- let the GUI updates itself
                    if gtk.events_pending():
                        gtk.main_iteration(block=False)
                    ldrLine = l.split()
                    if len(ldrLine) > 0 and ldrLine[0] == '1':
                        try:
                            ldrColor = int(ldrLine[1])
                        except ValueError:
                            ldrColor = 0
                        f = re.match('\s*1\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+[-.0-9]+\s+(.*)\s*\Z',
                                     l, re.IGNORECASE)
                        if f == None:
                            #------------------- unable to read filename
                            print "No file name in mpd " + l
                            continue
                        try:
                            subModelName = f.group(1).strip()
                        except IndexError:
                            #------------------- unable to read filename
                            print "no file name in mpd " + l
                            continue
                        if os.path.splitext(subModelName)[1].lower() == '.ldr':
                            #----- it is a submodel, expand it and add all parts
                            self.partCount.set_text("Reading '"+subModelName+"'")
                            expandSubFile(modelDir,subModelName,ldrColor,partList)
                        else:
                            ldrPart = subModelName.lower()
                            addPart(ldrPart,ldrColor,1,partList)
            ldrawFile.close()
            
            partID = 1
            currList = BrickListStore.BrickListStore(self.catalog)
            for p in partList.iteritems():
                while gtk.events_pending():
                    gtk.main_iteration(block=False)
                decorated = False
                ldrPart = p[0][0]
                ldrColor = p[0][1]
                qty = p[1]
                #===============================================================
                # part conversion
                #===============================================================
                #----------------------- check if exists a direct mapping to LDD
                result = self.catalog.partLdraw2Ldd(ldrPart)
                lddid = '-'
                if len(result) == 1:
                    #---------------------------------- we have a direct mapping
                    lddid = result[0][0]
                    r1 = self.catalog.partLdd2Bl(lddid)
                    if len(r1) == 0:
                        # part not in BL catalog (it can happen)
                        blid = ldrPart
                    elif len(r1) == 1:
                        blid = r1[0][0]
                    else:
                        #---------------- at the moment we consider only 1:1 mapping
                        raise BrickException('Duplicated part in BrickLink '+str(r1))
                    #--------------------------------- gets bl part informations
                    r1 = self.catalog.partBL(blid)
                    if len(r1) == 1:
                        blid = r1[0][0]
                        descr = r1[0][1]
                        blcategory = r1[0][2]
                    else:
                        blid = ldrPart
                        descr = 'no match found - unknown part'
                        blcategory = -1
                elif len(result) == 0:
                    blid = ldrPart
                    descr = 'no match found - unknown part'
                    blcategory = -1
                    decorated = False
                    lddid = '-'
                    result = self.catalog.getLDrawPart(ldrPart)
                    if len(result) == 1:
                        ldr,decorated = self.catalog.detectLDRdecoration(ldrPart)
                        descr = result[0][1]
                        if decorated and self.convertDecor.get_active():
                            r = self.catalog.partLdraw2Ldd(ldr+'.dat')
                            if len(r) == 1:
                                lddid = r[0][0]
#                                r1 = self.catalog.partLdd2Bl(lddid)
#                                if len(r1) == 0:
#                                    # part not in BL catalog (it can happen)
#                                    pass
#                                elif len(r1) == 1:
#                                    blid = r1[0][0]
#                                else:
#                                    #---------------- at the moment we consider only 1:1 mapping
#                                    raise BrickException('Duplicated part in BrickLink '+str(r1))
#                                #--------------------------------- gets bl part informations
#                                r1 = self.catalog.partBL(blid)
#                                if len(r1) == 1:
#                                    blcategory = r1[0][2]
#                                    descr = r1[0][1]
                else:
                    raise BrickException('Duplicated LDraw->LDD mapping in table, part: '+ldrPart)
                #===============================================================
                # color conversion
                #===============================================================
                result = self.catalog.colorLdraw2Ldd(ldrColor)
                #---------------------------------------------- checks for color
                if len(result) == 0:
                    lddcolorid = 0
                    lddcolor = None
                    lddcolorname = 'no match'
                    blcolorid = None
                else:
                    lddcolorid = result[0][0]
                    lddcolor = result[0][2]
                    lddcolorname = result[0][3]
                    blcolorid = result[0][1]
                    
                result = self.catalog.colorBL(blcolorid)
                if len(result) == 0:
                    blcolor = None
                    blcolorname = 'unknown'
                else:
                    blcolor = result[0][1]
                    blcolorname = result[0][0]
                
                currList.append(row=('P',    # type
                                    blid,    # blpartid
                                    qty,    # qty
                                    blcolorid,    # blcolorid
                                    False,    # extraPart
                                    False,    # counterpart
                                    descr,      # description (from BL)
                                    lddid,      # LDD part id
                                    lddcolor,   # rgb color from LDD
                                    lddcolorname,   # color name from LDD
                                    blcolor,    # rgb color from BL
                                    blcolorname,    # name from BL
                                    blcategory, # category in BLparts
                                    partID,      # unique ID for part
                                    lddcolorid,     # ldd color ID
                                    decorated,       # decorated part?
                                    False  # has alternatePart if != '0'
                                    ))
                if (partID % 20) == 0:
                    self.partCount.set_text(str(partID))
                partID += 1

            self.partCount.set_text(str(partID-1))
            self.currList = currList
            self.partListView.set_model(self.currList)
            self.partListView.set_sensitive(True)
            self.saveButton.set_sensitive(True)
            self.modified = True            
                
    #----------------------------------------------- save brick list
    def saveDBButtonCB(self,widget):
        self.currList.write()
        self.saveButton.set_sensitive(False)
        self.modified = False
    
    #------------------------------------------------------- delete button
    def deletePartCB(self,widget):
        selPath = self.partListView.get_cursor()[0]
        if selPath == None:
            return
        
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete part from import')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete item from bricklist
            self.currList.removePath(selPath)
            self.saveButton.set_sensitive(True)
            self.modified = True
    
    #-------------------------------------------------------- interface building
    def __init__(self,title='Import LDraw project', parent=None, flags=gtk.DIALOG_MODAL, buttons=None, catalog=None, filedlg=None):
        super(ImportLDraw,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.modified = False
        self.ldrFile = filedlg
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)
      
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
        self.openButton = gtk.ToolButton()
        self.openButton.set_stock_id(gtk.STOCK_OPEN)
        self.openButton.set_tooltip_text("Open LDraw project file")
        self.openButton
        self.openButton.connect('clicked',self.openButtonCB)
        self.toolBar.insert(self.openButton,-1)
        
        self.saveButton = gtk.ToolButton()
        self.saveButton.set_stock_id(gtk.STOCK_SAVE)
        self.saveButton.set_tooltip_text("Save imported part list")
        self.saveButton.set_sensitive(False)
        self.saveButton.connect('clicked',self.saveDBButtonCB)
        self.toolBar.insert(self.saveButton,-1)
        
        #------------- expand ON + draw OFF = next widgets go to left of toolbar
        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Return")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
        #----------------------------------------------- list store for treeview
        self.currList = BrickListStore.BrickListStore(self.catalog)
        self.partListView = BrickListView.BrickListView(self.currList)
        
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        self.vBox.add(self.listWindow)
        
        self.frame1 = gtk.Frame('Import Options')
        self.vBox.pack_start(self.frame1,False)
        
        self.table = gtk.Table(5,4)
        self.table.set_border_width(2)
        
        self.frame1.add(self.table)
      
        self.convertDecor = gtk.CheckButton('Try to convert decorated parts ')
        self.convertDecor.set_active(True)
        self.convertDecor.set_alignment(0,0.5)
        self.labCount = gtk.Label('Parts in file: ')
        self.labCount.set_alignment(1,0.5)
        self.partCount = gtk.Entry()
        self.partCount.set_editable(False)
        
        self.table.attach(self.convertDecor,0,3,0,1)
        self.table.attach(self.labCount,0,1,1,2,xoptions=gtk.FILL)
        self.table.attach(self.partCount,1,2,1,2,xoptions=gtk.SHRINK|gtk.FILL)

        self.table.attach(gtk.Label(' '),2,3,1,2)
        
        self.deletePart = gtk.Button('Remove part')
        self.deletePart.connect('clicked',self.deletePartCB)
        self.table.attach(self.deletePart,3,4,1,2,xoptions=gtk.SHRINK)
       
        self.set_default_size(600,400)
        self.vBox.show_all()
        
       

