'''
Created on 17/nov/2011

@author: Mario Pascucci
'''

import gtk
import gobject
import os
import sqlite3


class LCEditDialog(gtk.Dialog):

    def __init__(self,title=None, parent=None, flags=0, 
                 buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT)):
        super(LCEditDialog, self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
#        self.connect('response',self.responseCB)
        
        self.vBox = self.get_content_area()
        
        self.table = gtk.Table(rows=5, columns=2)
        
        self.lab1 = gtk.Label('LDD Main part ID')
        self.lab1.set_alignment(1,0.5)
        self.table.attach(self.lab1,0,1,0,1,xoptions=gtk.FILL,xpadding=3)
        
        self.lddid = gtk.Entry()
        self.table.attach(self.lddid,1,2,0,1)
        
        self.lab4 = gtk.Label('Sub-part ID for color')
        self.lab4.set_alignment(1,0.5)
        self.table.attach(self.lab4,0,1,1,2,xoptions=gtk.FILL,xpadding=3)
        
        self.colPart = gtk.Entry()
        self.table.attach(self.colPart,1,2,1,2)
        self.lab5 = gtk.Label('Index for color')
        self.lab5.set_alignment(1,0.5)
        self.table.attach(self.lab5,2,3,1,2,xoptions=gtk.FILL,xpadding=3)
        self.colIdx = gtk.Entry()
        self.table.attach(self.colIdx,3,4,1,2)
        
        self.lab6 = gtk.Label('Sub-part ID for decoration')
        self.lab6.set_alignment(1,0.5)
        self.table.attach(self.lab6,0,1,2,3,xoptions=gtk.FILL,xpadding=3)
        self.decorPart = gtk.Entry()
        self.table.attach(self.decorPart,1,2,2,3)
        self.lab7 = gtk.Label('Index for decoration')
        self.lab7.set_alignment(1,0.5)
        self.table.attach(self.lab7,2,3,2,3,xoptions=gtk.FILL,xpadding=3)
        self.decorIdx = gtk.Entry()
        self.table.attach(self.decorIdx,3,4,2,3)

        self.lab8 = gtk.Label('Notes')
        self.lab8.set_alignment(1,0.5)
        self.table.attach(self.lab8,0,1,3,4,xoptions=gtk.FILL,xpadding=3)
        
        self.notes = gtk.Entry()
        self.notes.set_size_request(300,-1)
        self.table.attach(self.notes,1,4,3,4)
        
        self.vBox.pack_start(self.table,False)
        self.vBox.show_all()


class LDDCompList(gtk.ListStore):

    def read(self):
        #------------------------------------------------- query all parts
        c = self.catalog.readLDDComposite()
        
        #------------------------------------ iterates through query results
        self.clear()
        for r in c:
            self.append(row=(r[0],
                             r[1],
                             r[2],
                             r[3],
                             r[4],
                             r[5] if r[5] != None else ''
                             )
                        )
    
    def write(self):
        self.catalog.cleanLDDCompositeTable()
        for row in self:
            self.catalog.insLDDComposite(row[0],row[1],row[2],row[3],row[4],row[5].decode('utf-8'))
        self.catalog.commit()
    
    
    def __init__(self,catalog=None):
        super(LDDCompList, self).__init__(gobject.TYPE_INT,     # lddid
                                          gobject.TYPE_INT,     # color subpart ID
                                          gobject.TYPE_INT,     # decoration subpart id
                                          gobject.TYPE_INT,     # color index
                                          gobject.TYPE_INT,     # decoration index
                                          gobject.TYPE_STRING   # notes
                                          )
        self.catalog = catalog


class LDDCompView(gtk.TreeView):

    def setSearchCB(self,widget,column):
        self.set_search_column(column)

    def __init__(self,listStore):
        super(LDDCompView, self).__init__(listStore)

        self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_HORIZONTAL)

        #---------------------- column and cell renderer definition for treeview
        self.crtLDDID = gtk.CellRendererText()
        self.tvcLDDID = gtk.TreeViewColumn('LDD ID',self.crtLDDID)
        self.tvcLDDID.set_resizable(True)
        
        self.crtColSubID = gtk.CellRendererText()
        self.tvcColSubID = gtk.TreeViewColumn('Color SubID',self.crtColSubID)
        self.tvcColSubID.set_resizable(True)
        
        self.crtColIdx = gtk.CellRendererText()
        self.tvcColIdx = gtk.TreeViewColumn('Color idx',self.crtColIdx)
        self.tvcColIdx.set_resizable(True)
        
        self.crtDecSubID = gtk.CellRendererText()
        self.tvcDecSubID = gtk.TreeViewColumn('Decor SubID',self.crtDecSubID)
        self.tvcDecSubID.set_resizable(True)
        
        self.crtDecIdx = gtk.CellRendererText()
        self.tvcDecIdx = gtk.TreeViewColumn('Decor idx',self.crtDecIdx)
        self.tvcDecIdx.set_resizable(True)
        
        self.crtnotes = gtk.CellRendererText()
        self.tvcnotes = gtk.TreeViewColumn('Notes & comments',self.crtnotes)
        self.tvcnotes.set_resizable(True)
        
        self.append_column(self.tvcLDDID)
        self.append_column(self.tvcColSubID)
        self.append_column(self.tvcColIdx)
        self.append_column(self.tvcDecSubID)
        self.append_column(self.tvcDecIdx)
        self.append_column(self.tvcnotes)
        
        self.tvcLDDID.set_attributes(self.crtLDDID,text=0)
        self.tvcColSubID.set_attributes(self.crtColSubID,text=1)
        self.tvcColIdx.set_attributes(self.crtColIdx,text=3)
        self.tvcDecSubID.set_attributes(self.crtDecSubID,text=2)
        self.tvcDecIdx.set_attributes(self.crtDecIdx,text=4)
        self.tvcnotes.set_attributes(self.crtnotes,text=5)
        
        self.set_search_column(0)
        self.tvcColSubID.set_sort_column_id(1)
        self.tvcDecSubID.set_sort_column_id(2)
        self.tvcnotes.set_sort_column_id(5)
        self.tvcLDDID.set_sort_column_id(0)
        self.tvcLDDID.connect('clicked',self.setSearchCB,0)
        self.tvcColSubID.connect('clicked',self.setSearchCB,1)
        self.tvcDecSubID.connect('clicked',self.setSearchCB,2)
        self.tvcnotes.connect('clicked',self.setSearchCB,5)



class LDDComposite(gtk.Dialog):
    '''
    classdocs
    '''
    
    def readCatalog(self):
        self.compList.read()
        self.compList.set_sort_column_id(0,gtk.SORT_ASCENDING)
        
        self.saveDBButton.set_sensitive(False)
        self.modified = False

    #----------------------------------------------------------------- callbacks
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')
        
    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Composite parts database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False    
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Composite parts database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

    #--------------------------------------------------- import button callback
    def importDBButtonCB(self,widget):
        #----------------------------- part DB modified, confirm open a new one
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Composite part database modified, ignore and lose changes?")
            msg.set_title('Confirm open')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        response = self.openDialog.run()
        self.openDialog.hide()
        if (response == gtk.RESPONSE_ACCEPT):
        # file selected, OK clicked
            partmapfile = self.openDialog.get_filenames()[0]
            #------------------------- reads database and compile list of parts
            conn = sqlite3.connect(partmapfile)
            c = conn.cursor()
            
            #------------------------------------------------- query composite part table
            c.execute('select lddid,colorsubid,decorsubid,coloridx,decoridx,notes from lddcomposite')
            
            #------------------------------------ iterates through query results
            self.compList.clear()
            for part in c:
                #--------------------------------- extract maximum key value from db
                self.compList.append(row=(part[0],            
                                          part[1],   # BL part ID
                                          part[2],
                                          part[3],
                                          part[4],
                                          part[5] if part[5] != None else ''    # notes
                                        ))
            conn.close()
            self.saveDBButton.set_sensitive(True)
            self.modified = True
            
    #----------------------------------------------- save partmap database file
    def saveDBButtonCB(self,widget):
        self.compList.write()
        self.saveDBButton.set_sensitive(False)
        self.modified = False
    
    #---------------------------------------- save partmap database as new file
    def exportDBButtonCB(self,widget):
        response = self.saveDialog.run()
        self.saveDialog.hide()
        if response == gtk.RESPONSE_ACCEPT:
            partfile = self.saveDialog.get_filename()
            if os.path.splitext(partfile)[1] == '':
                partfile += '.sq3'
            if os.path.exists(partfile):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                                message_format="File '" +partfile+"' exists, overwrite?")
                msg.set_title('Confirm overwrite')
                response = msg.run()
                msg.hide()
                msg.destroy()
                if response != gtk.RESPONSE_YES:
                    return                
        else:
            return
                
        conn = sqlite3.connect(partfile)        
        c = conn.cursor()

        # re-create table with part map list
        c.execute('drop table if exists lddcomposite')
        
        c.execute('create table if not exists lddcomposite (lddid integer, colorsubid integer,'+
                  'decorsubid integer,coloridx integer,decoridx integer,notes text)')
        
        for row in self.compList:
            c.execute('insert into lddcomposite (lddid,colorsubid,decorsubid,coloridx,decoridx,notes) '+
                      'values(?,?,?,?,?,?)',(row[0],row[1],row[2],row[3],row[4],row[5].decode('utf-8'))
                      )
        conn.commit()
        conn.close()
        
    #------------------------------------------------- new part button callback
    def newButtonCB(self,widget):
        dlg = LCEditDialog(title="New composite part",parent=self)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            # save edits
            try:
                lddid = int(dlg.lddid.get_text())
                if lddid == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                colPart = int(dlg.colPart.get_text())
                if colPart == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                colIdx = int(dlg.colIdx.get_text())
                if colIdx < 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Index must be numeric and not less than 0")
                msg.set_title('Wrong index')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try: 
                decorPart = int(dlg.decorPart.get_text())
                if decorPart == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                decorIdx = int(dlg.decorIdx.get_text())
                if decorIdx < 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Index must be numeric and not less than 0")
                msg.set_title('Wrong index')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            for r in self.compList:
                if r[0] == lddid:
                    msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL)
                    msg.set_markup("Duplicated composite part\nLDD ID: <b>"+str(lddid)+"</b>")
                    msg.set_title('Error')
                    msg.run()
                    msg.hide()
                    msg.destroy()
                    return
            self.compList.append(row=(lddid,colPart,decorPart,colIdx,decorIdx,dlg.notes.get_text()))
            self.saveDBButton.set_sensitive(True)
            self.modified = True
            
    
    #------------------------------------------------------- delete part button
    def deleteButtonCB(self,widget):
        selPath = self.compView.get_cursor()
        if selPath[0] == None:
            return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete composite part')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete part from DB
            selRow = self.compList.get_iter(selPath[0])
            self.compList.remove(selRow)
            self.saveDBButton.set_sensitive(True)
            self.modified = True
    
    def editButtonCB(self,widget):
        selPath = self.compView.get_cursor()
        if selPath[0] == None:
            return
        selRow = self.compList.get_iter(selPath[0])
        lddid,colPart,decorPart,colIdx,decorIdx,notes = self.compList.get(selRow,0,1,2,3,4,5)        
        dlg = LCEditDialog(title='Edit composite part',parent=self)
        dlg.lddid.set_text(str(lddid))
        dlg.colPart.set_text(str(colPart))
        dlg.colIdx.set_text(str(colIdx))
        dlg.decorPart.set_text(str(decorPart))
        dlg.decorIdx.set_text(str(decorIdx))
        dlg.notes.set_text(notes)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            # save edits
            # save edits
            try:
                lddid = int(dlg.lddid.get_text())
                if lddid == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                colPart = int(dlg.colPart.get_text())
                if colPart == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                colIdx = int(dlg.colIdx.get_text())
                if colIdx < 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Index must be numeric and not less than 0")
                msg.set_title('Wrong index')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try: 
                decorPart = int(dlg.decorPart.get_text())
                if decorPart == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD IDs must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            try:
                decorIdx = int(dlg.decorIdx.get_text())
                if decorIdx < 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                                message_format="Index must be numeric and not less than 0")
                msg.set_title('Wrong index')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            epath = self.compList.get_string_from_iter(selRow)
            for r in self.compList:
                if r[0] == lddid and self.compList.get_string_from_iter(r.iter) != epath:
                    msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL)
                    msg.set_markup("Duplicated composite part\nLDD ID: <b>"+str(lddid)+"</b>")
                    msg.set_title('Error')
                    msg.run()
                    msg.hide()
                    msg.destroy()
                    return                    
            self.compList[selPath[0]] = (lddid,
                                         colPart,
                                         decorPart,
                                         colIdx,
                                         decorIdx,
                                         dlg.notes.get_text()
                                         )
            self.modified = True
            self.saveDBButton.set_sensitive(True)
            

    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None):
        super(LDDComposite,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.dialogTitle = title
        self.modified = False
        
        #----------------------------------------------------------- main window
        self.connect('response',self.responseCB)
        self.connect('delete-event',self.closeCB)
         
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
        self.importDBButton = gtk.ToolButton()
        self.importDBButton.set_stock_id(gtk.STOCK_OPEN)
        self.importDBButton.set_tooltip_text("Import composite parts database")
        self.importDBButton.connect('clicked',self.importDBButtonCB)
        self.toolBar.insert(self.importDBButton,-1)
        
        self.saveDBButton = gtk.ToolButton()
        self.saveDBButton.set_stock_id(gtk.STOCK_SAVE)
        self.saveDBButton.set_tooltip_text("Save composite parts database in main catalog")
        self.saveDBButton.set_sensitive(False)
        self.saveDBButton.connect('clicked',self.saveDBButtonCB)
        self.toolBar.insert(self.saveDBButton,-1)
        
        self.exportDBButton = gtk.ToolButton()
        self.exportDBButton.set_stock_id(gtk.STOCK_SAVE_AS)
        self.exportDBButton.set_tooltip_text("Export composite parts database as new file")
        self.exportDBButton.connect('clicked',self.exportDBButtonCB)
        self.toolBar.insert(self.exportDBButton,-1)        

        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Quit to main")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
        #-------------------------------------------- various dialogs and popups
        self.openDialog = gtk.FileChooserDialog("Select composite parts file",self.window,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        
        self.saveDialog = gtk.FileChooserDialog("Save as composite parts file",self.window,
            action=gtk.FILE_CHOOSER_ACTION_SAVE,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        #self.saveDialog.set_do_overwrite_confirmation(True)
        
        #------------------------------------ add a filter for SQLite3 files
        sqliteFilter = gtk.FileFilter()
        sqliteFilter.add_pattern("*.sq3")
        sqliteFilter.set_name("SQlite database files")
        self.openDialog.add_filter(sqliteFilter)
        self.saveDialog.add_filter(sqliteFilter)
        #-------------------------------------- add a filter for any (all) files
        allFilter = gtk.FileFilter()
        allFilter.add_pattern("*")
        allFilter.set_name("All files")
        self.openDialog.add_filter(allFilter)
        self.saveDialog.add_filter(allFilter)
        
        #----------------------------------------------- list store for treeview
        self.compList = LDDCompList(self.catalog) 
        
        self.compView = LDDCompView(self.compList)
        
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.compView)
        self.vBox.add(self.listWindow)
        
        self.hButtonBox = gtk.HButtonBox()
        self.hButtonBox.set_layout(gtk.BUTTONBOX_END)
        
        self.newButton = gtk.Button(stock=gtk.STOCK_NEW)
        self.newButton.connect('clicked',self.newButtonCB)
        self.editButton = gtk.Button(stock=gtk.STOCK_EDIT)
        self.editButton.connect('clicked',self.editButtonCB)
        self.deleteButton = gtk.Button(stock=gtk.STOCK_DELETE)
        self.deleteButton.connect('clicked',self.deleteButtonCB)
        
        self.hButtonBox.pack_start(self.newButton,False)
        self.hButtonBox.pack_start(self.editButton,False)
        self.hButtonBox.pack_start(self.deleteButton,False)
        
        self.vBox.pack_end(self.hButtonBox,False,False)
        
        self.readCatalog()
        self.set_default_size(500,300)
        
        self.vBox.show_all()
        
