#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  LDrawMaintDialog - utility to edit LDD<->LDraw parts mapping DB
  Copyright (C) 2011 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
#===============================================================================



#===============================================================================
# to create ldrawmap table from LDraw part library
# - export bricklink part map library on a new file
# - create a part list file from ldraw parts:
#    rm /home/mario/ldr.txt ; for i in *.dat ; do b=`head -q -n1 "$i" | cut -c 3- | tr -d '\r'`; echo "$i|$b" >> /home/mario/ldr.txt ; done
# - create a table for ldraw parts:
#    create table ldr (id text, descr text);
# - fill ldr table:
#    import .import ldr.txt ldr
# - rename partmap table
#    alter table partmap rename to pm;
# - create a table for distinct ldd parts
#    create table partmap as select distinct lddid from pm;
# - create table for ldrawmap parts:
#    create table ldrawmap as select coalesce(p.lddid,(select lddid from partmap where lddid=(cast (l.id as integer)))) as lddid,id as ldrawid,0 as dattoldd,0 as lddtodat,coalesce(l.descr,(select notes from pm where lddid=p.lddid)) as notes,0 as lmod, 0 as lastmod from ldr as l left join partmap as p on (l.id = p.lddid);
# - add parts in LDD that doesn't exists in LDraw:
#    insert into ldrawmap select p.lddid,l.ldrawid,0,0,p.notes,0,0  from pm as p left outer join ldrawmap as l on (l.lddid=p.lddid) where l.lddid is null;
# - set correct flags:
#    update ldrawmap set lddtodat=1 where (cast(lddid as text)||'.dat')=ldrawid;
#    update ldrawmap set dattoldd=1 where ldrawid is not null and lddid is not null
# - clean up some parts (parts not in LDD)
#    delete from ldrawmap where notes like '%sticker%' and lddid is null;
# - deletes xNNN parts
#    delete from ldrawmap where ldrawid like 'x%';
# - adds parts from bl maps when ldrawid match blid on ldraw parts without lddid matching
#    create table x as as select p.lddid as ld,l.ldrawid as ldr from ldrawmap as l join pm as p on(l.ldrawid=(p.blid||'.dat')) where l.lddid is null and ldrawid=l.ldrawid and l.ldrawid in (select id from ldr);
#    update ldrawmap set lddid=(select ld from x where ldrawid=ldr),lddtodat=1 where lddid is null and ldrawid in (select l.ldrawid from ldrawmap as l join pm as p on(l.ldrawid=(p.blid||'.dat')) where l.lddid is null and l.ldrawid in (select id from ldr)); 
# - checks for duplicates in table:
#    create table dup as select count(*) as conto, ldrawid from ldrawmap group by ldrawid;
#    select * from dup where conto > 1;  >>>>> 0 rows
#    create table dup2 as select count(*) as conto, lddid,lddtodat from ldrawmap group by lddid,lddtodat;
#    select * from dup2 where conto > 1 and lddtodat;  >>>> 0 rows
# - checks duplicates for lddtodat flag
#
#
#
# to update part list when LDraw release new part list:
# - create a file with list of parts from new parts (as for previous ops)
# - import in a table "ldr" with id and descr fields
# - deletes unwanted parts:
#    delete from ldr where descr like 'Sticker %';
#    delete from ldr where descr like 'Duplo %';
#    delete from ldr where descr like 'Znap %';
# - insert into ldrawmap unlisted parts
#   insert into ldrawmap  select null as lddid,u.id as ldrawid,0 as dattoldd,0 as lddtodat,u.descr as notes,0 as lmod,0 as lastmod from ldr as u left join ldrawmap as l on (u.id=l.ldrawid) where u.id is not null and l.ldrawid is null;

#===============================================================================



#===============================================================================
# Imports
#===============================================================================

#import os.path
import time
#import sqlite3
import gtk
import gobject
import pygtk
from LDRPartSearch import LDRPartSearch


pygtk.require('2.0')



class PLEditDialog(gtk.Dialog):
    
    def searchPartCB(self,widget):
        dlg = LDRPartSearch(parent=self,catalog=self.catalog,imgCache=self.imgCache)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            c = dlg.resview.get_cursor()[0]
            if c == None:
                return
            i = dlg.reslist.get_iter(c)
            self.ldrawid.set_text(dlg.reslist.get(i,0)[0])
            self.notes.set_text(dlg.reslist.get(i,1)[0])

    def __init__(self,title=None, parent=None, flags=0, 
                 buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT),catalog=None,imgCache=None):
        super(PLEditDialog, self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.imgCache = imgCache
        
        self.vBox = self.get_content_area()
        
        self.table = gtk.Table(rows=5, columns=3)
        
        self.lab1 = gtk.Label('LDD design ID')
        self.lab1.set_alignment(1,0.5)
        self.table.attach(self.lab1,0,1,0,1,xoptions=gtk.FILL,xpadding=3)
        
        self.lddid = gtk.Entry()
        self.table.attach(self.lddid,1,3,0,1)
        
        self.lab4 = gtk.Label('LDraw ID')
        self.lab4.set_alignment(1,0.5)
        self.table.attach(self.lab4,0,1,1,2,xoptions=gtk.FILL,xpadding=3)
        
        self.ldrawid = gtk.Entry()
        self.table.attach(self.ldrawid,1,2,1,2)
        
        self.searchPart = gtk.Button('Search...')
        self.table.attach(self.searchPart,2,3,1,2)
        self.searchPart.connect('clicked',self.searchPartCB)
                
        self.lab8 = gtk.Label('Notes')
        self.lab8.set_alignment(1,0.5)
        self.table.attach(self.lab8,0,1,2,3,xoptions=gtk.FILL,xpadding=3)
        
        self.notes = gtk.Entry()
        self.notes.set_size_request(300,-1)
        self.table.attach(self.notes,1,3,2,3)
        
        lab1 = gtk.Label('Mapping direction:')
        lab1.set_alignment(1,0.5)
        self.table.attach(lab1,0,1,3,5)
        
        self.dat2ldd = gtk.CheckButton('LDraw -> LDD')
        self.table.attach(self.dat2ldd,1,2,3,4)
        self.ldd2dat = gtk.CheckButton('LDD -> LDraw')
        self.table.attach(self.ldd2dat,1,2,4,5)
        
        self.vBox.pack_start(self.table,False)
        self.vBox.show_all()
        

class PartList(gtk.ListStore):
    
    def read(self):
        #------------------------------------------------- query all parts
        c = self.catalog.getLDrawParts(self.qfilter)
        
        #------------------------------------ iterates through query results
        self.clear()
        
        for r in c:
            self.append(row=(r[0] if r[0] != None else 0,
                             r[1] if r[1] != None else '',
                             r[2],
                             r[3],
                             r[4] if r[4] != None else '',
                             r[5],
                             r[6] if r[6] != None else 0,
                             r[7]
                             ))
        self.set_sort_column_id(0,gtk.SORT_ASCENDING)
        

    def write(self):
        self.catalog.cleanLDrawmapTable()
        for row in self:
            self.catalog.insertLDrawPartMap(row[0],row[1],row[2],row[3],row[4].decode('utf-8'),row[5],row[6],row[7])
        self.catalog.commit()

    def setFilter(self,q):
        if q == self.qfilter:
            return
        self.qfilter = q
        self.read()
    
    def __init__(self,catalog=None):
        super(PartList, self).__init__(gobject.TYPE_INT,     # lddid
                                       gobject.TYPE_STRING,  # ldrawid
                                       gobject.TYPE_BOOLEAN, # dat to ldd
                                       gobject.TYPE_BOOLEAN, # ldd to dat
                                       gobject.TYPE_STRING,  # notes
                                       gobject.TYPE_BOOLEAN, # locally modified
                                       gobject.TYPE_INT,     # last modified
                                       gobject.TYPE_INT      # rowid
                                       )
        self.qfilter = None
        self.catalog = catalog

class LDrawListView(gtk.TreeView):

    def setSearchCB(self,widget,column):
        self.set_search_column(column)

    def __init__(self,listStore):
        super(LDrawListView, self).__init__(listStore)

        self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_HORIZONTAL)
        sel = self.get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)

        #---------------------- column and cell renderer definition for treeview
        self.crtLDDID = gtk.CellRendererText()
        self.tvcLDDID = gtk.TreeViewColumn('LDD ID',self.crtLDDID)
        self.tvcLDDID.set_resizable(True)
        
        self.renderLdrId = gtk.CellRendererText()
        self.tvcolLdrId = gtk.TreeViewColumn('LDraw ID',self.renderLdrId)
        self.tvcolLdrId.set_resizable(True)
        
        self.crtgDAT2LDD = gtk.CellRendererToggle()
        self.tvcDAT2LDD = gtk.TreeViewColumn('Ldr->LDD',self.crtgDAT2LDD)
        self.tvcDAT2LDD.set_resizable(True)

        self.crtgLDD2DAT = gtk.CellRendererToggle()
        self.tvcLDD2DAT = gtk.TreeViewColumn('LDD->Ldr',self.crtgLDD2DAT)
        self.tvcLDD2DAT.set_resizable(True)
        
        self.rendernotes = gtk.CellRendererText()
        self.tvcolnotes = gtk.TreeViewColumn('Notes & comments',self.rendernotes)
        self.tvcolnotes.set_resizable(True)

        self.append_column(self.tvcLDDID)
        self.append_column(self.tvcolLdrId)
        self.append_column(self.tvcDAT2LDD)
        self.append_column(self.tvcLDD2DAT)                   
        self.append_column(self.tvcolnotes)
        self.set_expander_column(self.tvcolnotes)
        self.tvcLDDID.set_attributes(self.crtLDDID,text=0)
        self.tvcolLdrId.set_attributes(self.renderLdrId,text=1)
        self.tvcDAT2LDD.set_attributes(self.crtgDAT2LDD,active=2)
        self.tvcLDD2DAT.set_attributes(self.crtgLDD2DAT,active=3)
        self.tvcolnotes.set_attributes(self.rendernotes,text=4)
        self.set_search_column(0)
        self.tvcLDDID.set_sort_column_id(0)
        self.tvcLDDID.connect('clicked',self.setSearchCB,0)
        self.tvcolLdrId.set_sort_column_id(1)
        self.tvcolLdrId.connect('clicked',self.setSearchCB,1)
        self.tvcolnotes.set_sort_column_id(4)
        self.tvcolnotes.connect('clicked',self.setSearchCB,4)
        

class LDrawPartsMaint(gtk.Dialog):
    
#    #---------------------------------------------------- some utility functions
#    def readCatalog(self):
#        self.partList.read()
#        
#        self.modified = False

    #----------------------------------------------------------------- callbacks
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')
        
    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="LDraw mapping database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="LDraw mapping database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)


    #--------------------------------------------------------- apply new filters
    def reFilterCB(self,widget):
        #----------------------------------------------- collect filter settings
        f = None
        if self.filterLmod.get_active():
            f = 'lmod'
        if self.filterNoLddid.get_active():
            if f == None:
                f = '(lddid=0 or lddid is null)'
            else:
                f += ' and (lddid=0 or lddid is null)'
        else:
            lddid = self.filterLddid.get_text()
            if len(lddid) != 0:
                if f == None:
                    f = 'cast(lddid as text) like \''+lddid+'%\''
                else:
                    f += ' and cast(lddid as text) like \''+lddid+'%\''
        if self.filterNoLdrid.get_active():
            if f == None:
                f = '(ldrawid=\'\' or ldrawid is null)'
            else:
                f += ' and (ldrawid=\'\' or ldrawid is null)'
        else:
            ldrawid = self.filterLdrid.get_text()
            if len(ldrawid) != 0:
                if f == None:
                    f = 'ldrawid like \'%'+ldrawid+'%\''
                else:
                    f += ' and ldrawid like \'%'+ldrawid+'%\''
        if self.filterL2D.get_active():
            if f == None:
                f = 'not lddtodat'
            else:
                f += ' and not lddtodat'
        if self.filterD2L.get_active():
            if f == None:
                f = 'not dattoldd'
            else:
                f += ' and not dattoldd'
        descr = self.filterDescr.get_text()
        if len(descr) > 0:
            if f == None:
                f = 'notes like \'%'+descr+'%\''
            else:
                f += ' and notes like \'%'+descr+'%\''
        if f == None:
            return
        self.partList = PartList(self.catalog)
        self.partList.setFilter(f)
        self.partListView.set_model(self.partList)

        
        
    def resetFilterCB(self,widget):
        self.filterLmod.set_active(False)
        self.filterNoLddid.set_active(False)
        self.filterNoLdrid.set_active(False)
        self.filterL2D.set_active(False)
        self.filterD2L.set_active(False)
        self.filterLddid.set_text('')
        self.filterLdrid.set_text('')
        self.filterDescr.set_text('')
        self.partList = PartList(self.catalog)
        self.partList.read()
        self.partListView.set_model(self.partList)
        
        
    def noLddidCB(self,widget):
        if widget.get_active():
            self.filterLddid.set_sensitive(False)
        else:
            self.filterLddid.set_sensitive(True)

    def noLdridCB(self,widget):
        if widget.get_active():
            self.filterLdrid.set_sensitive(False)
        else:
            self.filterLdrid.set_sensitive(True)



#    #--------------------------------------------------- import button callback
#    def importDBButtonCB(self,widget):
#        #----------------------------- part DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="LDraw mapping database modified, ignore and lose changes?")
#            msg.set_title('Confirm open')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            partmapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of parts
#            conn = sqlite3.connect(partmapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query part table
#            c.execute('select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod from ldrawmap')
#            
#            #------------------------------------ iterates through query results
#            self.partList.clear()
#            for part in c:
#                #--------------------------------- extract maximum key value from db
#                self.partList.append(row=(part[0] if part[0] != None else 0,            
#                                          part[1] if part[1] != None else '',   # BL part ID
#                                          part[2],
#                                          part[3],
#                                          part[4] if part[4] != None else '',    # notes
#                                          part[5],
#                                          part[6] if part[6] != None else 0
#                                           ))
#            conn.close()
#            self.saveDBButton.set_sensitive(True)
#            self.modified = True
#            
#    #------------------------------- update part map database with a new release
#    def updateDBButtonCB(self,widget):
#        #----------------------------- part DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="LDraw mapping database modified, ignore and lose changes?")
#            msg.set_title('Confirm update')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#        
#        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Want to keep local parts editings?")
#        msg.set_title('Keep your mappings?')
#        response = msg.run()
#        msg.hide()
#        msg.destroy()
#        if response != gtk.RESPONSE_YES and response != gtk.RESPONSE_NO:
#            return
#        keepEdits = response == gtk.RESPONSE_YES
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            partmapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of parts
#            conn = sqlite3.connect(partmapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query part table
#            c.execute('select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod from ldrawmap')
#            #-------------------------------------------- fills in working table
#            self.catalog.connection.execute('drop table if exists newpartmap')
#            self.catalog.connection.execute('create table newpartmap (lddid integer, ldrawid text,dattoldd bool,lddtodat bool,notes text, lmod bool, lastmod integer)')
#            for part in c:
#                self.catalog.connection.execute('insert into newpartmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) values(?,?,?,?,?,?,?)',
#                                     (part[0],part[1],part[2],part[3],part[4],part[5],part[6]))
#            if keepEdits:
#                #---------------------------------- creates working tables for update
#                self.catalog.connection.execute('drop table if exists updatedpartmap')
#                self.catalog.connection.execute('create table updatedpartmap (lddid integer, ldrawid text,dattoldd bool,lddtodat bool,notes text, lmod bool, lastmod integer)')
#                #----------------------------- insert parts not locally modified
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) '+
#                                     'select n.lddid,n.ldrawid,n.dattoldd,n.lddtodat,n.notes,n.lmod,n.lastmod from newpartmap as n join ldrawmap as o '+
#                                     'using (lddid,ldrawid) where n.lddid is not null and n.ldrawid is not null and o.lmod=0'
#                                     )
#                #--------------------------- insert new parts not locally listed
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) '+
#                                                'select n.lddid,n.ldrawid,n.dattoldd,n.lddtodat,n.notes,n.lmod,n.lastmod from newpartmap as n left join ldrawmap as o '+
#                                                'using (lddid,ldrawid) where o.lddid is null and o.ldrawid is null') 
#                #------------------------ insert locally modified from old table
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) '+
#                                     'select lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod from ldrawmap where lmod=1'
#                                     )
#                self.catalog.connection.execute('drop table if exists ldrawmap')
#                self.catalog.connection.execute('alter table updatedpartmap rename to ldrawmap')
#                self.catalog.connection.execute('drop table if exists newpartmap')
#            else:
#                self.catalog.connection.execute('drop table if exists ldrawmap')
#                self.catalog.connection.execute('alter table newpartmap rename to ldrawmap')
#            self.catalog.commit()
#            conn.close()
#            self.partList.read()
#            
#
#    #----------------------------------------------- save partmap database file
#    def saveDBButtonCB(self,widget):
#        self.partList.write()
#        self.saveDBButton.set_sensitive(False)
#        self.modified = False
#    
#    #---------------------------------------- save partmap database as new file
#    def exportDBButtonCB(self,widget):
#        response = self.saveDialog.run()
#        self.saveDialog.hide()
#        if response == gtk.RESPONSE_ACCEPT:
#            partmapfile = self.saveDialog.get_filename()
#            if os.path.splitext(partmapfile)[1] == '':
#                partmapfile += '.sq3'
#        else:
#            return
#                
#        conn = sqlite3.connect(partmapfile)        
#        c = conn.cursor()
#
#        # re-create table with part map list
#        c.execute('drop table if exists ldrawmap')
#        
#        c.execute('create table ldrawmap (lddid integer, ldrawid text,dattoldd bool,lddtodat bool,notes text, lmod bool, lastmod integer)')
#        
#        for row in self.partList:
#            c.execute('insert into ldrawmap(lddid,ldrawid,dattoldd,lddtodat,notes,lmod,lastmod) values(?,?,?,?,?,?,?)',
#                      (row[0],row[1],row[2],row[3],row[4].decode('utf-8'),row[5],row[6])
#                      )
#        conn.commit()
#        conn.close()
        
    #------------------------------------------------- new part button callback
    def newButtonCB(self,widget):
        dlg = PLEditDialog(title="New LDraw part mapping",parent=self,catalog=self.catalog,imgCache=self.imgCache)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try:
                lddid = int(dlg.lddid.get_text())
            except ValueError:
                lddid = 0
            if lddid == 0 and (dlg.ldd2dat.get_active() or dlg.dat2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD ID must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            ldrawid = dlg.ldrawid.get_text().lower()
            if len(ldrawid) == 0 and (dlg.ldd2dat.get_active() or dlg.dat2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDraw ID cannot be empty")
                msg.set_title('Wrong LDraw ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            if lddid == 0 and len(ldrawid) == 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="At least LDraw ID or LDD ID must be given")
                msg.set_title('Empty IDs')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            
            d = self.catalog.ldrMapDuplicated(lddid,ldrawid)
            if len(d) > 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                msg.set_markup("Duplicated part mapping\nLDD ID: <b>"+str(lddid)+
                                "</b> LDraw ID: <b>"+ldrawid+'</b> ID: <b>'+str(d[0][2])+'</b>\n')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            b = self.catalog.ldrMapExistsD2L(ldrawid)
            l = self.catalog.ldrMapExistsL2D(lddid)            
            if (dlg.dat2ldd.get_active() and len(b) > 0) or (dlg.ldd2dat.get_active() and len(l) > 0):
                #------------- only one part can be converted from LDD to LDraw
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                if len(b) > 0:
                    msg.set_markup("Conversion from LDraw to LDD for this part already exists: LDD ID <b>"+str(b[0][0])+
                               "</b> Ldraw ID: <b>"+b[0][1]+'</b> ID: <b>'+str(b[0][2])+'</b>')
                else:
                    msg.set_markup("Conversion from LDD to LDraw for this part already exists: LDD ID <b>"+str(l[0][0])+
                               "</b> LDraw ID: <b>"+l[0][1]+'</b> ID: <b>'+str(l[0][2])+'</b>')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            
            rowid = self.catalog.insertLDrawPartMap(lddid,ldrawid,dlg.dat2ldd.get_active(),
                                                   dlg.ldd2dat.get_active(),dlg.notes.get_text().decode('utf-8'),True)
            self.catalog.commit()
            self.partList.append(row=(lddid,dlg.ldrawid.get_text(),dlg.dat2ldd.get_active(),
                                      dlg.ldd2dat.get_active(),dlg.notes.get_text(),True,int(time.time()),rowid))
            
    
    #------------------------------------------------------- delete part button
    def deleteButtonCB(self,widget):
        sel = self.partListView.get_selection()
        if sel.count_selected_rows() == 0:
            selPath = self.partListView.get_cursor()
            if selPath[0] == None:
                return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete LDraw part mapping')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete part mapping from DB
            if sel.count_selected_rows() > 0:
                while sel.count_selected_rows() > 0:
                    p = sel.get_selected_rows()[1][0]
                    r = self.partList.get_iter(p)
                    rowid, = self.partList.get(r,7)
                    self.catalog.deleteLDrawMap(rowid)
                    self.partList.remove(r)
            else:
                selRow = self.partList.get_iter(selPath[0])
                rowid, = self.partList.get(r,7)
                self.catalog.deleteLDrawMap(rowid)
                self.partList.remove(selRow)
            self.catalog.commit()
    
    def editButtonCB(self,widget,selPath=None,selRow=None):
        if selPath == None:
            sel = self.partListView.get_selection()
            if sel.count_selected_rows() > 1:
                # only one at time
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Please select only one element to edit")
                msg.set_title('Too many selected')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            selPath = self.partListView.get_cursor()
        if selPath[0] == None:
            return
        selRow = self.partList.get_iter(selPath[0])
        lddid,ldrawid,dat2ldd,ldd2dat,notes,rowid = self.partList.get(selRow,0,1,2,3,4,7)        
        dlg = PLEditDialog(title='Edit LDraw part mapping',parent=self,catalog=self.catalog,imgCache=self.imgCache)
        dlg.lddid.set_text(str(lddid))
        dlg.dat2ldd.set_active(dat2ldd)
        dlg.ldd2dat.set_active(ldd2dat)
        dlg.ldrawid.set_text(ldrawid)
        dlg.notes.set_text(notes)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            # save edits
            try:
                lddid = int(dlg.lddid.get_text())
            except ValueError:
                lddid = 0
            if lddid == 0 and (dlg.ldd2dat.get_active() or dlg.dat2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD ID must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            ldrawid = dlg.ldrawid.get_text().lower()
            if len(ldrawid) == 0 and (dlg.ldd2dat.get_active() or dlg.dat2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDraw ID cannot be empty")
                msg.set_title('Wrong LDraw ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            if lddid == 0 and len(ldrawid) == 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="At least LDraw ID or LDD ID must be given")
                msg.set_title('Empty IDs')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            d = self.catalog.ldrMapDuplicated(lddid,ldrawid,rowid)
            if len(d) > 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                msg.set_markup("Duplicated part mapping\nLDD ID: <b>"+str(lddid)+
                                "</b> LDraw ID: <b>"+ldrawid+'</b> ID: <b>'+str(d[0][2])+'</b>\n')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            b = self.catalog.ldrMapExistsD2L(ldrawid,rowid)
            l = self.catalog.ldrMapExistsL2D(lddid,rowid)            
            if (dlg.dat2ldd.get_active() and len(b) > 0) or (dlg.ldd2dat.get_active() and len(l) > 0):
                #------------- only one part can be converted from LDD to LDraw
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                if len(b) > 0:
                    msg.set_markup("Conversion from LDraw to LDD for this part already exists: LDD ID <b>"+str(b[0][0])+
                               "</b> Ldraw ID: <b>"+b[0][1]+'</b> ID: <b>'+str(b[0][2])+'</b>')
                else:
                    msg.set_markup("Conversion from LDD to LDraw for this part already exists: LDD ID <b>"+str(l[0][0])+
                               "</b> LDraw ID: <b>"+l[0][1]+'</b> ID: <b>'+str(l[0][2])+'</b>')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            
            self.catalog.updateLDrawMap(lddid,ldrawid,dlg.dat2ldd.get_active(),dlg.ldd2dat.get_active(),dlg.notes.get_text().decode('utf-8'),rowid)
            self.partList[selPath[0]] = (lddid,
                                         ldrawid,
                                         dlg.dat2ldd.get_active(),
                                         dlg.ldd2dat.get_active(),
                                         dlg.notes.get_text(),
                                         True,
                                         int(time.time()),
                                         rowid
                                         )
            self.catalog.commit()
            

    def cursorChangedCB(self,widget):
        selPath = self.partListView.get_cursor()
        if selPath[0] == None:
            return
        selRow = self.partList.get_iter(selPath[0])
        ldrawid, = self.partList.get(selRow,1)
        brickFile = self.imgCache.brickImgLdraw(ldrawid)
        if brickFile == None:
            self.ldrImage.clear()
            return
        #print brickFile
        if self.ldrImage != None:
            r = self.ldrImage.get_allocation()
            x = r.width
            y = x
            img = gtk.gdk.pixbuf_new_from_file(brickFile)
            img = img.scale_simple(x,y,gtk.gdk.INTERP_BILINEAR)
            self.ldrImage.set_from_pixbuf(img)        

            

    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None,imageCache=None):
        super(LDrawPartsMaint,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.dialogTitle = title
        self.modified = False
        self.imgCache = imageCache
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)         
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
#        self.importDBButton = gtk.ToolButton()
#        self.importDBButton.set_stock_id(gtk.STOCK_OPEN)
#        self.importDBButton.set_tooltip_text("Import LDraw partmap database")
#        self.importDBButton.connect('clicked',self.importDBButtonCB)
#        self.toolBar.insert(self.importDBButton,-1)
#        
#        self.saveDBButton = gtk.ToolButton()
#        self.saveDBButton.set_stock_id(gtk.STOCK_SAVE)
#        self.saveDBButton.set_tooltip_text("Save LDraw partmap database in main catalog")
#        self.saveDBButton.set_sensitive(False)
#        self.saveDBButton.connect('clicked',self.saveDBButtonCB)
#        self.toolBar.insert(self.saveDBButton,-1)
#        
#        self.exportDBButton = gtk.ToolButton()
#        self.exportDBButton.set_stock_id(gtk.STOCK_SAVE_AS)
#        self.exportDBButton.set_tooltip_text("Export LDraw partmap database as new file")
#        self.exportDBButton.connect('clicked',self.exportDBButtonCB)
#        self.toolBar.insert(self.exportDBButton,-1)
#        
#        self.updateDBButton = gtk.ToolButton()
#        self.updateDBButton.set_stock_id(gtk.STOCK_REFRESH)
#        self.updateDBButton.set_label('Update')
#        self.updateDBButton.set_tooltip_text('Update LDraw partmap database from file')
#        self.updateDBButton.connect('clicked',self.updateDBButtonCB)
#        self.toolBar.insert(self.updateDBButton,-1)

        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Quit to main")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
#        #-------------------------------------------- various dialogs and popups
#        self.openDialog = gtk.FileChooserDialog("Select LDraw partmap file",self.window,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        
#        self.saveDialog = gtk.FileChooserDialog("Save as LDraw partmap file",self.window,
#            action=gtk.FILE_CHOOSER_ACTION_SAVE,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        self.saveDialog.set_do_overwrite_confirmation(True)
#        
#        #------------------------------------ add a filter for SQLite3 files
#        sqliteFilter = gtk.FileFilter()
#        sqliteFilter.add_pattern("*.sq3")
#        sqliteFilter.set_name("SQlite database files")
#        self.openDialog.add_filter(sqliteFilter)
#
#        #-------------------------------------- add a filter for any (all) files
#        allFilter = gtk.FileFilter()
#        allFilter.add_pattern("*")
#        allFilter.set_name("All files")
#        self.openDialog.add_filter(allFilter)
#        self.saveDialog.add_filter(allFilter)
        
        #----------------------------------------------- list store for treeview
        self.partList = PartList(self.catalog)
        self.partList.read() 
        
        self.partListView = LDrawListView(self.partList)
        self.partListView.set_size_request(500,250)
        self.partListView.connect('cursor-changed',self.cursorChangedCB)
        self.partListView.connect('row-activated',self.editButtonCB)
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        table = gtk.Table(2,2)
        table.set_border_width(3)
        table.set_col_spacings(3)
        table.set_row_spacings(3)
        table.attach(self.listWindow,0,1,0,1)
        self.ldrImage = gtk.Image()
        self.ldrImage.set_size_request(200,200)
        frame = gtk.Frame('Brick shape')
        frame.add(self.ldrImage)
        table.attach(frame,1,2,0,1,xoptions=gtk.SHRINK,yoptions=gtk.SHRINK)

        #------------------------------------------------ filter definition area
        self.filterFrame = gtk.Frame("Filter part by")
 
        ftable = gtk.Table(3,4)
        ftable.set_row_spacings(4)
        ftable.set_col_spacings(4)
        ftable.set_border_width(3)
        self.filterFrame.add(ftable)
        #---------------------------------------- place widget to select filters
        #-------------------------------------------- filter by modified locally
        self.filterLmod = gtk.CheckButton("Locally modified")
        #------------------------------------------------- filter by no ldd code
        self.filterNoLddid = gtk.CheckButton("No LDD ID code")
        self.filterNoLddid.connect('toggled',self.noLddidCB)
        #------------------------------------------------ filter by no ldrawid code
        self.filterNoLdrid = gtk.CheckButton("No LDraw ID code")
        self.filterNoLdrid.connect('toggled',self.noLdridCB)
        #----------------------------------- filter by no bl2ldd or lddtodat flag
        self.filterL2D = gtk.CheckButton("No convert LDD > LDraw")
        self.filterD2L = gtk.CheckButton("No convert LDraw > LDD")
        self.filterLddid = gtk.Entry()
        self.filterLdrid = gtk.Entry()
        self.filterLddidLabel = gtk.Label("LDD ID:")
        self.filterLddidLabel.set_alignment(1,0.5)
        self.filterLdridLabel = gtk.Label("Ldraw ID:")
        self.filterLdridLabel.set_alignment(1,0.5)
    
        #--------------------------------------- filter by string in description
        self.filterDescr = gtk.Entry()
        self.filterDescrLabel = gtk.Label("Notes contains string:")
        self.filterDescrLabel.set_alignment(1,0.5)
        self.filterRefilter = gtk.Button("Apply filter")
        self.filterReset = gtk.Button("Reset")
        
        ftable.attach(self.filterLddidLabel,1,2,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterLddid,2,3,0,1,yoptions=gtk.FILL,xpadding=3)
        ftable.attach(self.filterNoLddid,1,3,1,2,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterL2D,1,3,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterLdridLabel,3,4,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterLdrid,4,5,0,1,yoptions=gtk.FILL,xpadding=3)
        ftable.attach(self.filterNoLdrid,3,5,1,2,yoptions=gtk.FILL,xpadding=3)
        ftable.attach(self.filterD2L,3,5,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterDescrLabel,5,6,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterDescr,6,7,0,1,yoptions=gtk.FILL,xpadding=3)
        ftable.attach(self.filterLmod,5,6,1,2,xoptions=gtk.FILL,yoptions=gtk.FILL)
        ftable.attach(self.filterRefilter,6,7,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL,xpadding=3)
        ftable.attach(self.filterReset,5,6,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL,xpadding=3)
        
        table.attach(self.filterFrame,0,2,1,2,xoptions=gtk.FILL,yoptions=gtk.SHRINK)
        
        self.filterRefilter.connect('clicked',self.reFilterCB)
        self.filterReset.connect('clicked',self.resetFilterCB)
        
        self.hButtonBox = gtk.HButtonBox()
        self.hButtonBox.set_layout(gtk.BUTTONBOX_END)
        
        self.newButton = gtk.Button(stock=gtk.STOCK_NEW)
        self.newButton.connect('clicked',self.newButtonCB)
        self.editButton = gtk.Button(stock=gtk.STOCK_EDIT)
        self.editButton.connect('clicked',self.editButtonCB)
        self.deleteButton = gtk.Button(stock=gtk.STOCK_DELETE)
        self.deleteButton.connect('clicked',self.deleteButtonCB)
        
        self.hButtonBox.pack_start(self.newButton,False)
        self.hButtonBox.pack_start(self.editButton,False)
        self.hButtonBox.pack_start(self.deleteButton,False)
        table.attach(self.hButtonBox,0,2,2,3,xoptions=gtk.FILL,yoptions=gtk.SHRINK)
        self.vBox.pack_end(table,True,True)
        
        self.set_default_size(500,400)
        
        self.vBox.show_all()
        
