'''
Created on 21/nov/2011

@author: mario
'''

import gtk
import xml.parsers.expat
from BrickException import BrickException


class ParseLDDXML():
    '''
    parse a LDD project as XML file
    '''

    def hStartEl(self,name,attrs):
        #-------------------------------------------- let the GUI updates itself
        while gtk.events_pending():
            gtk.main_iteration(block=False)
        if name == 'Bricks':
            #--------------------------------- ok, this is a start in BL catalog
            self.fBricks = True
            self.currList.clear()
            return
        if name == 'Brick':
            #-------------------------- it's ok if we are into "catalog" element
            if self.fBricks:
                self.fBrick = True
                self.parts = []
                self.partID = attrs['designID']
                return
            else:
                raise BrickException("Invalid LDD LXF/LXFML file, 'Brick' element outside 'Bricks'")
        if name == 'Part':
            if self.fBrick:
                if (attrs.has_key('materials')):
                    material = attrs['materials']
                else:
                    material = ''
                if (attrs.has_key('decoration')):
                    decoration = attrs['decoration']
                else:
                    decoration = ''
                self.parts.append((attrs['designID'],material,decoration))
            else:
                raise BrickException("Invalid LDD LXF/LXFML file, 'Part' element outside 'Brick'")
            
    def hEndEl(self,name):
        if name == 'Brick':
            try:
                lddid = int(self.partID)
            except ValueError:
                raise BrickException("Invalid designID: '"+self.partID+"'. Please check.")
            lddcolor = 0
            decorated = False
            if len(self.parts) == 1:
                #--------------------------------------- standard "simple" brick
                color = self.parts[0][1].split(',')
                if len(color) >= 1:
                    #-------- for now, we suppose that color is always the first
                    try:
                        lddcolor = int(color[0])
                    except ValueError:
                        lddcolor = 0

                for d in self.parts[0][2].split(','):
                    if d != '0' and len(d) > 0:
                        decorated = True
                        break
            else:
                #-------------------------------------- manage "composite" brick
                compBrick = self.catalog.getLDDComposite(lddid)
                if len(compBrick) == 1:
                    for p in self.parts:
                        #------------------------------ search subpart for color
                        try:
                            id = int(p[0])
                        except ValueError:
                            id = 0
                        if id == compBrick[0][1]:
                            try:
                                lddcolor = int(p[1].split(',')[compBrick[0][3]])
                            except ValueError:
                                lddcolor = 0 
                        #------------------------- search subpart for decoration
                        if id == compBrick[0][2]:
                            for d in p[2].split(','):
                                if d != '0'and len(d) > 0:
                                    decorated = True
                                    break
                elif len(compBrick) > 1:
                    raise BrickException("Composite brick duplicated in database.")
                
            qty = 1

            if self.partIdx.has_key((lddid,lddcolor,decorated)):
                idx = self.partIdx[(lddid,lddcolor,decorated)]
                self.currList[idx][2] += 1
            else:
                bl = self.catalog.convertLDD2BL(lddid,lddcolor,decorated)
                self.partIdx[(lddid,lddcolor,decorated)] = self.currList.append(row=['P',      # type
                                bl['blid'],           # blpartid
                                qty,            # qty
                                bl['blcolorid'],      # blcolorid
                                False,          # no extraPart
                                False,          # no counterpart
                                bl['descr'],          # description (from BL)
                                lddid,          # LDD part id
                                bl['lddrgb'],   # rgb color from LDD
                                bl['lddcolorname'],   # color name from LDD
                                bl['blrgb'],    # rgb color from BL
                                bl['blcolorname'],    # name from BL
                                bl['blcategory'],     # category in BLparts
                                self.partCount,         # unique ID for part
                                lddcolor,       # ldd color ID
                                decorated,      # decorated part?
                                False           # no alternate parts in LDD
                                ])
#            for part in self.currList:
#                if lddid == int(part[7]) and lddcolor == part[14] and decorated == part[15]:
#                    qty = part[2] + 1
#                    self.currList.remove(part.iter)
#                    break
#                
#            self.currList.append(row=['P',      # type
#                                bl['blid'],           # blpartid
#                                qty,            # qty
#                                bl['blcolorid'],      # blcolorid
#                                False,          # no extraPart
#                                False,          # no counterpart
#                                bl['descr'],          # description (from BL)
#                                lddid,          # LDD part id
#                                bl['lddrgb'],   # rgb color from LDD
#                                bl['lddcolorname'],   # color name from LDD
#                                bl['blrgb'],    # rgb color from BL
#                                bl['blcolorname'],    # name from BL
#                                bl['blcategory'],     # category in BLparts
#                                self.partCount,         # unique ID for part
#                                lddcolor,       # ldd color ID
#                                decorated,      # decorated part?
#                                False           # no alternate parts in LDD
#                                ])
            if (self.partCount % 20) == 0: 
                self.partCountWidget.set_text(str(self.partCount))
            self.partCount += 1
            self.fBrick = False            
            return
        if name == 'Bricks':
            self.fBricks = False
            self.partCountWidget.set_text(str(self.partCount-1))
            return



    def __init__(self,lddXml=None,currList=None,catalog=None,partCountWidget=None):
        '''
        Takes as arguments: 
        lddxml - file handle of lxf or lxfml file
        currList - a gtk.ListStore for current brick list
        catalog - connection to sqlite3 catalog db
        partCountWidget - widget for counting part imported
        autoMap - consider LDD part id == BL id if not found a mapping in part mapping DB
        '''
        
        self.lddXml = lddXml
        self.catalog = catalog
        self.partCountWidget = partCountWidget
        self.currList = currList
        #-------------------------------------------------- creates Expat parser
        self.Parser = xml.parsers.expat.ParserCreate()
        #----------------------------------------------------- defines callbacks
        self.Parser.StartElementHandler = self.hStartEl
        self.Parser.EndElementHandler = self.hEndEl
        self.Parser.buffer_text = True
        self.partCount = 1
        self.partID = 0
        self.fBricks = False
        self.fBrick = False
        self.partIdx = dict()
        
        
    def run(self):        
        self.Parser.ParseFile(self.lddXml)


