#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
  PartsMaintDialog - utility to edit parts mapping DB
  Copyright (C) 2011 Mario Pascucci 
  http://bricksnspace.wordpress.com/ <mpascucci gmail com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''

#===============================================================================
# TODO
#
#===============================================================================



#===============================================================================
# Useful queries.
# 
# - checks if a part is changed or deleted from BrickLink catalog
# select p.blid,p.lddid,p.notes from partmap as p left join blpartlist as b on (blid=itemid) where b.itemid is null;
#
# - checks if a part previous not available in BL, is now enlisted
# select p.blid,p.lddid,p.notes from partmap as p left join blpartlist as b on (blid=itemid) where (not p.bltoldd) and (not p.lddtobl) and b.itemid is not null;
#
# - list BL parts not in partmap
# select b.itemid,b.itemname from blpartlist as b left join partmap as p on (blid=itemid) where p.lddid is null;
#
#
# Update bricklink parts, adding new parts in partmap:
# - add parts not in partmap:
#    insert into partmap select 0 as lddid, b.itemid as blid,0 as bltoldd, 0 as lddtobl,b.itemname as notes, 0 as lmod, 0 as lastmod from blpartlist as b left join partmap as p on(b.itemid=p.blid) where p.blid is null;
# - delete Duplo, stickers, znap, modulex, clikits, homemaker:
#    delete from partmap where notes like '%duplo%' or notes like '%znap%' or notes like '%modulex%' or notes like '%sticker%' or notes like '%clikits%' or notes like '%homemaker%';
# - delete some other parts:
#    delete from partmap where blid like 'crssprt%' or blid like 'dupstr%' or blid like 'gal%' or blid like 'pri%';
# - delete some "decorated" parts:
#    delete from partmap where blid like '3626bp%' or blid like '3068bp%';
#    delete from partmap where blid like '973p%' and not bltoldd and not lddtobl;
#    delete from partmap where notes like 'paper,%';
#
#
# add new parts from LDD .lif file
# - strings -e l .wine/drive_c/users/mario/Applicazioni/LEGO\ Company/LEGO\ Digital\ Designer/db.lif > ldd-ver.txt
#    open file in editor and search for "3001.xml"
#    look for a list called "Assembly" followed by a list of .lxfml files
#    after that look for "Primitives" followed by a list of .xml files
#    delete all but these lists and remove .lxfml, .xml extension
#    now you have the list of parts in .lif file
# - create table in sqlite using command line, importing part list file 
#    create table lddnew(lddid text);
#    .import /home/mario/ldd-436.txt lddnew
# - obtain a list of new parts
#    select * from lddnew l left join partmap p using(lddid) where p.lddid is null;
# - obtain a list of LDD parts not in LDraw <-> LDD mapping table
#    select p.lddid,p.notes from partmap p left join ldrawmap l where l.lddid is null and (p.lddtobl or p.bltoldd);
#===============================================================================



#===============================================================================
# Imports
#===============================================================================

#import os.path
import time
#import sqlite3
import gtk
import gobject
import pygtk
from BLPartSearch import BLPartSearch

pygtk.require('2.0')



class PMEditDialog(gtk.Dialog):
    
    def searchPartCB(self,widget):
        dlg = BLPartSearch(parent=self,catalog=self.catalog,imgCache=self.imgCache)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            c = dlg.resview.get_cursor()[0]
            if c == None:
                return
            i = dlg.reslist.get_iter(c)
            self.blid.set_text(dlg.reslist.get(i,0)[0])
            self.notes.set_text(dlg.reslist.get(i,1)[0])

    def __init__(self,title=None, parent=None, flags=0, 
                 buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT),catalog=None,imgCache=None):
        super(PMEditDialog, self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.imgCache = imgCache
        
        self.vBox = self.get_content_area()
        
        self.table = gtk.Table(rows=5, columns=3)
        
        self.lab1 = gtk.Label('LDD design ID')
        self.lab1.set_alignment(1,0.5)
        self.table.attach(self.lab1,0,1,0,1,xoptions=gtk.FILL,xpadding=3)
        
        self.lddid = gtk.Entry()
        self.table.attach(self.lddid,1,3,0,1)
        
        self.lab4 = gtk.Label('Bricklink ID')
        self.lab4.set_alignment(1,0.5)
        self.table.attach(self.lab4,0,1,1,2,xoptions=gtk.FILL,xpadding=3)
        
        self.blid = gtk.Entry()
        self.table.attach(self.blid,1,2,1,2)
        
        self.searchPart = gtk.Button('Search...')
        self.table.attach(self.searchPart,2,3,1,2)
        self.searchPart.connect('clicked',self.searchPartCB)
                
        self.lab8 = gtk.Label('Notes')
        self.lab8.set_alignment(1,0.5)
        self.table.attach(self.lab8,0,1,2,3,xoptions=gtk.FILL,xpadding=3)
        
        self.notes = gtk.Entry()
        self.notes.set_size_request(300,-1)
        self.table.attach(self.notes,1,3,2,3)
        
        lab1 = gtk.Label('Mapping direction:')
        lab1.set_alignment(1,0.5)
        self.table.attach(lab1,0,1,3,5)
        
        self.bl2ldd = gtk.CheckButton('BrickLink -> LDD')
        self.table.attach(self.bl2ldd,1,2,3,4)
        self.ldd2bl = gtk.CheckButton('LDD -> BrickLink')
        self.table.attach(self.ldd2bl,1,2,4,5)
        
        self.vBox.pack_start(self.table,False)
        self.vBox.show_all()
        


class PartList(gtk.ListStore):
 
    def read(self):
        #------------------------------------------------- query all parts
        c = self.catalog.getFilteredParts(self.qfilter)
        
        #------------------------------------ iterates through query results
        self.clear()

        for r in c:
            self.append(row=(r[0],
                             r[1] if r[1] != None else '',
                             r[2],
                             r[3],
                             r[4] if r[4] != None else '',
                             r[5],
                             r[6],
                             r[7]
                             ))
        
        self.set_sort_column_id(0,gtk.SORT_ASCENDING)


    def write(self):
        self.catalog.cleanPartMappingTable()
        for row in self:
            self.catalog.insertPartMapping(row[0],row[1],row[2],row[3],row[4].decode('utf-8'),row[5],row[6],row[7])
        self.catalog.commit()
        
    def setFilter(self,q):
        if q == self.qfilter:
            return
        self.qfilter = q
        self.read()
        
    def __init__(self,catalog=None):
        super(PartList, self).__init__(gobject.TYPE_INT,     # lddid
                                       gobject.TYPE_STRING,  # blid
                                       gobject.TYPE_BOOLEAN, # bl to ldd
                                       gobject.TYPE_BOOLEAN, # ldd to bl
                                       gobject.TYPE_STRING,  # notes
                                       gobject.TYPE_BOOLEAN, # locally modified
                                       gobject.TYPE_INT,     # last modified
                                       gobject.TYPE_INT      # rowid from db
                                       )
        self.qfilter = None
        self.catalog = catalog

class PartListView(gtk.TreeView):

    def setSearchCB(self,widget,column):
        self.set_search_column(column)
        
    def __init__(self,listStore):
        super(PartListView, self).__init__(listStore)

        self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_HORIZONTAL)
        sel = self.get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)

        #---------------------- column and cell renderer definition for treeview
        self.crtLDDID = gtk.CellRendererText()
        self.tvcLDDID = gtk.TreeViewColumn('LDD ID',self.crtLDDID)
        self.tvcLDDID.set_resizable(True)
        
        self.renderblid = gtk.CellRendererText()
        self.tvcolblid = gtk.TreeViewColumn('BrickLink ID',self.renderblid)
        self.tvcolblid.set_resizable(True)
        
        self.crtgBL2LDD = gtk.CellRendererToggle()
        self.tvcBL2LDD = gtk.TreeViewColumn('BL->LDD',self.crtgBL2LDD)
        self.tvcBL2LDD.set_resizable(True)

        self.crtgLDD2BL = gtk.CellRendererToggle()
        self.tvcLDD2BL = gtk.TreeViewColumn('LDD->BL',self.crtgLDD2BL)
        self.tvcLDD2BL.set_resizable(True)
        
        self.rendernotes = gtk.CellRendererText()
        self.tvcolnotes = gtk.TreeViewColumn('Notes & comments',self.rendernotes)
        self.tvcolnotes.set_resizable(True)

        self.append_column(self.tvcLDDID)
        self.append_column(self.tvcolblid)
        self.append_column(self.tvcBL2LDD)
        self.append_column(self.tvcLDD2BL)                   
        self.append_column(self.tvcolnotes)
        self.set_expander_column(self.tvcolnotes)
        self.tvcLDDID.set_attributes(self.crtLDDID,text=0)
        self.tvcolblid.set_attributes(self.renderblid,text=1)
        self.tvcBL2LDD.set_attributes(self.crtgBL2LDD,active=2)
        self.tvcLDD2BL.set_attributes(self.crtgLDD2BL,active=3)
        self.tvcolnotes.set_attributes(self.rendernotes,text=4)
        self.set_search_column(0)
        self.tvcLDDID.set_sort_column_id(0)
        self.tvcLDDID.connect('clicked',self.setSearchCB,0)
        self.tvcolblid.set_sort_column_id(1)
        self.tvcolblid.connect('clicked',self.setSearchCB,1)
        self.tvcolnotes.set_sort_column_id(4)
        self.tvcolnotes.connect('clicked',self.setSearchCB,4)
        

class BrickPartsMaint(gtk.Dialog):
    
    #----------------------------------------------------------------- callbacks
    def responseCB(self,widget,response,*args):
        #------- needed to prevent closing if user press [x] in the window frame
        # see pyGTK FAQ 10.13 (http://faq.pygtk.org/index.py?req=show&file=faq10.013.htp)
        if response != gtk.RESPONSE_OK:
            #--------------- prevent closing if response is not from [OK] button
            self.emit_stop_by_name('response')
        
    
    def closeCB(self,widget,event):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Part mapping database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return True        
        return False    
    
    def quitCB(self,widget):
        if self.modified:
            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                            message_format="Part mapping database modified, ignore and lose changes?")
            msg.set_title('Confirm exit')
            response = msg.run()
            msg.hide()
            msg.destroy()
            if response != gtk.RESPONSE_YES:
                return
        self.response(gtk.RESPONSE_OK)

    #--------------------------------------------------------- apply new filters
    def reFilterCB(self,widget):
        #----------------------------------------------- collect filter settings
        f = None
        if self.filterLmod.get_active():
            f = 'lmod'
        if self.filterNoLddid.get_active():
            if f == None:
                f = '(lddid=0 or lddid is null)'
            else:
                f += ' and (lddid=0 or lddid is null)'
        else:
            lddid = self.filterLddid.get_text()
            if len(lddid) != 0:
                if f == None:
                    f = 'cast(lddid as text) like \''+lddid+'%\''
                else:
                    f += ' and cast(lddid as text) like \''+lddid+'%\''
        if self.filterNoBlid.get_active():
            if f == None:
                f = '(blid=\'\' or blid is null)'
            else:
                f += ' and (blid=\'\' or blid is null)'
        else:
            blid = self.filterBlid.get_text()
            if len(blid) != 0:
                if f == None:
                    f = 'blid like \'%'+blid+'%\''
                else:
                    f += ' and blid like \'%'+blid+'%\''
        if self.filterL2B.get_active():
            if f == None:
                f = 'not lddtobl'
            else:
                f += ' and not lddtobl'
        if self.filterB2L.get_active():
            if f == None:
                f = 'not bltoldd'
            else:
                f += ' and not bltoldd'
        descr = self.filterDescr.get_text()
        if len(descr) > 0:
            if f == None:
                f = 'notes like \'%'+descr+'%\''
            else:
                f += ' and notes like \'%'+descr+'%\''
        if f == None:
            return
        self.partList = PartList(self.catalog)
        self.partList.setFilter(f)
        self.partListView.set_model(self.partList)

        
        
    def resetFilterCB(self,widget):
        self.filterLmod.set_active(False)
        self.filterNoLddid.set_active(False)
        self.filterNoBlid.set_active(False)
        self.filterL2B.set_active(False)
        self.filterB2L.set_active(False)
        self.filterLddid.set_text('')
        self.filterBlid.set_text('')
        self.filterDescr.set_text('')
        self.partList = PartList(self.catalog)
        self.partList.read()
        self.partListView.set_model(self.partList)
        
        
    def noLddidCB(self,widget):
        if widget.get_active():
            self.filterLddid.set_sensitive(False)
        else:
            self.filterLddid.set_sensitive(True)

    def noBlidCB(self,widget):
        if widget.get_active():
            self.filterBlid.set_sensitive(False)
        else:
            self.filterBlid.set_sensitive(True)


    #--------------------------------------------------- import button callback
#    def importDBButtonCB(self,widget):
#        #----------------------------- part DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Part mapping database modified, ignore and lose changes?")
#            msg.set_title('Confirm open')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            partmapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of parts
#            conn = sqlite3.connect(partmapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query part table
#            c.execute('select lddid,blid,bltoldd,lddtobl,notes,ldmod,lastmod from partmap')
#            
#            #------------------------------------ iterates through query results
#            self.partList.clear()
#            for part in c:
#                #--------------------------------- extract maximum key value from db
#                self.partList.append(row=(part[0],            
#                                          part[1] if part[1] != None else '',   # BL part ID
#                                          part[2],
#                                          part[3],
#                                          part[4] if part[4] != None else '',    # notes
#                                          part[5],
#                                          part[6]
#                                           ))
#            conn.close()
#            self.saveDBButton.set_sensitive(True)
#            self.modified = True
#            
#    #------------------------------- update part map database with a new release
#    def updateDBButtonCB(self,widget):
#        #----------------------------- part DB modified, confirm open a new one
#        if self.modified:
#            msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Part mapping database modified, ignore and lose changes?")
#            msg.set_title('Confirm update')
#            response = msg.run()
#            msg.hide()
#            msg.destroy()
#            if response != gtk.RESPONSE_YES:
#                return
#        
#        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                            message_format="Want to keep local parts editings?")
#        msg.set_title('Keep your mappings?')
#        response = msg.run()
#        msg.hide()
#        msg.destroy()
#        if response != gtk.RESPONSE_YES and response != gtk.RESPONSE_NO:
#            return
#        keepEdits = response == gtk.RESPONSE_YES
#        response = self.openDialog.run()
#        self.openDialog.hide()
#        if (response == gtk.RESPONSE_ACCEPT):
#        # file selected, OK clicked
#            partmapfile = self.openDialog.get_filenames()[0]
#            #------------------------- reads database and compile list of parts
#            conn = sqlite3.connect(partmapfile)
#            c = conn.cursor()
#            
#            #------------------------------------------------- query part table
#            c.execute('select lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod from partmap')
#            #-------------------------------------------- fills in working table
#            self.catalog.connection.execute('drop table if exists newpartmap')
#            self.catalog.connection.execute('create table newpartmap (lddid integer, blid text,bltoldd bool,lddtobl bool,notes text, lmod bool, lastmod integer)')
#            for part in c:
#                self.catalog.connection.execute('insert into newpartmap (lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) values(?,?,?,?,?,?,?)',
#                                     (part[0],part[1],part[2],part[3],part[4],part[5],part[6]))
#            if keepEdits:
#                #---------------------------------- creates working tables for update
#                self.catalog.connection.execute('drop table if exists updatedpartmap')
#                self.catalog.connection.execute('create table updatedpartmap (lddid integer, blid text,bltoldd bool,lddtobl bool,notes text, lmod bool, lastmod integer)')
#                #----------------------------- insert parts not locally modified
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) '+
#                                     'select n.lddid,n.blid,n.bltoldd,n.lddtobl,n.notes,n.lmod,n.lastmod from newpartmap as n join partmap as o '+
#                                     'using (lddid,blid) where n.lddid is not null and n.blid is not null and o.lmod=0'
#                                     )
#                #--------------------------- insert new parts not locally listed
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) '+
#                                                'select n.lddid,n.blid,n.bltoldd,n.lddtobl,n.notes,n.lmod,n.lastmod from newpartmap as n left join partmap as o '+
#                                                'using (lddid,blid) where o.lddid is null and o.blid is null') 
#                #------------------------ insert locally modified from old table
#                self.catalog.connection.execute('insert into updatedpartmap (lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) '+
#                                     'select lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod from partmap where lmod=1'
#                                     )
#                self.catalog.connection.execute('drop table if exists partmap')
#                self.catalog.connection.execute('alter table updatedpartmap rename to partmap')
#                self.catalog.connection.execute('drop table if exists newpartmap')
#            else:
#                self.catalog.connection.execute('drop table if exists partmap')
#                self.catalog.connection.execute('alter table newpartmap rename to partmap')
#            self.catalog.commit()
#            conn.close()
#            self.partList.read()
#            
#
#    #----------------------------------------------- save partmap database file
#    def saveDBButtonCB(self,widget):
#        self.partList.write()
#        self.saveDBButton.set_sensitive(False)
#        self.modified = False
#    
#    #---------------------------------------- save partmap database as new file
#    def exportDBButtonCB(self,widget):
#        response = self.saveDialog.run()
#        self.saveDialog.hide()
#        if response == gtk.RESPONSE_ACCEPT:
#            partmapfile = self.saveDialog.get_filename()
#            if os.path.splitext(partmapfile)[1] == '':
#                partmapfile += '.sq3'
#        else:
#            return
#                
#        conn = sqlite3.connect(partmapfile)        
#        c = conn.cursor()
#
#        # re-create table with part map list
#        c.execute('drop table if exists partmap')
#        
#        c.execute('create table partmap (lddid integer, blid text,bltoldd bool,lddtobl bool,notes text, lmod bool, lastmod integer)')
#        
#        for row in self.partList:
#            c.execute('insert into partmap(lddid,blid,bltoldd,lddtobl,notes,lmod,lastmod) values(?,?,?,?,?,?,?)',
#                      (row[0],row[1],row[2],row[3],row[4].decode('utf-8'),row[5],row[6])
#                      )
#        conn.commit()
#        conn.close()
        
    #------------------------------------------------- new part button callback
    def newButtonCB(self,widget):
        dlg = PMEditDialog(title="New part mapping",parent=self,catalog=self.catalog,imgCache=self.imgCache)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try:
                lddid = int(dlg.lddid.get_text())
            except ValueError:
                lddid = 0
            if lddid == 0 and (dlg.ldd2bl.get_active() or dlg.bl2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD ID must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            blid = dlg.blid.get_text().lower()
            if len(blid) == 0 and (dlg.ldd2bl.get_active() or dlg.bl2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Bricklink ID cannot be empty")
                msg.set_title('Wrong BLink ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            if lddid == 0 and len(blid) == 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="At least BL ID or LDD ID must be given")
                msg.set_title('Empty IDs')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return

            d = self.catalog.partMapDuplicated(lddid,blid)
            if len(d) > 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                msg.set_markup("Duplicated part mapping\nLDD ID: <b>"+str(lddid)+
                                "</b> BL ID: <b>"+blid+'</b> ID: <b>'+str(d[0][2])+'</b>\n')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            b = self.catalog.partMapExistsB2L(blid)
            l = self.catalog.partMapExistsL2B(lddid)
            if (dlg.bl2ldd.get_active() and len(b) > 0) or (dlg.ldd2bl.get_active() and len(l) > 0):
                #------------- only one part can be converted from LDD to BL
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                if len(b) > 0:
                    msg.set_markup("Conversion from BL to LDD for this part already exists: LDD ID <b>"+str(b[0][0])+
                               "</b> BL ID: <b>"+b[0][1]+'</b> ID: <b>'+str(b[0][2])+'</b>')
                else:
                    msg.set_markup("Conversion from LDD to BL for this part already exists: LDD ID <b>"+str(l[0][0])+
                               "</b> BL ID: <b>"+l[0][1]+'</b> ID: <b>'+str(l[0][2])+'</b>')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return

            rowid = self.catalog.insertPartMapping(lddid,blid,dlg.bl2ldd.get_active(),
                                                   dlg.ldd2bl.get_active(),dlg.notes.get_text().decode('utf-8'),True)
            self.catalog.commit()
            self.partList.append(row=(lddid,blid,dlg.bl2ldd.get_active(),
                                      dlg.ldd2bl.get_active(),dlg.notes.get_text(),True,int(time.time()),rowid))

    
    #------------------------------------------------------- delete part button
    def deleteButtonCB(self,widget):
        selPath = ()
        sel = self.partListView.get_selection()
        if sel.count_selected_rows() == 0:
            selPath = self.partListView.get_cursor()
            if selPath[0] == None:
                return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete part mapping')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #-------------------------------------- delete part mapping from DB
            if sel.count_selected_rows() > 0:
                while sel.count_selected_rows() > 0:
                    p = sel.get_selected_rows()[1][0]
                    r = self.partList.get_iter(p)
                    rowid, = self.partList.get(r,7)
                    self.catalog.deletePartMap(rowid)
                    self.partList.remove(r)
            else:
                selRow = self.partList.get_iter(selPath[0])
                rowid, = self.partList.get(selRow,7)
                self.catalog.deletePartMap(rowid)
                self.partList.remove(selRow)
            self.catalog.commit()

    
    def editButtonCB(self,widget,selPath=None,selRow=None):
        if selPath == None:
            sel = self.partListView.get_selection()
            if sel.count_selected_rows() > 1:
                # only one at time
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Please select only one element to edit")
                msg.set_title('Too many selected')
                msg.run()
                msg.hide()
                msg.destroy()
                return            
            selPath = self.partListView.get_cursor()
        if selPath[0] == None:
            return
        selRow = self.partList.get_iter(selPath[0])
        lddid,blid,bl2ldd,ldd2bl,notes,rowid = self.partList.get(selRow,0,1,2,3,4,7)        
        dlg = PMEditDialog(title='Edit part mapping',parent=self,catalog=self.catalog,imgCache=self.imgCache)
        dlg.lddid.set_text(str(lddid))
        dlg.bl2ldd.set_active(bl2ldd)
        dlg.ldd2bl.set_active(ldd2bl)
        dlg.blid.set_text(blid)
        dlg.notes.set_text(notes)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            # save edits
            try:
                lddid = int(dlg.lddid.get_text())
            except ValueError:
                lddid = 0
            if lddid == 0 and (dlg.ldd2bl.get_active() or dlg.bl2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="LDD ID must be numeric and greater than 0")
                msg.set_title('Wrong LDD ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            blid = dlg.blid.get_text().lower()
            if len(blid) == 0 and (dlg.ldd2bl.get_active() or dlg.bl2ldd.get_active()):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Bricklink ID cannot be empty")
                msg.set_title('Wrong BLink ID')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            if lddid == 0 and len(blid) == 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="At least BL ID or LDD ID must be given")
                msg.set_title('Empty IDs')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            d = self.catalog.partMapDuplicated(lddid,blid,rowid)
            if len(d) > 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                msg.set_markup("Duplicated part mapping\nLDD ID: <b>"+str(lddid)+
                                "</b> BL ID: <b>"+blid+'</b> ID: <b>'+str(d[0][2])+'</b>')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            b = self.catalog.partMapExistsB2L(blid,rowid)
            l = self.catalog.partMapExistsL2B(lddid,rowid)
            if (dlg.bl2ldd.get_active() and len(b) > 0) or (dlg.ldd2bl.get_active() and len(l) > 0):
                #------------- only one part can be converted from LDD to BL
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL)
                if len(b) > 0:
                    msg.set_markup("Conversion from BL to LDD for this part already exists: LDD ID <b>"+str(b[0][0])+
                               "</b> BL ID: <b>"+b[0][1]+'</b> ID: <b>'+str(b[0][2])+'</b>')
                else:
                    msg.set_markup("Conversion from LDD to BL for this part already exists: LDD ID <b>"+str(l[0][0])+
                               "</b> BL ID: <b>"+l[0][1]+'</b> ID: <b>'+str(l[0][2])+'</b>')
                msg.set_title('Error')
                response = msg.run()
                msg.hide()
                msg.destroy()
                return
            
            self.catalog.updatePartMap(lddid,blid,dlg.bl2ldd.get_active(),dlg.ldd2bl.get_active(),dlg.notes.get_text().decode('utf-8'),rowid)
            self.partList[selPath[0]] = (lddid,
                                         dlg.blid.get_text(),
                                         dlg.bl2ldd.get_active(),
                                         dlg.ldd2bl.get_active(),
                                         dlg.notes.get_text(),
                                         True,
                                         int(time.time()),
                                         rowid
                                         )
            self.catalog.commit()
            

    #-------------------------------------------------------- interface building
    def __init__(self,title=None, parent=None, flags=0, buttons=None, catalog=None,imageCache=None):
        super(BrickPartsMaint,self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        
        self.catalog = catalog
        self.dialogTitle = title
        self.modified = False
        self.imgCache = imageCache
        
        #----------------------------------------------------------- main window
        self.connect('delete-event',self.closeCB)
        self.connect('response',self.responseCB)         
        #-------------------------------------------------------- primary layout
        self.vBox = self.get_content_area()
        self.vBox.set_spacing(4)
         
        #------------------------------------------------------------ handle box
        self.handleBox = gtk.HandleBox()
        self.vBox.pack_start(self.handleBox,False)
        
        #--------------------------------------------------- toolbar and buttons
        self.toolBar = gtk.Toolbar()
        self.handleBox.add(self.toolBar)
        
#        self.importDBButton = gtk.ToolButton()
#        self.importDBButton.set_stock_id(gtk.STOCK_OPEN)
#        self.importDBButton.set_tooltip_text("Import partmap database")
#        self.importDBButton.connect('clicked',self.importDBButtonCB)
#        self.toolBar.insert(self.importDBButton,-1)
#        
#        self.saveDBButton = gtk.ToolButton()
#        self.saveDBButton.set_stock_id(gtk.STOCK_SAVE)
#        self.saveDBButton.set_tooltip_text("Save partmap database in main catalog")
#        self.saveDBButton.set_sensitive(False)
#        self.saveDBButton.connect('clicked',self.saveDBButtonCB)
#        self.toolBar.insert(self.saveDBButton,-1)
#        
#        self.exportDBButton = gtk.ToolButton()
#        self.exportDBButton.set_stock_id(gtk.STOCK_SAVE_AS)
#        self.exportDBButton.set_tooltip_text("Export partmap database as new file")
#        self.exportDBButton.connect('clicked',self.exportDBButtonCB)
#        self.toolBar.insert(self.exportDBButton,-1)
#        
#        self.updateDBButton = gtk.ToolButton()
#        self.updateDBButton.set_stock_id(gtk.STOCK_REFRESH)
#        self.updateDBButton.set_label('Update')
#        self.updateDBButton.set_tooltip_text('Update partmap database from file')
#        self.updateDBButton.connect('clicked',self.updateDBButtonCB)
#        self.toolBar.insert(self.updateDBButton,-1)

        self.sep1 = gtk.SeparatorToolItem()
        self.sep1.set_expand(True)
        self.sep1.set_draw(False)
        self.toolBar.insert(self.sep1,-1)
        
        self.exitButton = gtk.ToolButton()
        self.exitButton.set_stock_id(gtk.STOCK_QUIT)
        self.exitButton.set_tooltip_text("Quit to main")
        self.exitButton.connect('clicked',self.quitCB)
        self.toolBar.insert(self.exitButton,-1)
        
#        #-------------------------------------------- various dialogs and popups
#        self.openDialog = gtk.FileChooserDialog("Select partmap file",self.window,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        
#        self.saveDialog = gtk.FileChooserDialog("Save as partmap file",self.window,
#            action=gtk.FILE_CHOOSER_ACTION_SAVE,
#            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
#        self.saveDialog.set_do_overwrite_confirmation(True)
#        
#        #------------------------------------ add a filter for SQLite3 files
#        sqliteFilter = gtk.FileFilter()
#        sqliteFilter.add_pattern("*.sq3")
#        sqliteFilter.set_name("SQlite database files")
#        self.openDialog.add_filter(sqliteFilter)
#
#        #-------------------------------------- add a filter for any (all) files
#        allFilter = gtk.FileFilter()
#        allFilter.add_pattern("*")
#        allFilter.set_name("All files")
#        self.openDialog.add_filter(allFilter)
#        self.saveDialog.add_filter(allFilter)
        
        #----------------------------------------------- list store for treeview
        self.partList = PartList(self.catalog)
        self.partList.read()
        
        self.partListView = PartListView(self.partList)
        self.partListView.connect('row-activated',self.editButtonCB)
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        self.vBox.add(self.listWindow)
        self.vBox.pack_start(gtk.HSeparator(),False)
        self.filterFrame = gtk.Frame("Filter part by")
 
        self.table = gtk.Table(3,4)
        self.table.set_row_spacings(4)
        self.table.set_col_spacings(4)
        self.table.set_border_width(3)
        self.filterFrame.add(self.table)
        #---------------------------------------- place widget to select filters
        #-------------------------------------------- filter by modified locally
        self.filterLmod = gtk.CheckButton("Locally modified")
        #------------------------------------------------- filter by no ldd code
        self.filterNoLddid = gtk.CheckButton("No LDD ID code")
        self.filterNoLddid.connect('toggled',self.noLddidCB)
        #------------------------------------------------ filter by no blid code
        self.filterNoBlid = gtk.CheckButton("No BL ID code")
        self.filterNoBlid.connect('toggled',self.noBlidCB)
        #----------------------------------- filter by no bl2ldd or lddtobl flag
        self.filterL2B = gtk.CheckButton("No convert LDD > BL")
        self.filterB2L = gtk.CheckButton("No convert BL > LDD")
        self.filterLddid = gtk.Entry()
        self.filterBlid = gtk.Entry()
        self.filterLddidLabel = gtk.Label("LDD ID:")
        self.filterLddidLabel.set_alignment(1,0.5)
        self.filterBlidLabel = gtk.Label("BL ID:")
        self.filterBlidLabel.set_alignment(1,0.5)
    
        #--------------------------------------- filter by string in description
        self.filterDescr = gtk.Entry()
        self.filterDescrLabel = gtk.Label("Notes contains string:")
        self.filterDescrLabel.set_alignment(1,0.5)
        self.filterRefilter = gtk.Button("Apply filter")
        self.filterReset = gtk.Button("Reset")
        
        self.table.attach(self.filterLddidLabel,1,2,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterLddid,2,3,0,1,yoptions=gtk.FILL,xpadding=3)
        self.table.attach(self.filterNoLddid,1,3,1,2,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterL2B,1,3,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterBlidLabel,3,4,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterBlid,4,5,0,1,yoptions=gtk.FILL,xpadding=3)
        self.table.attach(self.filterNoBlid,3,5,1,2,yoptions=gtk.FILL,xpadding=3)
        self.table.attach(self.filterB2L,3,5,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterDescrLabel,5,6,0,1,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterDescr,6,7,0,1,yoptions=gtk.FILL,xpadding=3)
        self.table.attach(self.filterLmod,5,6,1,2,xoptions=gtk.FILL,yoptions=gtk.FILL)
        self.table.attach(self.filterRefilter,6,7,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL,xpadding=3)
        self.table.attach(self.filterReset,5,6,2,3,xoptions=gtk.FILL,yoptions=gtk.FILL,xpadding=3)
        self.vBox.pack_start(self.filterFrame,False)
        
        self.filterRefilter.connect('clicked',self.reFilterCB)
        self.filterReset.connect('clicked',self.resetFilterCB)
        
        self.hButtonBox = gtk.HButtonBox()
        self.hButtonBox.set_layout(gtk.BUTTONBOX_END)
        
        self.newButton = gtk.Button(stock=gtk.STOCK_NEW)
        self.newButton.connect('clicked',self.newButtonCB)
        self.editButton = gtk.Button(stock=gtk.STOCK_EDIT)
        self.editButton.connect('clicked',self.editButtonCB)
        self.deleteButton = gtk.Button(stock=gtk.STOCK_DELETE)
        self.deleteButton.connect('clicked',self.deleteButtonCB)
        
        self.hButtonBox.pack_start(self.newButton,False)
        self.hButtonBox.pack_start(self.editButton,False)
        self.hButtonBox.pack_start(self.deleteButton,False)
        
        self.vBox.pack_end(self.hButtonBox,False,False)
        
        self.set_default_size(600,500)
        
        self.vBox.show_all()
        
