'''
Created on 21/ott/2011

checks for "existence" rule for parts

@author: mario
'''

import gtk
import CatalogStore
import CatalogView



class SanityCheck(gtk.Dialog):
    '''
    classdocs
    '''
    

    def viewErrorCB(self,widget):
        selPath = widget.get_cursor()[0]
        if selPath == None:
            return
        lddid = self.warningList[selPath][1]
        msg,lvl,hint = self.warningMessages[self.warningList[selPath][0]]
        brickFile = self.imgCache.brickImgLDD(lddid)
        if brickFile == None:
            self.needImage.clear()
        else:            
            if self.needImage != None:
                r = self.needImage.get_allocation()
                x = r.width
                y = x
                img = gtk.gdk.pixbuf_new_from_file(brickFile)
                img = img.scale_simple(x,y,gtk.gdk.INTERP_BILINEAR)
                self.needImage.set_from_pixbuf(img)
        self.warnMsg.set_text(msg)
        if lvl == 1:
            self.warnLevel.set_text('minor')
        elif lvl == 2:
            self.warnLevel.set_text('moderate')
        elif lvl == 3:
            self.warnLevel.set_text('critical')
        else:
            self.warnLevel.set_text('unknown')
        self.warnAction.set_text(hint)
        
                

    def listWarnings(self):
        c,msg,rank,hint = self.catalog.sanityChecks()
        self.warningList.clear()
        self.warningMessages = dict()
        
        for r in c:
            #------------------ checks if lddid is numeric, i.e. is a valid part
            try:
                lddid = int(r[1])
            except ValueError:
                lddid = 0
            self.warningList.append(row=(r[0],   # rowid
                                        lddid,   # lddid
                                        r[2],   # lddcolorid
                                        r[3],   # decorated
                                        r[4],   # descr
                                        r[5],   # qty
                                        r[6],   # ldd color name
                                        r[7]   # hex rgb
                                        ))
            self.warningMessages[r[0]] = (msg,rank,hint)
        



    def __init__(self,title="Parts sanity check", parent=None, flags=0, 
                 buttons=(gtk.STOCK_CLOSE,gtk.RESPONSE_ACCEPT),
                 catalog=None,brickList=None,imgCache=None):
        '''
        Constructor
        '''
        super(SanityCheck, self).__init__(title=title, parent=parent,flags=flags,buttons=buttons)
        self.catalog = catalog
        self.brickList = brickList
        self.imgCache = imgCache
        self.warningMessages = dict()
        
        self.warningList = CatalogStore.CatalogStore()
        self.table = gtk.Table(2,2)
        self.table.set_row_spacings(3)
        self.labBuildPerc = gtk.Label(' ')
        self.listWarnings()
        self.warnView = CatalogView.CatalogView(self.warningList)
        self.warnView.connect('cursor-changed',self.viewErrorCB)
        self.vPaned = gtk.VBox(spacing=4)
        self.vBox = self.get_content_area()
        self.sWindow = gtk.ScrolledWindow()
        self.sWindow.set_border_width(3)
        self.sWindow.add(self.warnView)
        #------------------------------------------------ message area
        self.warnMsg = gtk.Label(' ')
        self.warnMsg.set_alignment(0,0.5)
        self.frame1 = gtk.Frame('Parts with errors')
        self.hbox1 = gtk.HBox()
        self.hbox1.set_spacing(3)
        self.frame2 = gtk.Frame('Error explaining')
        table = gtk.Table(3,2)
        table.set_row_spacings(3)
        table.set_col_spacings(3)
        table.set_border_width(4)
        table.attach(gtk.Label('Type:'),0,1,0,1,yoptions=gtk.SHRINK)
        table.attach(self.warnMsg,1,2,0,1,yoptions=gtk.SHRINK)
        self.warnLevel = gtk.Label(' ')
        self.warnLevel.set_alignment(0,0.5)
        table.attach(gtk.Label('Severity:'),0,1,1,2,yoptions=gtk.SHRINK)
        table.attach(self.warnLevel,1,2,1,2,yoptions=gtk.SHRINK)
        self.warnAction = gtk.Label(' ')
        self.warnAction.set_alignment(0,0.5)
        table.attach(gtk.Label('Hint:'),0,1,2,3,yoptions=gtk.SHRINK)
        table.attach(self.warnAction,1,2,2,3,yoptions=gtk.SHRINK)
        self.frame1.set_border_width(3)
        self.frame2.set_border_width(3)
        self.frame1.add(self.sWindow)
        self.frame2.add(table)
        self.vPaned.pack_start(self.frame1)
        self.vPaned.pack_start(self.frame2,expand=False)
        self.table.attach(self.vPaned,0,1,0,2)
        self.needImage = gtk.Image()
        frame = gtk.Frame('Brick shape')
        frame.set_border_width(3)
        self.needImage.set_size_request(200,200)
        frame.add(self.needImage)
        self.table.attach(frame,1,2,0,1,xoptions=gtk.SHRINK,yoptions=gtk.SHRINK)
        self.set_default_size(min(gtk.gdk.screen_width()-400,800),
                              min(gtk.gdk.screen_height()-300,500))
        self.vBox.pack_start(self.table,True)
        self.vBox.show_all()
        


        