#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
  pyBrickUtils - 
  Copyright (C) 2011 Mario Pascucci http://bricksnspace.wordpress.com/ <mpascucci gmail com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  or
  visit the URL http://www.fsf.org/

'''


#===============================================================================
# Imports
#===============================================================================


import xml.dom.minidom
import xml.parsers.expat
try: 
    import glib
    import gtk
    import pygtk
except ImportError:
    print ("Please install python-gtk bindings.\nUsually pygtk or python-gtk or pygtk2 on Linux.\n"+
           "On Mac OSX>=10.6 you can get a package here:\nhttp://macpkg.sourceforge.net/")
    exit(1)
import platform
import os.path
import warnings
import zipfile


# BrickUtils modules
from BrickUtils import ColorMaintDialog, progInfo
from BrickUtils import PartsMaintDialog
from BrickUtils import LDrawMaintDialog
from BrickUtils import UpdateBLParts
from BrickUtils import UpdateBLColors
from BrickUtils import ImportBLSetDialog
from BrickUtils.ImportLDDProjectDialog import ImportLDDProject
from BrickUtils import BrickListStore
from BrickUtils import BrickListView
from BrickUtils import PartEditDialog
from BrickUtils import CatalogStore
from BrickUtils import CatalogView
from BrickUtils import SetSelection
from BrickUtils import SetView
from BrickUtils import SetStore
from BrickUtils import CatalogDB
from BrickUtils import CatalogImportExport
from BrickUtils import UpgradeDatabases
from BrickUtils import ParseBLCodesXML
from BrickUtils.LDDComposite import LDDComposite
from BrickUtils.BrickEditDialog import BrickEdit
from BrickUtils.CheckBuildableDialog import CheckBuildable
from BrickUtils.BrickException import BrickException
from BrickUtils.BLExportXML import BLExportXML
from BrickUtils.ImportLDrawDialog import ImportLDraw
from BrickUtils.BrickImage import BrickImage
from BrickUtils.ProgressDialog import ProgressDialog
from BrickUtils.ExportHTML import ExportHTML
from BrickUtils.DBUpdate import DBUpdate
from BrickUtils.LDrawImportLib import LDrawImportLib
from BrickUtils.SanityChecksDialog import SanityCheck

pygtk.require('2.0')



class BrickUtilsApp:
    
    def updateBrickList(self):
        self.blCountLab.set_text(str(len(self.currList)))
        self.blCountParts.set_text(str(self.catalog.countParts()))
        
        
    def updateCatalogInfo(self):
        c,t = self.catalog.getCatalogCount()
        self.catCount.set_text(str(c))
        self.catCountParts.set_text(str(t))
    
    #--------------------------------------------------------- callbacks methods
    def quitCB(self,widget):
        self.catalog.close()
        gtk.main_quit()

    #---------------------------------------------- delete a part from BrickList
    def blDeleteButtonCB(self,widget):
        sel = self.partListView.get_selection()
        if sel.count_selected_rows() == 0:
            selPath = self.partListView.get_cursor()[0]
            if selPath == None:
                return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Selected part will be deleted and cannot be recovered.")
        msg.set_title('Delete part from brick list')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            #---------------------------------- delete bricks from working list
            if sel.count_selected_rows() > 0:
                while sel.count_selected_rows() > 0:
                    p = sel.get_selected_rows()[1][0]
                    r = self.currList.get_iter(p)
                    self.currList.remove(r)
            else:
                self.currList.removePath(selPath)
            self.currList.write()
            self.updateBrickList()
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Part removed.')
            self.brickImage.clear()
            

    #----------------------------------------- removes counterparts marked items
    def blRemoveCounterpartsCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Part marked as 'counterparts' will be deleted and cannot be recovered.")
        msg.set_title('Delete parts from brick list')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            self.currList.removeCounterparts()
            self.updateBrickList()
            self.currList.write()
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Parts marked as \'counterpart\' removed.')


    #----------------------------------------- removes extra marked items
    def blRemoveExtraCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="Part marked as 'extra' will be deleted and cannot be recovered.")
        msg.set_title('Delete parts from brick list')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            self.currList.removeExtra()
            self.updateBrickList()
            self.currList.write()
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Parts marked as \'extra\' removed.')


    def blAddPartCB(self,widget):
        dlg = BrickEdit(title='Add new part',editFlag=False,
                        parent=self.window,flags=gtk.DIALOG_MODAL,
                        catalog=self.catalog,currList=self.currList,imgCache=self.imageCache)
        r = dlg.run()
        dlg.hide()
        if r == gtk.RESPONSE_ACCEPT:
            if len(dlg.part) > 0:
                dlg.part[13] = self.currList.getMaxPartID() + 1
                self.catalog.add2CurrList(dlg.part)
                self.catalog.commit()
                self.currList.read()
                self.updateBrickList()
                self.brickImage.clear()
                

            
    def blEditPartCB(self,widget,selPath=None,selRow=None):
        if selPath == None:
            sel = self.partListView.get_selection()
            if sel.count_selected_rows() > 1:
                # only one at time
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Please select only one element to edit")
                msg.set_title('Too many selected')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            selPath = self.partListView.get_cursor()[0]
        if selPath == None:
            return
        dialog = PartEditDialog.PartEditDialog(parent=self.window,store=self.currList,
                                               view=self.partListView,
                                               catalog=self.catalog,imgCache=self.imageCache)
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_ACCEPT:
            self.currList.write()
            self.updateBrickList()
            self.brickImage.clear()
            

    def blEmptyListCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK_CANCEL,flags=gtk.DIALOG_MODAL,
                                        message_format="All part in list will be deleted and cannot be recovered.")
        msg.set_title('Delete ALL parts from list')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_OK:        
            self.currList.clear()
            self.updateBrickList()
            self.currList.write()
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Part list is now empty.')
            self.brickImage.clear()
            
    
    def blSaveListCB(self,widget):
        self.currList.write()
        self.status.pop(self.sbContext)
        self.status.push(self.sbContext,'List saved.')
        
    def blAdd2CatalogCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                        message_format="Delete parts from brick list after added to Catalog?")
        msg.set_title('Delete after add')
        response = msg.run()
        msg.hide()
        msg.destroy()
        if response == gtk.RESPONSE_YES:
            self.addToCatalog(delpart=True)
            self.currList.write()
            self.brickImage.clear()
            self.updateBrickList()
        elif response == gtk.RESPONSE_NO:
            self.addToCatalog(delpart=False)
            self.updateBrickList()
        self.updateCatalogInfo()


    def blAdd2CatalogSetCB(self,widget):
#        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
#                                        message_format="Delete parts from brick list after added to Catalog?")
#        msg.set_title('Delete after add')
#        delbricks = msg.run()
#        msg.hide()
#        msg.destroy()
        dlg = SetSelection.Dialog(parent=self.window,catalog=self.catalog)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            if dlg.addTypeMoc.get_active():
                setid = '@' + dlg.mocName.get_text()
            else:
                c = dlg.resview.get_cursor()[0]
                i = dlg.reslist.get_iter(c)
                setid = dlg.reslist.get(i,1)[0]
            addExtra = dlg.addExtra.get_active()
            if dlg.delParts.get_active():
                self.addToSet(setid,True,addExtra)
                self.currList.write()
                self.brickImage.clear()
            else:
                self.addToSet(setid,False,addExtra)
            self.setList.read()
            self.updateBrickList()
            self.updateCatalogInfo()

    '''
    checks if we have all bricks in current bricklist, so we can build a model
    '''
    def blCheckBuildableCB(self,widget):
        if len(self.currList) == 0:
            return
        dlg = CheckBuildable(parent=self.window,catalog=self.catalog,brickList=self.currList,imgCache=self.imageCache)
        dlg.run()
        dlg.hide()

    '''
    do some sanity checks on brick list
    '''
    def blSanityChecksCB(self,widget):
        if len(self.currList) == 0:
            return
        dlg = SanityCheck(parent=self.window,catalog=self.catalog,brickList=self.currList,imgCache=self.imageCache)
        dlg.run()
        dlg.hide()
        

    def setDeleteCB(self,widget):
        sel = self.setView.get_selection()
        if sel.count_selected_rows() == 0:        
            selPath = self.setView.get_cursor()[0]
            if selPath == None:
                return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                        message_format="Delete selected set?")
        msg.set_title('Delete set')
        delset = msg.run()
        msg.hide()
        msg.destroy()
        if delset != gtk.RESPONSE_YES:
            return
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                        message_format="Remove bricks from catalog?")
        msg.set_title('Delete bricks')
        delbricks = msg.run()
        msg.hide()
        msg.destroy()
        if sel.count_selected_rows() > 0:
            while sel.count_selected_rows() > 0:
                p = sel.get_selected_rows()[1][0]
                r = self.setList.get_iter(p)
                setid = self.setList.get(r,0)[0]
                if delbricks == gtk.RESPONSE_YES:
                    #---------------------------------------- delete bricks from catalog
                    blist = self.catalog.getAllBrickSet(setid)
                    for brick in blist:
                        self.catalog.removeBricks(brick[2],brick[3])
                self.catalog.delAllBrickSet(setid)
                self.catalog.deleteSet(setid)
                self.setList.remove(r)
        else:
            selrow = self.setList.get_iter(selPath)
            setid = self.setList.get(selrow,0)[0]
            if delbricks == gtk.RESPONSE_YES:
                #---------------------------------------- delete bricks from catalog
                blist = self.catalog.getAllBrickSet(setid)
                for brick in blist:
                    self.catalog.removeBricks(brick[2],brick[3])
            self.catalog.delAllBrickSet(setid)
            self.catalog.deleteSet(setid)
            self.setList.remove(selrow)
        self.catalog.commit()
        self.catList.read()
        self.updateCatalogInfo()

    '''
    add bricks in current list to set catalog
    '''
    def addToSet(self,setid,delpart=False,addExtra=True):
        if len(self.currList) == 0:
            return
        idx = BrickListStore.getIndex()
        setrowid = self.catalog.insertSet(setid)
            
        #-------------------------------------- adds brick to brick-to-set table
        for r in self.currList:
            if r[idx['lddid']] == '-':
                continue
            if int(r[idx['lddcolorid']]) == 0:
                continue
            if r[idx['extra']] and not addExtra:
                continue 

            db = self.catalog.getBrick(r[idx['lddid']],r[idx['lddcolorid']],r[idx['decorated']])
                
            if len(db) == 0:
                #----------------------------------------- it's a new brick type
                brickid = self.catalog.insert2Catalog(r[idx['lddid']],
                                                      r[idx['lddcolorid']],
                                                      r[idx['decorated']],
                                                      r[idx['descr']],
                                                      0
                                                      )       
            elif len(db) == 1:
                #----------------------- update quantity of already owned bricks            
                brickid = db[0][0]
            else:
                raise BrickException('Duplicate brick type in table catalog'+repr(db))
            #------------------------------------------- search for brick in set
            db = self.catalog.getBrickSet(setrowid,brickid)
            if len(db) == 0:
                #---------------------------------------------- it's a new brick
                self.catalog.insert2BrickSet(setrowid,brickid,r[idx['qty']])
            elif len(db) == 1:
                rowid = db[0][0]
                n = db[0][3] + r[idx['qty']]
                #------------------------------------------------ it's an update
                self.catalog.updateBrickSetQty(rowid, n)
            else:
                raise BrickException('Duplicate record in table setbricks: brick id '+
                                     brickid+' setid: '+setrowid+' '+repr(db))
        self.catalog.commit()            
        self.addToCatalog(delpart,addExtra)
        self.updateCatalogInfo()
            
    
    def addToCatalog(self,delpart=False,addExtra=True):
        if len(self.currList) == 0:
            return
        totpart = 0
        newpart = 0
        nopartid = 0
        nocolorid = 0
        noextra = 0
        delList = ()
        idx = BrickListStore.getIndex()
        
        for r in self.currList:

            if r[idx['lddid']] == '-':
                nopartid += 1
                continue
            if int(r[idx['lddcolorid']]) == 0:
                nocolorid += 1
                continue 
            if r[idx['extra']] and not addExtra:
                noextra += 1
                continue 
            totpart += 1    
            db = self.catalog.getBrick(r[idx['lddid']],r[idx['lddcolorid']],r[idx['decorated']])
            if len(db) == 0:
                #----------------------------------------- it's a new brick type
                newpart += 1
                self.catalog.insert2Catalog(r[idx['lddid']],
                                            r[idx['lddcolorid']],
                                            r[idx['decorated']],
                                            r[idx['descr']],
                                            r[idx['qty']])
            elif len(db) == 1:
                #----------------------- update quantity of already owned bricks
                if db[0][4] == 0:
                    newpart += 1
                quantity = db[0][4] + r[idx['qty']]
                self.catalog.updateBrickQty(db[0][0],quantity)
            else:
                raise BrickException('Duplicate brick record in table catalog : '+repr(db))                                    
            if delpart:
                delList += (r[idx['partid']],)
            
        self.catalog.commit()
        if delpart:
            self.currList.removeParts(delList)
            self.currList.write()
            self.updateBrickList()
        #--------------------------------------------------- update catalog view
        self.catList = CatalogStore.CatalogStore(self.catalog)
        self.catList.read()
        self.catView.set_model(self.catList)
        msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                message_format=str(totpart)+" parts added to catalog. New parts: "+
                                str(newpart)+". Part not added: "+str(nopartid+nocolorid+noextra)+
                                " ("+str(nopartid)+" missing part ID, "+str(nocolorid)+" missing color ID, "+str(noextra)+" extra parts).")
        msg.set_title('Add to Catalog complete.')
        msg.run()
        msg.hide()
        msg.destroy()
        self.updateCatalogInfo()
                

    def catEditCB(self,widget,selPath=None,selRow=None):
        if selPath == None:
            sel = self.catView.get_selection()
            if sel.count_selected_rows() > 1:
                # only one at time
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_CANCEL,flags=gtk.DIALOG_MODAL,
                                                message_format="Please select only one element to edit")
                msg.set_title('Too many selected')
                msg.run()
                msg.hide()
                msg.destroy()
                return
            selPath = self.catView.get_cursor()[0]
        if selPath == None:
            return
        selrow = self.catList.get_iter(selPath)
        brickid,lddid,lddcolor,decor,descr,qty = self.catList.get(selrow,0,1,2,3,4,5)    # brick id, or rowid in table        
        dlg = BrickEdit(title='Edit Brick',
                        parent=self.window,flags=gtk.DIALOG_MODAL,
                        catalog=self.catalog,imgCache=self.imageCache)
        dlg.decoratedBrick.set_active(decor)
        dlg.lddid.set_text(str(lddid))
        dlg.setBrickColor(lddcolor)
        dlg.descr.set_text(descr)
        dlg.quantity.set_value(qty)
        resp = dlg.run()
        dlg.hide()
        if resp == gtk.RESPONSE_ACCEPT:
            try:
                lddid = int(dlg.lddid.get_text())
                if lddid == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Part ID must be numeric and not zero.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            i = dlg.lddcolor.get_active_iter()
            if i != None:
                model = dlg.lddcolor.get_model()
                lddcolor = model.get(i,0)[0]
            else:
                lddcolor = 0

            if lddcolor < 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Undefined color for brick, please choose a valid one.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            c = self.catalog.getBrick(lddid, lddcolor, dlg.decoratedBrick.get_active())
            if len(c) != 0 and c[0][0] != brickid:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="There already is a brick in catalog like this, no duplicated allowed.")
                msg.run()
                msg.hide()
                msg.destroy()
                return                            
            lddname,rgb = self.catalog.getLDDColor(lddcolor)
            self.catList[selPath] = (brickid,
                                     lddid,
                                     lddcolor,
                                     dlg.decoratedBrick.get_active(),
                                     dlg.descr.get_text(),
                                     dlg.quantity.get_value_as_int(),
                                     lddname,
                                     rgb if rgb != None else '#fff'
                                     )
            self.catalog.updateBrick(brickid,lddid,lddcolor,dlg.decoratedBrick.get_active(),
                                     dlg.descr.get_text(),dlg.quantity.get_value_as_int())
            self.catalog.commit()
            self.catView.cursorChangedCB(self.catView)
            self.updateCatalogInfo()
    
    def catDeleteCB(self,widget):
        sel = self.catView.get_selection()
        if sel.count_selected_rows() == 0:
            selPath = self.catView.get_cursor()[0]
            if selPath == None:
                return
            selrow = self.catList.get_iter(selPath)
            brickid = self.catList.get(selrow,0)[0]    # brick id, or rowid in table
            count = self.catalog.countSetWithBrick(brickid)
        else:
            count = 0
            for sr in sel.get_selected_rows()[1]:
                r = self.catList.get_iter(sr)
                b = self.catList.get(r,0)[0]
                count += self.catalog.countSetWithBrick(b)
        if count == 0:
            message = "Delete selected brick (operation cannot be undone)?"
        else:
            message = ("Delete selected brick (operation cannot be undone)?" + 
                       " NOTE: brick is listed in one or more set(s).")
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                        message_format=message)
        msg.set_title('Delete brick')
        delbrick = msg.run()
        msg.hide()
        msg.destroy()
        if delbrick != gtk.RESPONSE_YES:
            return
        if sel.count_selected_rows() > 0:
            while sel.count_selected_rows() > 0:
                r = self.catList.get_iter(sel.get_selected_rows()[1][0])
                b = self.catList.get(r,0)[0]
                self.catalog.deleteBrick(b)
                self.catList.remove(r)
        else:
            self.catalog.deleteBrick(brickid)
            self.catList.remove(selrow)
        self.catalog.commit()
        self.catalogImage.clear()
        self.updateCatalogInfo()
    
    
    def catDelZeroCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                message_format="Delete brick descriptions with zero bricks in catalog (operation cannot be undone)?")
        msg.set_title('Delete empty bricks')
        delbrick = msg.run()
        msg.hide()
        msg.destroy()
        if delbrick != gtk.RESPONSE_YES:
            return
        self.catalog.deleteEmptyBricks()
        self.catalog.commit()
        self.catList.read()
        self.catalogImage.clear()
        self.updateCatalogInfo()
    
    def catAddBrickCB(self,widget):
        dlg = BrickEdit(title='Add New Brick',editFlag=False,
                        parent=self.window,flags=gtk.DIALOG_MODAL,
                        catalog=self.catalog,imgCache=self.imageCache)        
        resp = dlg.run()
        dlg.hide()
        if resp == gtk.RESPONSE_ACCEPT:
            try:
                lddid = int(dlg.lddid.get_text())
                if lddid == 0:
                    raise ValueError
            except ValueError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Part ID must be numeric and not zero.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            i = dlg.lddcolor.get_active_iter()
            if i != None:
                model = dlg.lddcolor.get_model()
                lddcolor = model.get(i,0)[0]
            else:
                lddcolor = 0

            if lddcolor < 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Undefined color for brick, please choose a valid one.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            
            decor = dlg.decoratedBrick.get_active()
            c = self.catalog.getBrick(lddid, lddcolor, decor)
            if len(c) != 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="There already is a brick in catalog like this, no duplicated allowed.")
                msg.run()
                msg.hide()
                msg.destroy()
                return                
            
            lddname,rgb = self.catalog.getLDDColor(lddcolor)
            brickid = self.catalog.insert2Catalog(lddid,lddcolor,decor,
                                     dlg.descr.get_text(),dlg.quantity.get_value_as_int())
            self.catalog.commit()
            i = self.catList.append(row=(brickid,
                                     lddid,
                                     lddcolor,
                                     decor,
                                     dlg.descr.get_text(),
                                     dlg.quantity.get_value_as_int(),
                                     lddname,
                                     rgb if rgb != None else '#fff'
                                     ))
            p = self.catList.get_path(i)
            self.catView.set_cursor(p)
            self.updateCatalogInfo()
    
    #---------------------------------- reads and update Bricklink parts database
    
    def updBLPartsCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Parts XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Reading Bricklink parts file... ')
                    
            #-------------------------- starts import
            try:
                xmlfile = UpdateBLParts.UpdateBLParts(blxml,self.catalog,self.status,self.sbContext)
                xmlfile.parse()
            except xml.parsers.expat.ExpatError as (code,lineno):
                message="File isn't a valid XML file, or isn't a BrickLink Parts dump. Parser error: '{0}' line: {1}".format(xml.parsers.expat.ErrorString(code), lineno)
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format=message)
                msg.run()
                msg.hide()
                msg.destroy()
            except BrickException as (args):
                message="File isn't a valid XML file, or isn't a BrickLink Parts dump. Parser error: '{0}'".format(args)
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format=message)
                msg.run()
                msg.hide()
                msg.destroy()
            blxml.close()


    #------------------------------- reads and update Bricklink part codes database
    
    def updBLCodesCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Part Codes XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Reading Bricklink part codes file... ')
                    
            #-------------------------- starts import
            try:
                xmlfile = ParseBLCodesXML.UpdateBLCodes(blxml,self.catalog,self.status,self.sbContext)
                xmlfile.parse()
            except xml.parsers.expat.ExpatError as (code,lineno):
                message="File isn't a valid XML file, or isn't a BrickLink Part Codes dump. Parser error: '{0}' line: {1}".format(xml.parsers.expat.ErrorString(code), lineno)
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format=message)
                msg.run()
                msg.hide()
                msg.destroy()
            except BrickException as (args):
                message="File isn't a valid XML file, or isn't a BrickLink Part Codes dump. Parser error: '{0}'".format(args)
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format=message)
                msg.run()
                msg.hide()
                msg.destroy()
            blxml.close()


    #------------------------------- reads and updates Bricklink colors database
    def updBLColorsCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Color XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Reading Bricklink colors file... ')
                    
            # at this point blxml contains a "file" object, ready for parsing
            # parse Bricklink XML dump
            try :
                bldump = xml.dom.minidom.parse(blxml)
            except xml.parsers.expat.ExpatError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a valid XML file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return                
            
            # explore the XML to get elements
            catalog = bldump.getElementsByTagName("CATALOG")
            if catalog.length != 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a Bricklink XML dump.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            
            colorlist = catalog[0].getElementsByTagName('ITEM')

            if colorlist.length < 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Warning: empty Bricklink XML dump, nothing to do...")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            #-------------------------- starts import
            UpdateBLColors.UpdateBLColors(blxml,colorlist,self.catalog,self.status,self.sbContext)
            blxml.close()
    
    #------------------------------------ reads and updates Bricklink Categories
    def updBLCategoriesCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Categories XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Reading Bricklink categories file... ')
                    
            # at this point blxml contains a "file" object, ready for parsing
            # parse Bricklink XML dump
            try :
                bldump = xml.dom.minidom.parse(blxml)
            except xml.parsers.expat.ExpatError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a valid XML file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return                
            
            # explore the XML to get elements
            catalog = bldump.getElementsByTagName("CATALOG")
            if catalog.length != 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a Bricklink XML dump.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            
            catlist = catalog[0].getElementsByTagName('ITEM')

            if catlist.length < 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Warning: empty Bricklink XML dump, nothing to do...")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            
            # create table with color map list
            self.catalog.cleanBLCategoriesTable()
            
            i = 0
    
            for cat in catlist:
                if gtk.events_pending():
                    gtk.main_iteration(block=False)
                try: catId = cat.getElementsByTagName("CATEGORY")[0].firstChild.data
                except AttributeError: catId = None
                
                try: catName = cat.getElementsByTagName("CATEGORYNAME")[0].firstChild.data
                except AttributeError: catName = None 
               
                self.catalog.insertBLCategory(catId,catName)
                
                i+=1
                    
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Finished, categories in database: '+str(i))
        
            self.catalog.commit()
            
            blxml.close()

    #------------------------------------ reads and updates Bricklink Sets
    def updBLSetsCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Sets XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                bldumpfile = self.blXMLfile.get_filename()
                blxml = open(bldumpfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return

            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Reading Bricklink sets file... ')
                    
            # at this point blxml contains a "file" object, ready for parsing
            # parse Bricklink XML dump
            try :
                bldump = xml.dom.minidom.parse(blxml)
            except xml.parsers.expat.ExpatError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a valid XML file.")
                msg.run()
                msg.hide()
                msg.destroy()
                blxml.close()
                return                
            
            # explore the XML to get elements
            catalog = bldump.getElementsByTagName("CATALOG")
            if catalog.length != 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: file isn't a Bricklink XML dump.")
                msg.run()
                msg.hide()
                msg.destroy()
                blxml.close()
                return
            
            setlist = catalog[0].getElementsByTagName('ITEM')

            if setlist.length < 1:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Warning: empty Bricklink XML dump, nothing to do...")
                msg.run()
                msg.hide()
                msg.destroy()
                blxml.close()
                return
            
            # create table with color map list
            self.catalog.cleanBLSetTable()
            
            i = 0
    
            for blset in setlist:
                if gtk.events_pending():
                    gtk.main_iteration(block=False)
                try: setId = blset.getElementsByTagName("ITEMID")[0].firstChild.data
                except AttributeError: setId = None
                
                try: setName = blset.getElementsByTagName("ITEMNAME")[0].firstChild.data
                except AttributeError: setName = None 
               
                try: setCategory = blset.getElementsByTagName("CATEGORY")[0].firstChild.data
                except AttributeError: setCategory = None
                 
                try: setYear = blset.getElementsByTagName("ITEMYEAR")[0].firstChild.data
                except AttributeError: setYear = None 

                self.catalog.insertBLSet(setId,setName,setCategory,setYear)
                
                i+=1
                    
            self.status.pop(self.sbContext)
            self.status.push(self.sbContext,'Finished, sets in database: '+str(i))
        
            self.catalog.commit()
            
            blxml.close()

    
    def toolColorMapCB(self,widget):
        colorMaint = ColorMaintDialog.BrickColorMaint(title='Manage color mapping database',
                                                      parent=self.window,flags=gtk.DIALOG_MODAL,
                                                      catalog=self.catalog)
        colorMaint.run()
        colorMaint.hide()
    
    def toolPartMapCB(self,widget):
        partMaint = PartsMaintDialog.BrickPartsMaint(title='Manage BrickLink part mapping database',
                                                     parent=self.window,flags=gtk.DIALOG_MODAL,catalog=self.catalog,
                                                     imageCache=self.imageCache)
        partMaint.run()
        partMaint.hide()
        
    def toolLDrawMapCB(self,widget):
        ldrMaint = LDrawMaintDialog.LDrawPartsMaint(title='Manage LDraw part mapping database',
                                                     parent=self.window,flags=gtk.DIALOG_MODAL,catalog=self.catalog,
                                                     imageCache=self.imageCache)
        ldrMaint.run()
        ldrMaint.hide()
        
    
        
    def toolCompositeCB(self,widget):
        compositeDialog = LDDComposite(title='Manage LDD Composite part',parent=self.window,
                                       flags=gtk.DIALOG_MODAL,catalog=self.catalog)
        compositeDialog.run()
        compositeDialog.hide()
        
        
    def toolEmptyCacheCB(self,widget):
        msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                message_format="Empty cache for brick images (operation cannot be undone)?")
        msg.set_title('Delete brick shape images')
        delbrick = msg.run()
        msg.hide()
        msg.destroy()
        if delbrick != gtk.RESPONSE_YES:
            return
        else:
            filelist = os.listdir(self.imgCache)
            numfiles = len(filelist)
            if numfiles == 0:
                return
            totfiles = numfiles
            pg = ProgressDialog(title='Empty brick Image cache',
                                        parent=self.window,text=str(numfiles)+' files to go...')
            pg.show()
            for f in filelist:
                while gtk.events_pending():
                    gtk.main_iteration(block=False)
                try:
                    os.remove(os.path.join(self.imgCache,f))
                except (IOError,OSError):
                    pass
                v = pg.get_property("visible")
                if not v:
                    break
                numfiles -= 1
                pg.progressBar.set_text(str(numfiles)+' files to go...')
                pg.progressBar.set_fraction(1.0-(float(numfiles)/totfiles))
            pg.hide()
            pg.destroy()
                    
        
    def aboutButtonCB(self,widget):
        self.aboutDialog.run()
        self.aboutDialog.hide()

    def importLDDProjectCB(self,widget):
        importDialog = ImportLDDProject(title='Import LDD Project file',
                                        parent=self.window,flags=gtk.DIALOG_MODAL,
                                        catalog=self.catalog,filedlg=self.LDDfile)
        importDialog.run()
        importDialog.hide()
        self.currList.read()
        self.updateBrickList()        
    
    def importBLSetDumpCB(self,widget):
        self.blXMLfile.set_title("Select Bricklink Inventory XML file")
        importDialog = ImportBLSetDialog.ImportBLSet(title='Import Bricklink XML inventory',
                                                     parent=self.window,flags=gtk.DIALOG_MODAL,
                                                     catalog=self.catalog,filedlg=self.blXMLfile)
        importDialog.run()
        importDialog.hide()
        self.currList.read()
        self.updateBrickList()
        
    def importLDrawFileCB(self,widget):
        importDialog = ImportLDraw(parent=self.window,catalog=self.catalog,filedlg=self.LDrawfile)
        importDialog.run()
        importDialog.hide()
        self.currList.read()
        self.updateBrickList()        
        
    def exportBLMassUploadCB(self,widget):
        if len(self.currList) == 0:
            return
        dlg = BLExportXML(parent=self.window,flags=gtk.DIALOG_MODAL,currList=self.currList,catalog=self.catalog)
        dlg.run()
        dlg.hide()
        
    def exportHTMLCB(self,widget):
        if len(self.currList) == 0:
            return
        dlg = ExportHTML(parent=self.window,flags=gtk.DIALOG_MODAL,currList=self.currList,
                         catalog=self.catalog,imageCache=self.imageCache)
        dlg.run()
        dlg.hide()
        
#    def updLdrawXMLCB(self,widget):
#        dlg = UpdateLDrawDialog(parent=self.window,flags=gtk.DIALOG_MODAL,catalog=self.catalog)
#        dlg.run()
#        dlg.hide()
        
    def catalogExportCB(self,widget):
        dlg = gtk.FileChooserDialog(title='Choose Catalog export file',parent=self.window,action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT))
        dlg.set_do_overwrite_confirmation(True)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try:
                xfile = dlg.get_filename()
                x = open(xfile,'wt')
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot write on file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return                
            CatalogImportExport.exportBrickCatalog(self.catalog, x)
            x.close()
    
    def catalogImportCB(self,widget):
        self.blXMLfile.set_title("Select Catalog exported XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                xfile = self.blXMLfile.get_filename()
                x = open(xfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            ci = CatalogImportExport.importBrickCatalog(x,self.catalog)
            ci.run()
            x.close()
            self.catList.read()
            self.updateCatalogInfo()
            
    
    def updLDrawLibCB(self,widget):
        response = self.LDrlibfile.run()
        self.LDrlibfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try:
                r = LDrawImportLib(self.window,self.LDrlibfile.get_filename(),self.catalog)
                if not r:
                    raise IOError
            except (IOError,zipfile.BadZipfile):
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file, or file isn't a LDraw part Library.")
                msg.run()
                msg.hide()
                msg.destroy()
                
                
      
    def setExportCB(self,widget):
        dlg = gtk.FileChooserDialog(title='Choose Sets export file',parent=self.window,action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_SAVE,gtk.RESPONSE_ACCEPT))
        dlg.set_do_overwrite_confirmation(True)
        response = dlg.run()
        dlg.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try:
                xfile = dlg.get_filename()
                x = open(xfile,'wt')
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot write on file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            CatalogImportExport.exportSetCatalog(self.catalog, x)
            x.close()

    
    def setImportCB(self,widget):
        self.blXMLfile.set_title("Select Sets exported XML file")
        response = self.blXMLfile.run()
        self.blXMLfile.hide()
        if response == gtk.RESPONSE_ACCEPT:
            try :
                xfile = self.blXMLfile.get_filename()
                x = open(xfile)
            except IOError:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Error: cannot read file.")
                msg.run()
                msg.hide()
                msg.destroy()
                return
            si = CatalogImportExport.importSetCatalog(x,self.catalog)
            si.run()
            x.close()
            self.setList.read()
            self.updateCatalogInfo()
            
            
    def nbPageSelectCB(self,widget, page, page_num):
        if page_num == 0:
            #------------------------------------- working brick list is selected
            self.menuBar.remove(self.setImportMenuItem)
            self.menuBar.remove(self.setEditMenuItem)
            self.menuBar.remove(self.catEditMenuItem)
            self.menuBar.remove(self.catImportMenuItem)
            self.menuBar.insert(self.advancedMenuItem,0)
            self.menuBar.insert(self.catalogMenuItem,0)
            self.menuBar.insert(self.importMenuItem,0)
            self.menuBar.insert(self.editMenuItem,0)
        elif page_num == 1:
            self.menuBar.remove(self.setImportMenuItem)
            self.menuBar.remove(self.setEditMenuItem)
            self.menuBar.remove(self.importMenuItem)
            self.menuBar.remove(self.editMenuItem)
            self.menuBar.remove(self.advancedMenuItem)
            self.menuBar.remove(self.catalogMenuItem)
            self.menuBar.insert(self.catImportMenuItem,0)
            self.menuBar.insert(self.catEditMenuItem,0)
        elif page_num == 2:
            self.menuBar.remove(self.importMenuItem)
            self.menuBar.remove(self.editMenuItem)
            self.menuBar.remove(self.advancedMenuItem)
            self.menuBar.remove(self.catalogMenuItem)
            self.menuBar.remove(self.catImportMenuItem)
            self.menuBar.remove(self.catEditMenuItem)   
            self.menuBar.insert(self.setImportMenuItem,0)
            self.menuBar.insert(self.setEditMenuItem,0)
        self.menuBar.show_all()


            
            

    def __init__(self):
        #-------------------- find catalog position
        if platform.system() == "Windows":
            # it is a "portable" installation
            self.catalogFile = os.path.join(os.getcwd(),'catalog.sq3')
            self.newCatalog = os.path.join(os.getcwd(),'catalog.new')
        else:   # Linux/Mac
            # "portable" mode
            self.catalogFile = 'catalog.sq3'
            self.newCatalog = 'catalog.new'
        if not os.access(self.catalogFile, os.F_OK):
            if os.access(self.newCatalog,os.F_OK):
                #------------------------ this is a new installation, no upgrade
                os.rename(self.newCatalog,self.catalogFile)
            else:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Program can't find main database.\nPlease check if a file named 'catalog.sq3' or "+
                                    "a file named 'catalog.new' is located in program folder.\nIf not, you have a corrupted copy of program, please get a new one.")
                msg.set_title('Database not found')
                msg.run()
                msg.hide()
                msg.destroy()
                exit(1)
                

                    
        #-------------------------------------- checks for a database web update
        if not os.access(self.newCatalog, os.F_OK):
            #------------ there isn't a local update update file, do web check
            dbu = DBUpdate(self.newCatalog,"ilpettegolo.altervista.org",CatalogDB.getDBVersion(self.catalogFile))
            ver = dbu.CheckDB()
            if ver > 10:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                        message_format="A new brick catalog database is available, but your program is too old to use it, so upgrade it first.")
                msg.set_title('Update available')
                upd = msg.run()
                msg.hide()
                msg.destroy()
            elif ver > 0:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,buttons=gtk.BUTTONS_YES_NO,flags=gtk.DIALOG_MODAL,
                                        message_format="A new brick catalog database is available: do you want to update now?")
                msg.set_title('Update confirmation')
                upd = msg.run()
                msg.hide()
                msg.destroy()
                if upd == gtk.RESPONSE_YES:
                    if not dbu.GetDB():
                        msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                            message_format="Upgrade file corrupted or failed download, update cancelled")
                        msg.set_title('Error in upgrade')
                        msg.run()
                        msg.hide()
                        msg.destroy()
                        
            
        #----------------------------- checks if a database upgrade is available
        if os.access(self.newCatalog, os.F_OK):
            versionNew = CatalogDB.getDBVersion(self.newCatalog)
            versionCurr = CatalogDB.getDBVersion(self.catalogFile)
            if versionNew > versionCurr:
                #------------------------------- upgrade is available and needed
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Catalog database needs upgrade. This version: "+
                                    str(versionCurr)+". New version: "+str(versionNew))
                msg.set_title('Proceed to upgrade')
                msg.run()
                msg.hide()
                msg.destroy()
                UpgradeDatabases.UpgradeDatabases(self.catalogFile,self.newCatalog)
                msg = gtk.MessageDialog(type=gtk.MESSAGE_INFO,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="Catalog database updated")
                msg.set_title('Upgrade OK')
                msg.run()
                msg.hide()
                msg.destroy()
            else:
                msg = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
                                    message_format="A catalog update file is found, but database version is identical, so file will be ignored and deleted")
                msg.set_title('No upgrade needed')
                msg.run()
                msg.hide()
                msg.destroy()                
                os.unlink(self.newCatalog)
        
        #------------------------------------------------- image cache directory
        self.imgCache = os.path.join(os.path.dirname(self.catalogFile),'imgcache')

        self.catalog = CatalogDB.CatalogDB(self.catalogFile)
        self.catalog.checkTables()
        self.catalog.checkDBversion()
        
        if not os.path.isdir(self.imgCache):
            try:
                os.mkdir(self.imgCache)
            except (IOError,OSError):
                self.imgCache = None
        if self.imgCache != None:
            self.imageCache = BrickImage(self.imgCache,self.catalog)

            
            
        self.currList = BrickListStore.BrickListStore(self.catalog)
        self.currList.read()
        
        #-------------------------------------------------------- interface building
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(progInfo.Name)
        self.window.set_default_size(min(gtk.gdk.screen_width()-200,800),
                                     min(gtk.gdk.screen_height()-200,600))

        # It's a good idea to do this for all windows.
        self.window.connect("destroy", lambda wid: gtk.main_quit())
        self.window.connect("delete_event", lambda a1,a2:gtk.main_quit())

        #---------------------------------------------------------- file dialogs
        
        #---------------------------------------------- filter for various files
        
        #------------------------------------ BrickLink/XML files (*.xml, *.txt)
        xmlFilter = gtk.FileFilter()
        xmlFilter.add_pattern("*.xml")
        xmlFilter.add_pattern('*.txt')
        xmlFilter.set_name("XML file")

        #----------------------------------------------- LDD file filter (*.lxf)
        LDDFilter = gtk.FileFilter()
        LDDFilter.add_pattern('*.lxf')
        LDDFilter.add_pattern('*.lxfml')
        LDDFilter.add_pattern('*.xml')
        LDDFilter.set_name("LDD files")
        
        #------------------------------- LDraw file filter (*.dat, *.mpd, *.ldr)
        ldrawFilter = gtk.FileFilter()
        ldrawFilter.add_pattern('*.ldr')
        ldrawFilter.add_pattern('*.dat')
        ldrawFilter.add_pattern('*.mpd')
        ldrawFilter.set_name("LDraw files")
                
        #----------------------------------------- Ldraw lib file filter (*.zip)
        ldrlibFilter = gtk.FileFilter()
        ldrlibFilter.add_pattern('*.zip')
        ldrlibFilter.set_name("LDraw library files")

        #-------------------------------------- add a filter for any (all) files
        allFilter = gtk.FileFilter()
        allFilter.add_pattern("*")
        allFilter.set_name("All files")

        #--------------------------------------------------- BrickLink/xml files
        self.blXMLfile = gtk.FileChooserDialog("Select Bricklink XML file",self.window,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        self.blXMLfile.add_filter(xmlFilter)
        self.blXMLfile.add_filter(allFilter)
        
        #----------------------------------------------------- LDD project files
        self.LDDfile = gtk.FileChooserDialog("Select LDD project or export file",self.window,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        self.LDDfile.add_filter(LDDFilter)
        self.LDDfile.add_filter(allFilter)
        
        #--------------------------------------------------- LDraw project files
        self.LDrawfile = gtk.FileChooserDialog("Select LDraw project file",self.window,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        self.LDrawfile.add_filter(ldrawFilter)
        self.LDrawfile.add_filter(allFilter)

        #----------------------------------------------- LDraw part library file
        self.LDrlibfile = gtk.FileChooserDialog("Select LDraw library file",self.window,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT,gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        self.LDrlibfile.add_filter(ldrlibFilter)
        self.LDrlibfile.add_filter(allFilter)
        
        #----------------------------------------------------------- create menu
        #------------------------------------------------------------ tools menu
        self.toolMenu = gtk.Menu()

        self.toolColorMap = gtk.MenuItem('Manage _color mapping...')
        self.toolPartMap = gtk.MenuItem('Manage BrickLink _part mapping...')
        self.toolLDrawMap = gtk.MenuItem('Manage _Ldraw part mapping...')
        self.toolComposite = gtk.MenuItem('Manage LDD composite part...')
        self.toolEmptyCache = gtk.MenuItem('Empty brick images cache...')
        self.toolMenu.append(self.toolColorMap)
        self.toolMenu.append(self.toolPartMap)
        self.toolMenu.append(self.toolLDrawMap)
        self.toolMenu.append(self.toolComposite)
        self.toolMenu.append(gtk.SeparatorMenuItem())
        self.toolMenu.append(self.toolEmptyCache)
        self.toolColorMap.connect('activate',self.toolColorMapCB)
        self.toolPartMap.connect('activate',self.toolPartMapCB)
        self.toolLDrawMap.connect('activate',self.toolLDrawMapCB)
        self.toolComposite.connect('activate',self.toolCompositeCB)
        self.toolEmptyCache.connect('activate',self.toolEmptyCacheCB) 
        
        #----------------------------------------------------- about & quit menu
        self.programMenu = gtk.Menu()
        
        self.aboutItem = gtk.MenuItem('About...')
        self.programMenu.append(self.aboutItem)
        self.aboutItem.connect('activate',self.aboutButtonCB)
        self.programMenu.append(gtk.SeparatorMenuItem())
        self.quitItem = gtk.MenuItem('Exit Program')
        self.programMenu.append(self.quitItem)
        self.quitItem.connect('activate',self.quitCB)
        
        #----------------------------------------------------------- update menu
        self.updateMenu = gtk.Menu()
        self.updBLParts = gtk.MenuItem('Update BL _parts...')
        self.updBLSets = gtk.MenuItem('Update BL _sets...')
        self.updBLColors = gtk.MenuItem('Update BL _colors...')
        self.updBLCategories = gtk.MenuItem('Update BL cate_gories...')
        self.updBLCodes = gtk.MenuItem('Update BL part/color co_des...')
        self.updLDrawLib = gtk.MenuItem('Update LDraw library...')
        self.updateMenu.append(self.updBLParts)
        self.updateMenu.append(self.updBLSets)
        self.updateMenu.append(self.updBLColors)
        self.updateMenu.append(self.updBLCategories)
        self.updateMenu.append(self.updBLCodes)
        self.updateMenu.append(gtk.SeparatorMenuItem())
        self.updateMenu.append(self.updLDrawLib)
        self.updBLParts.connect('activate',self.updBLPartsCB)
        self.updBLSets.connect('activate',self.updBLSetsCB)
        self.updBLColors.connect('activate',self.updBLColorsCB)
        self.updBLCategories.connect('activate',self.updBLCategoriesCB)
        self.updBLCodes.connect('activate',self.updBLCodesCB)
        self.updLDrawLib.connect('activate',self.updLDrawLibCB)
        
        
        #----------------------------------------------------------- import menu
        self.importMenu = gtk.Menu()
        self.importLDDFile = gtk.MenuItem('Import LDD Project...')
        self.importLDDFile.connect('activate',self.importLDDProjectCB)
        self.importBLSet = gtk.MenuItem('Import BL Inventory...')
        self.importBLSet.connect('activate',self.importBLSetDumpCB)
        self.importLDrawFile = gtk.MenuItem('Import LDraw file...')
        self.importLDrawFile.connect('activate',self.importLDrawFileCB)
        self.exportBLXML = gtk.MenuItem('Export as BL mass upload...')
        self.exportBLXML.connect('activate',self.exportBLMassUploadCB)
        self.exportHTML = gtk.MenuItem('Export as printable HTML...')
        self.exportHTML.connect('activate',self.exportHTMLCB)
        self.importMenu.append(self.importLDDFile)
        self.importMenu.append(self.importBLSet)
        self.importMenu.append(self.importLDrawFile)
        self.importMenu.append(self.exportBLXML)
        self.importMenu.append(self.exportHTML)
        
        self.importMenuItem = gtk.MenuItem('_Import/Export')
        self.importMenuItem.set_submenu(self.importMenu)
        
        #-------------------------------------------------------- list edit menu
        
        self.editMenu = gtk.Menu()
        self.blAddPart = gtk.MenuItem('Add part...')
        self.blAddPart.connect('activate',self.blAddPartCB)
        self.blEdit = gtk.MenuItem('Edit part...')
        self.blEdit.connect('activate',self.blEditPartCB)
        self.blDelete = gtk.MenuItem('Delete part')
        self.blDelete.connect('activate',self.blDeleteButtonCB)
        self.blRemoveCounterpart = gtk.MenuItem('Remove Counterparts')
        self.blRemoveCounterpart.connect('activate',self.blRemoveCounterpartsCB)
        self.blRemoveExtra = gtk.MenuItem('Remove Extra parts')
        self.blRemoveExtra.connect('activate',self.blRemoveExtraCB)
        self.blEmptyList = gtk.MenuItem('Empty part list')
        self.blEmptyList.connect('activate',self.blEmptyListCB)
        self.editMenu.append(self.blAddPart)
        self.editMenu.append(self.blEdit)
        self.editMenu.append(self.blDelete)
        self.editMenu.append(self.blRemoveCounterpart)
        self.editMenu.append(self.blRemoveExtra)
        self.editMenu.append(self.blEmptyList)  
        
        self.editMenuItem = gtk.MenuItem('_Edit')
        self.editMenuItem.set_submenu(self.editMenu)
        
        #------------------------------------------------ main catalog interface
        self.vBox = gtk.VBox()
        
        self.menuBar = gtk.MenuBar()

        self.toolItem = gtk.MenuItem('_Tools')
        self.toolItem.set_submenu(self.toolMenu)
        self.menuBar.append(self.toolItem)

        self.updateItem = gtk.MenuItem('_Update')
        self.updateItem.set_submenu(self.updateMenu)
        self.menuBar.append(self.updateItem)

        self.aboutMenuItem = gtk.MenuItem('_Program')
        self.aboutMenuItem.set_right_justified(True)
        self.aboutMenuItem.set_submenu(self.programMenu)
        self.menuBar.append(self.aboutMenuItem)

        self.vBox.pack_start(self.menuBar,False)
        
        #-------------------------------------------- main application work area
        #------------------------------------------------ current brick list tab
        self.brickImage = gtk.Image()
        self.partListView = BrickListView.BrickListView(self.currList,self.imageCache,self.brickImage)
        self.partListView.connect('row-activated',self.blEditPartCB)
        sel = self.partListView.get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)
        self.listWindow = gtk.ScrolledWindow()
        self.listWindow.add(self.partListView)
        self.blVBox = gtk.VBox()
        self.blVBox.pack_start(self.listWindow)
        self.bltable = gtk.Table(5,5)
        self.bltable.set_border_width(3)

        #----------------------------------------------- catalog and advanced
        
        self.catalogMenu = gtk.Menu()
        self.blAdd2Catalog = gtk.MenuItem('Add to Catalog...')
        self.blAdd2Catalog.connect('activate',self.blAdd2CatalogCB)
        self.blAdd2CatalogSet = gtk.MenuItem('Add to Catalog as a Set...')
        self.blAdd2CatalogSet.connect('activate',self.blAdd2CatalogSetCB)
        self.catalogMenu.append(self.blAdd2Catalog)
        self.catalogMenu.append(self.blAdd2CatalogSet)    
        
        self.catalogMenuItem = gtk.MenuItem('_Catalog')
        self.catalogMenuItem.set_submenu(self.catalogMenu)    
        
        #---------------------------------------------------- advanced functions
        self.advancedMenuItem = gtk.MenuItem('A_dvanced')
        self.advancedMenu = gtk.Menu()

        self.blCheckBuildable = gtk.MenuItem('Can build?')
        self.blCheckBuildable.connect('activate',self.blCheckBuildableCB)
        self.blSanityChecks = gtk.MenuItem('Checks for errors...')
        self.blSanityChecks.connect('activate',self.blSanityChecksCB)
        self.advancedMenu.append(self.blCheckBuildable)
        self.advancedMenu.append(self.blSanityChecks)
        self.advancedMenuItem.set_submenu(self.advancedMenu)
        
        #---------------------------------------------------------- list info
        frame = gtk.Frame('List info')
        table = gtk.Table(2,2)
        table.set_border_width(3)
        frame.set_border_width(3)
        frame.add(table)
        lab = gtk.Label('Part type count:')
        lab.set_alignment(1,0.5)
        self.blCountLab = gtk.Entry()
        self.blCountLab.set_editable(False)
        table.attach(lab,0,1,0,1,xoptions=gtk.FILL)
        table.attach(self.blCountLab,1,2,0,1)
        self.blCountParts = gtk.Entry()
        self.blCountParts.set_editable(False)
        lab = gtk.Label('Total part count:')
        lab.set_alignment(1,0.5)
        table.attach(lab,0,1,1,2,xoptions=gtk.FILL)
        table.attach(self.blCountParts,1,2,1,2)
        
        self.bltable.attach(frame,1,3,0,1,xoptions=gtk.SHRINK)
        self.bltable.attach(gtk.Label(' '),3,4,1,2)

        frame = gtk.Frame('Brick shape')
        frame.set_border_width(3)
        self.brickImage.set_size_request(200,200)
        frame.add(self.brickImage)
        self.bltable.attach(frame,4,5,0,2,xoptions=gtk.SHRINK,yoptions=gtk.SHRINK)
        
        self.blVBox.pack_end(self.bltable,False)

        #------------------------------------------------------ main catalog tab
        self.catalogImage = gtk.Image()
        self.catList = CatalogStore.CatalogStore(self.catalog)
        self.catView = CatalogView.CatalogView(self.catList,self.imageCache,self.catalogImage)
        sel = self.catView.get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)

        self.catView.connect('row-activated',self.catEditCB)
        self.catWindow = gtk.ScrolledWindow()
        self.catWindow.add(self.catView)
        self.catVBox = gtk.VBox()
        self.catVBox.pack_start(self.catWindow)
        self.ctable = gtk.Table(3,3)
        self.ctable.set_border_width(3)
        
        self.catEdit = gtk.MenuItem('Edit brick...')
        self.catDelete = gtk.MenuItem('Delete brick...')
        self.catDelZero = gtk.MenuItem('Remove empty bricks')
        self.catAddBrick = gtk.MenuItem('Add brick...')
        self.catEdit.connect('activate',self.catEditCB)
        self.catDelete.connect('activate',self.catDeleteCB)
        self.catDelZero.connect('activate',self.catDelZeroCB)
        self.catAddBrick.connect('activate',self.catAddBrickCB)
        self.catEditMenuItem = gtk.MenuItem('_Edit')
        self.catEditMenu = gtk.Menu()
        self.catEditMenu.append(self.catAddBrick)
        self.catEditMenu.append(self.catEdit)
        self.catEditMenu.append(self.catDelete)
        self.catEditMenu.append(self.catDelZero)
        self.catEditMenuItem.set_submenu(self.catEditMenu)

        self.catImportMenuItem = gtk.MenuItem('_Import/Export')
        self.catImportMenu = gtk.Menu()
        self.catalogExport = gtk.MenuItem('Backup brick catalog...')
        self.catalogImport = gtk.MenuItem('Restore brick catalog...')
        self.catalogExport.connect('activate',self.catalogExportCB)
        self.catalogImport.connect('activate',self.catalogImportCB)
        self.catImportMenu.append(self.catalogExport)
        self.catImportMenu.append(self.catalogImport)
        self.catImportMenuItem.set_submenu(self.catImportMenu)
        
        #---------------------------------------------------------- catalog info
        frame = gtk.Frame('Catalog info')
        table = gtk.Table(2,2)
        table.set_border_width(3)
        frame.set_border_width(3)
        frame.add(table)
        lab = gtk.Label('Part type count:')
        lab.set_alignment(1,0.5)
        self.catCount = gtk.Entry()
        self.catCount.set_editable(False)
        table.attach(lab,0,1,0,1,xoptions=gtk.FILL)
        table.attach(self.catCount,1,2,0,1)
        self.catCountParts = gtk.Entry()
        self.catCountParts.set_editable(False)
        lab = gtk.Label('Total part count:')
        lab.set_alignment(1,0.5)
        table.attach(lab,0,1,1,2,xoptions=gtk.FILL)
        table.attach(self.catCountParts,1,2,1,2)        
        
        self.ctable.attach(frame,2,3,0,2,xoptions=gtk.SHRINK)
        
        label = gtk.Label(' ')
        self.ctable.attach(label,3,4,0,3)
        
        frame = gtk.Frame('Brick shape')
        frame.set_border_width(3)
        self.catalogImage.set_size_request(200,200)
        frame.add(self.catalogImage)
        self.ctable.attach(frame,4,5,0,3,xoptions=gtk.SHRINK,yoptions=gtk.SHRINK)        
        
        self.catVBox.pack_end(self.ctable,False)

        #------------------------------------------------------ set catalog view
        self.setList = SetStore.SetStore(self.catalog)
        self.setView = SetView.SetView(self.setList)
        sel = self.setView.get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)
        self.setsWindow = gtk.ScrolledWindow()
        self.setsWindow.add(self.setView)
        self.setsVBox = gtk.VBox()
        self.setsVBox.pack_start(self.setsWindow)
        self.stable = gtk.Table(2,3)

        self.setEditMenu = gtk.Menu()
        self.setEditMenuItem = gtk.MenuItem('Edit')
        
        self.setDelete = gtk.MenuItem('Delete Set')
        self.setDelete.connect('activate',self.setDeleteCB)
        self.setEditMenu.append(self.setDelete)
        self.setEditMenuItem.set_submenu(self.setEditMenu)
        
        self.setImportMenu = gtk.Menu()
        self.setImportMenuItem = gtk.MenuItem('Import/Export')
        
        self.setExport = gtk.MenuItem('Backup set catalog...')
        self.setImport = gtk.MenuItem('Restore set catalog...')
        self.setExport.connect('activate',self.setExportCB)
        self.setImport.connect('activate',self.setImportCB)
        self.setImportMenu.append(self.setExport)
        self.setImportMenu.append(self.setImport)
        self.setImportMenuItem.set_submenu(self.setImportMenu)
        
        self.setsVBox.pack_end(self.stable,False)
        
        #-------------------------------------------------------- tab view setup
        self.noteBook = gtk.Notebook()
        self.blVBox.set_border_width(4)
        self.catVBox.set_border_width(4)
        self.setsVBox.set_border_width(4)
        self.noteBook.append_page(self.blVBox,gtk.Label('Current bricks List'))
        self.noteBook.append_page(self.catVBox,gtk.Label('My bricks catalog'))
        self.noteBook.append_page(self.setsVBox,gtk.Label('My sets catalog'))
        self.noteBook.connect('switch-page',self.nbPageSelectCB)
        
        self.vBox.pack_start(self.noteBook)
        
        self.status = gtk.Statusbar()
        self.vBox.pack_end(self.status,False)
        self.sbContext = self.status.get_context_id('status')
        
        self.updateBrickList()
        self.updateCatalogInfo()

        #---------------------------------------------------------- about dialog
        try:
            self.logo = gtk.gdk.pixbuf_new_from_file(progInfo.Logo)
        except glib.GError:
            try:
                self.logo = gtk.gdk.pixbuf_new_from_file('/usr/local'+progInfo.Name+'/'+progInfo.Logo)
            except glib.GError:
                self.logo = None
        self.aboutDialog = gtk.AboutDialog()
        self.aboutDialog.set_program_name(progInfo.Name)
        self.aboutDialog.set_version(progInfo.Version)
        self.aboutDialog.set_copyright(progInfo.Copyright)
        self.aboutDialog.set_authors([progInfo.Author,])
        self.aboutDialog.set_website(progInfo.WebURL)
        self.aboutDialog.set_website_label(progInfo.WebName)
        self.aboutDialog.set_comments('Database version: '+self.catalog.getProperty('dbver')+'\nThis program exists thanks to the great work of BrickLink, LDraw and LEGO(R) people.')
        self.aboutDialog.set_logo(self.logo)
        self.aboutDialog.set_license(progInfo.Name + 
''' is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

'''+progInfo.Name+''' is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with '''+progInfo.Name+'''; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA'''
                                )
        self.aboutDialog.set_wrap_license(True)                                     
                
        self.window.add(self.vBox)
        self.window.show_all()
        

def main():
    gtk.main()
    return 0     


if __name__ == "__main__":
    warnings.simplefilter('ignore')
    vercheck = gtk.check_version(2,6,0)
    if (vercheck != None):
        print vercheck
        print "BrickUtils require pyGTK version 2.6 or above"
        exit()
    BrickUtilsApp()
    main()
#    except Exception as e:
#        msg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,buttons=gtk.BUTTONS_OK,flags=gtk.DIALOG_MODAL,
#                            message_format="Unhandled exception:"+repr(sys.exc_info()))
#        msg.run()
#        msg.hide()
#        msg.destroy()

    
